new;
library frm,pgraph,optmum;

rndseed 123456;

params =  0.055 |  0.005 | -0.045 | 1.8;
t = seqa(0,1/12,20*12+1);
t[1] = 1e-5;
Rt = NelsonSiegel_Spot_Rate(params,t);
R_gVasicek = Rt;

use_model = 1;
Vasicek_Parameters = zeros(4,2);

proc (4) = Vasicek_Parameters_Model1(theta);
  local r0,a,b,sigma;
  theta = sqrt(theta^2);
  r0 = theta[1];
  a = theta[2];
  b = theta[3];
  sigma = theta[4];
  retp(r0,a,b,sigma);
endp;

proc (4) = Vasicek_Parameters_Model2(theta);
  local r0,a,b,sigma;
  theta = sqrt(theta^2);
  r0 = Rt[1];
  a = theta[1];
  b = theta[2];
  sigma = theta[3];
  retp(r0,a,b,sigma);
endp;

proc rss(theta);
  local r0,a,b,sigma,ZC_Vasicek,R_Vasicek,u;
  if use_model == 1;
    {r0,a,b,sigma} = Vasicek_Parameters_Model1(theta);
  else;
    {r0,a,b,sigma} = Vasicek_Parameters_Model2(theta);
  endif;
  {ZC_Vasicek,R_Vasicek} = Vasicek(r0,a,b,sigma,0,t);
  u = Rt - R_Vasicek;
  retp(u'u);
endp;

use_model = 1;
sv = Rt[1]|1|Rt[201]|0.5;
{theta,fmin,grd,retcode} = optmum(&rss,sv);
{r0,a,b,sigma} = Vasicek_Parameters_Model1(theta);
{ZC_Vasicek,R_Vasicek} = Vasicek(r0,a,b,sigma,0,t);
R_Vasicek_Model1 = R_Vasicek;
Vasicek_Parameters[.,1] = r0|a|b|sigma;

use_model = 2;
sv = 1|Rt[201]|0.5;
{theta,fmin,grd,retcode} = optmum(&rss,sv);
{r0,a,b,sigma} = Vasicek_Parameters_Model2(theta);
{ZC_Vasicek,R_Vasicek} = Vasicek(r0,a,b,sigma,0,t);
R_Vasicek_Model2 = R_Vasicek;
Vasicek_Parameters[.,2] = r0|a|b|sigma;

R_Vasicek = R_Vasicek_Model1;

Ft = NelsonSiegel_Forward_Rate(params,t,0);
bt = gVasicek_NelsonSiegel(params,0.5~1.0~1.5,0.01~0.05~0.10,t);

cls;
print ftosa(Vasicek_Parameters,"%lf",3,4);

graphset;
  begwind;
  window(2,2,0);

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|3|1; _pcolor = 1|4|2; _plwidth = 10;
  _paxht = 0.225; _pnumht = 0.225; _ptitlht = 0.275;

  xtics(0,20,5,2);
  xlabel("\216Maturity (in years)");
  ytics(4.0,6.0,0.5,2);

  setwind(1);
    title("\216Nelson-Siegel yield curve (in %)");
    xy(t,100*Rt);

  setwind(2);
    title("\216Instantaneous forward rate (in %)");
    xy(t,100*Ft);

  setwind(4);
    title("\216Calibrated Vasicek yield curve (in %)");
    _plegstr = "#1 (extended Vasicek)\000#2 (Vasicek)";
    _plegctl = {2 5 3 4.0};
    xy(t,100*(R_gVasicek~R_Vasicek));

  ytics(2.0,6.0,1.0,2);

  setwind(3);
    title("\216Mean-reversion parameter b(t) (in %)");
    _plegstr = "a=0.5, \202s\201=1%\000a=1.0, \202s\201=5%\000a=1.5, \202s\201=10%";
    _plegctl = {2 5 4.0 2.5};
    xy(t,100*bt);

  endwind;