new;
library frm,pgraph;
frmset;

cls;

declare matrix option_model,option_K,option_sigma,option_gamma,mesh_h,mesh_k;

option_K = 100;
sigma_mean = 0.20;
sigma_min = 0.15;
sigma_max = 0.25;
tau = 1.00;
r = 0.05;
b = r;

proc aProc(t,x);
  local cnd,local_sigma;

  if option_model == 2;                  // V+
    cnd = option_gamma .>= 0;
    local_sigma = sigma_max .* cnd + sigma_min .* (1-cnd); 
    local_sigma = trimr(local_sigma,1,1);
  elseif option_model == 3;              // V-
    cnd = option_gamma .>= 0;
    local_sigma = sigma_min .* cnd + sigma_max .* (1-cnd); 
    local_sigma = trimr(local_sigma,1,1);
  else;
    local_sigma = option_sigma;
  endif;

  retp( 0.5*(local_sigma^2) .* (x^2) );
endp;

proc bProc(t,x);
  retp( b .* x );
endp;

proc cProc(t,x);
  retp( r .* ones(rows(x),1) );
endp;

proc dProc(t,x);
  retp( zeros(rows(x),1) );
endp;

proc eProc(t,x,U);

  if (option_model == 2) or (option_model == 3);
    option_gamma = ((lagn(U,-1) - 2 * U + lagn(U,1))) ./ (mesh_h^2);
    option_gamma[1] = option_gamma[2];
    option_gamma[rows(U)] = option_gamma[rows(U)-1];
  endif;

  retp(U);
endp;

proc tminBound(t,x);
  local U;
  U = x - option_K;
  U = U .* (U .> 0);

  if (option_model == 2) or (option_model == 3);
    option_gamma = ((lagn(U,-1) - 2 * U + lagn(U,1))) ./ (mesh_h^2);
    option_gamma[1] = option_gamma[2];
    option_gamma[rows(U)] = option_gamma[rows(U)-1];
  endif;

  retp(U);
endp;

proc xMinBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc DxMaxBound(t,x);
  retp( ones(rows(x),1) );
endp;

xMin = 50; xMax = 150; nX = 251;
tMin = 0; tMax = tau; nT = 1001;
mesh_k = (tMax - tMin)/(nT-1); 
mesh_h = (xMax - xMin)/(nX-1);  

_PDE_SaveLastIter = 0;
_PDE_PrintIters = 50;

call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
         &tminBound,&xMinBound,0,0,&DxMaxBound);

theta = 1;

V = zeros(nX,5);

iter = 1;
do until iter > 5;
  if iter == 1;
    option_model = 1;
    option_sigma = sigma_mean;
  elseif iter == 2;
    option_model = 1;
    option_sigma = sigma_max;
  elseif iter == 3;
    option_model = 1;
    option_sigma = sigma_min;
  elseif iter == 4;
    option_model = 2;
  elseif iter == 5;
    option_model = 3;
  endif;  
  
  call PDE_solve(tMin,tMax,nT,xMin,xMax,nX,"pde",theta);
  t = PDE_readFile("pde","t");
  x = PDE_readFile("pde","x");
  V[.,iter] = PDE_readFile("pde","t"|t[nT]);

  iter = iter + 1;
endo;

sigma = sigma_mean~sigma_max~sigma_min;
C = EuropeanBS(x,option_K,sigma,tau,b,r);

pqgwin many;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3|2|2|2; _plwidth = 10; _pcolor = 1|4|2|3|5|6;
  xy(x,V[.,1:3]~C);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3|6|2; _plwidth = 10; _pcolor = 1|4|2|3|5;
  xy(x,V[.,1 4 5]);
