new;
library frm,pgraph;
frmset;

cls;

declare matrix option_model,option_K,option_L,option_H,option_sigma,option_gamma,mesh_h,mesh_k;
declare matrix option_gamma_matrix,option_gamma_iter;

option_K = 100;
option_L = 80;
option_H = 120;
sigma_mean = 0.20;
sigma_min = 0.15;
sigma_max = 0.25;
tau = 1.00;
r = 0.05;
b = r;

proc aProc(t,x);
  local cnd,local_sigma;

  if option_model == 1;
    local_sigma = option_sigma;
  elseif option_model == 2;                  // V+
    cnd = option_gamma .>= 0;
    local_sigma = sigma_max .* cnd + sigma_min .* (1-cnd); 
    local_sigma = trimr(local_sigma,1,1);
  elseif option_model == 3;                 // V-
    cnd = option_gamma .>= 0;
    local_sigma = sigma_min .* cnd + sigma_max .* (1-cnd); 
    local_sigma = trimr(local_sigma,1,1);
  elseif option_model == 4;
    local_sigma = option_sigma;
  elseif option_model == 5;                  // V+
    cnd = option_gamma .>= 0;
    local_sigma = sigma_max .* cnd + sigma_min .* (1-cnd); 
    local_sigma = trimr(local_sigma,1,1);
  elseif option_model == 6;                  // V-
    cnd = option_gamma .>= 0;
    local_sigma = sigma_min .* cnd + sigma_max .* (1-cnd); 
    local_sigma = trimr(local_sigma,1,1);
  endif;

  retp( 0.5*(local_sigma^2) .* (x^2) );
endp;

proc bProc(t,x);
  retp( b .* x );
endp;

proc cProc(t,x);
  retp( r .* ones(rows(x),1) );
endp;

proc dProc(t,x);
  retp( zeros(rows(x),1) );
endp;

proc eProc(t,x,U);
  local cnd;

  if (option_model == 1) or (option_model == 2) or (option_model == 3);
    cnd = (x .>= option_L) .and (x .<= option_H);
  elseif (option_model == 4) or (option_model == 5) or (option_model == 6);
    if (t > 0.25) .and (t < 0.5);
      cnd = (x .>= option_L) .and (x .<= option_H);
    else;
      cnd = 1.0;
    endif;
  endif;

  if (option_model == 2) or (option_model == 3) or (option_model == 5) or (option_model == 6);
    option_gamma = ((lagn(U,-1) - 2 * U + lagn(U,1))) ./ (mesh_h^2);
    option_gamma[1] = option_gamma[2];
    option_gamma[rows(U)] = option_gamma[rows(U)-1];

    option_gamma_matrix[.,option_gamma_iter] = option_gamma;
    option_gamma_iter = option_gamma_iter + 1;

  endif;

  U = cnd .* U;

  retp(U);
endp;

proc tminBound(t,x);
  local U,cnd;
  U = x - option_K;
  U = U .* (U .> 0);

  if (option_model == 1) or (option_model == 2) or (option_model == 3);
    cnd = (x .>= option_L) .and (x .<= option_H);
  elseif (option_model == 4) or (option_model == 5) or (option_model == 6);
    if (t > 0.25) .and (t < 0.75);
      cnd = (x .>= option_L) .and (x .<= option_H);
    else;
      cnd = 1.0;
    endif;
  endif;

  option_gamma_iter = 1;
  if (option_model == 2) or (option_model == 3);
    option_gamma = ((lagn(U,-1) - 2 * U + lagn(U,1))) ./ (mesh_h^2);
    option_gamma[1] = option_gamma[2];
    option_gamma[rows(U)] = option_gamma[rows(U)-1];
    option_gamma_matrix[.,option_gamma_iter] = option_gamma;
  endif;

  U = cnd .* U;

  retp(U);
endp;

proc xMinBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc xMaxBound(t,x);
  retp( zeros(rows(x),1) );
endp;

xMin = 20; xMax = 180; nX = 501;
tMin = 0; tMax = tau; nT = 1001;
mesh_k = (tMax - tMin)/(nT-1); 
mesh_h = (xMax - xMin)/(nX-1);  

_PDE_SaveLastIter = 0;
_PDE_PrintIters = 50;

call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
         &tminBound,&xMinBound,&xMaxBound,0,0);

theta = 0.5;

option_gamma_matrix = zeros(nX,nT);

V1 = zeros(nX,5);
iter = 1;
do until iter > 5;
  if iter == 1;
    option_model = 1;
    option_sigma = sigma_mean;
  elseif iter == 2;
    option_model = 1;
    option_sigma = sigma_max;
  elseif iter == 3;
    option_model = 1;
    option_sigma = sigma_min;
  elseif iter == 4;
    option_model = 2;
  elseif iter == 5;
    option_model = 3;
  endif;  
  
  call PDE_solve(tMin,tMax,nT,xMin,xMax,nX,"pde",theta);
  t = PDE_readFile("pde","t");
  x = PDE_readFile("pde","x");
  V1[.,iter] = PDE_readFile("pde","t"|t[nT]);

  if iter == 4;
    gamma1 = option_gamma_matrix;
  endif;

  iter = iter + 1;
endo;

_fcmptol = 1e-5;
ST = seqa(80,5,9);
indx = closest_index(ST,x);
ST = x[indx];
ST = x;
C = EuropeanBS_KOC(ST,option_K,sigma_mean,tau,b,r,tau,option_L,option_H);

V2 = zeros(nX,5);
iter = 1;
do until iter > 5;
  if iter == 1;
    option_model = 4;
    option_sigma = sigma_mean;
  elseif iter == 2;
    option_model = 4;
    option_sigma = sigma_max;
  elseif iter == 3;
    option_model = 4;
    option_sigma = sigma_min;
  elseif iter == 4;
    option_model = 5;
  elseif iter == 5;
    option_model = 6;
  endif;  
  
  call PDE_solve(tMin,tMax,nT,xMin,xMax,nX,"pde",theta);
  t = PDE_readFile("pde","t");
  x = PDE_readFile("pde","x");
  V2[.,iter] = PDE_readFile("pde","t"|t[nT]);

  if iter == 4;
    gamma2 = option_gamma_matrix;
  endif;

  iter = iter + 1;
endo;

graphset;
  begwind;
  window(2,2,0);

  fonts("simplex simgrma");

  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3; _plwidth = 10; _pcolor = 1|4|2;
  _paxht = 0.225; _pnumht = 0.225; _ptitlht = 0.30;

  xtics(80,120,10,2);
  ytics(0,3.0,0.5,2);
  xlabel("\216S]0[");

  setwind(1);
    title("\216Continuous barrier (BS)");
    _plegstr = "\202s\201 = 25%\000\202s\201 = 20%\000\202s\201 = 15%";
    _plegctl = {2 5 5.5 4.0};
    xy(x,V1[.,2 1 3]);

  setwind(2);
    title("\216Continuous barrier (UVM)");
    _plegstr = "Upper bound\000\202s\201 = 20%\000Lower bound";
    xy(x,V1[.,4 1 5]);

  xtics(50,150,25,2);
  ytics(0,6,1,2);

  setwind(3);
    title("\216Window barrier (BS)");
    _plegstr = "\202s\201 = 25%\000\202s\201 = 20%\000\202s\201 = 15%";
    _plegctl = {2 5 1.0 4.0};
    xy(x,V2[.,2 1 3]);

  setwind(4);
    title("\216Window barrier (UVM)");
    _plegstr = "Upper bound\000\202s\201 = 20%\000Lower bound";
    xy(x,V2[.,4 1 5]);

  endwind;