new;
library frm,pgraph;
frmset;

cls;

declare matrix model;

S0 = 100;
K = 100;
sigma = 0.20;
tau = 0.5;
r = 0.05;
b = 0.02;

proc SigmaSurface(tau,K);
  retp( sigma + 0.*tau + 0.* K );
endp;

proc (1) = aProc(t,x);
  local v;
  if model == 1;
    v = 0.5*(sigma^2) .* (x^2);
  else;
    v = 0.5*(SigmaSurface(t,x)^2) .* (x^2);
  endif;
  retp(v);
endp;

proc (1) = bProc(t,x);
  local v;
  if model == 1;
    v = b .* x ;
  else;
    v = -b .* x;
  endif;
  retp(v);
endp;

proc (1) = cProc(t,x);
  local v;
  if model == 1;
    v = r .* ones(rows(x),1);
  else;
    v = (r - b) .* ones(rows(x),1);
  endif;
  retp(v);
endp;

proc dProc(t,x);
  retp( zeros(rows(x),1) );
endp;

proc (1) = tminBound(t,x);
  local v;
  if model == 1;
    v = EuropeanPayoff(x,K);
  else;
    v = EuropeanPayoff(S0,x);
  endif;
  retp(v);
endp;

proc (1) = xminBound(t,x);
  local v;
  if model == 1;
    v = zeros(rows(x),1);
  elseif model == 2;

  elseif model == 3;
    v = exp((b-r)*t * S0);
  endif;
  retp(v);
endp;

proc (1) = xmaxBound(t,x);
  local v;
  if model == 1;
    
  elseif model == 2;
    v = zeros(rows(x),1);
  endif;
  retp(v);
endp;

proc (1) = DxminBound(t,x);
  local v;
  if model == 1;
    
  elseif model == 2;
    v = -ones(rows(x),1);
  endif;
  retp(v);
endp;

proc (1) = DxmaxBound(t,x);
  local v;
  if model == 1;
    v = ones(rows(x),1);    
  elseif model == 2;

  elseif model == 3;
    v = zeros(rows(x),1);    
  endif;
  retp(v);
endp;

// Parameters of the PDE

_PDE_SaveLastIter = 1;
_PDE_PrintIters = 50;
__output = 1;

xmin = 0; xmax = 200; nX = 1001;
tmin = 0; tmax = tau; nT = 2500;

// Backward PDE

model = 1;
call PDE(&aProc,&bProc,&cProc,&dProc,0,&tminBound,&xminBound,0,0,&DxmaxBound);
fileName = "dupire1";
call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,fileName,0.5);

// Forward PDE

model = 2;
call PDE(&aProc,&bProc,&cProc,&dProc,0,&tminBound,0,&xmaxBound,&DxminBound,0);
fileName = "dupire2";
call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,fileName,0.5);

model = 3;
call PDE(&aProc,&bProc,&cProc,&dProc,0,&tminBound,&xminBound,0,0,&DxmaxBound);
fileName = "dupire3";
call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,fileName,0.5);

// Comparison

fileName = "dupire1";

tau = PDE_readFile(fileName,"t");
x = PDE_readFile(fileName,"x");
U = PDE_readFile(fileName,"t"|tau);
e = x .> 10 .and x .< 190;
U1 = selif(U,e);
x1 = selif(x,e);
C1 = EuropeanBS(x1,K,sigma,tau,b,r);

fileName = "dupire2";

tau = PDE_readFile(fileName,"t");
x = PDE_readFile(fileName,"x");
U = PDE_readFile(fileName,"t"|tau);
e = x .> 10 .and x .< 160;
U2 = selif(U,e);
x2 = selif(x,e);
C2 = EuropeanBS(S0,x2,sigma,tau,b,r);

fileName = "dupire3";

tau = PDE_readFile(fileName,"t");
x = PDE_readFile(fileName,"x");
U = PDE_readFile(fileName,"t"|tau);
e = x .> 10 .and x .< 160;
U3 = selif(U,e);
x3 = selif(x,e);
C3 = EuropeanBS(S0,x3,sigma,tau,b,r);

reldiff1 = (U1-C1)./C1;
reldiff2 = (U2-C2)./C2;
reldiff3 = (U3-C3)./C3;

N = maxc(rows(x1)|rows(x2));
x = miss(zeros(N,3),0);
reldiff = miss(zeros(N,3),0);
x[1:rows(x1),1] = x1;
x[1:rows(x2),2] = x2;
x[1:rows(x2),2] = x3;

reldiff[1:rows(x1),1] = reldiff1;
reldiff[1:rows(x2),2] = reldiff2;
reldiff[1:rows(x3),3] = reldiff3;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0; 
  _plwidth = 10; _pcolor = 1|4; _pltype = 6|1;
  xlabel("\216State variable (S]0[ or K)");
  ylabel("\216Relative error (in bps)");
  ytics(-0.50,1.00,0.25,0);
  xtics(50,150,10,2);
  _plegstr = "Backward PDE\000Forward PDE";
  _plegctl = {2 5 6 5};
  _pline = 1~3~100~-0.50~100~-0.22~1~0~10 |
           1~3~0~-0.22~100~-0.22~1~0~10 ;
  xy(x[.,1:2],1e4*reldiff[.,1:2]);

