/*
**> In this program, we compute the local volatility from the implied volatility
** 
*/

new;
library frm,pgraph;
frmset;

Smile_T = 1/12|1/4|1/2|1|2;
let Smile_K[9,5] = 
        87         77         66         53        37 
        92         85         78         69        56 
        96         93         89         86        80 
        98         97         96         96        96 
       100        101        102        104        103 
       103        106        111        119        137 
       106        111        119        133        163 
       110        121        136        166        229 
       116        134        161        217        347; 

let Smile_ImpVol[9,5] = 
      0.137       0.149       0.168       0.190       0.219 
      0.137       0.149       0.168       0.190       0.219 
      0.133       0.141       0.155       0.170       0.200 
      0.132       0.140       0.150       0.160       0.185 
      0.130       0.135       0.145       0.155       0.185 
      0.131       0.138       0.150       0.165       0.190 
      0.132       0.142       0.155       0.175       0.195 
      0.135       0.151       0.168       0.185       0.208 
      0.135       0.151       0.168       0.185       0.208;

data = miss(zeros(10,9),0);
indx = seqa(1,2,5);
data[indx,.] = Smile_K';
data[indx+1,.] = 100*Smile_ImpVol';

print ftosa(data,"%lf",3,1);

x_K = seqa(0,1,201);
S_K = zeros(rows(x_K),rows(Smile_T));

i = 1;
do until i > rows(Smile_T);
  x = smile_K[.,i];
  y = Smile_ImpVol[.,i]; 
  c = csspline(x,y,1,1);
  S_K[.,i] = fspline(c,x_K);
  i = i + 1;
endo;

S0 = 100;
b = 0.05;
r = 0.05;

K = seqa(0,1,201);

t = seqa(0,10/250,75);

call ImpVol_to_LocalVol_CSGI(Smile_K,Smile_ImpVol,Smile_T,T,K,S0,b,r,"chap13-2-3-4");
Implied_Volatility = loadd("chap13-2-3-4-impvol");
Local_Volatility = loadd("chap13-2-3-4-localvol");

pqgwin many;

sym = (8|9|10|12|13)' .* ones(rows(Smile_K),1);

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pframe = 0; _pnum = 2;
  _plwidth = 10; _pltype = 6|1|3; _pcolor = 1|2|3|4|5|6;
  xlabel("\216K");
  ylabel("\216S]m[\202\34\201K\202\118\201");
  _plegstr = "T = 1/12\000T = 3/12\000T = 6/12\000T = 1\000T = 2";
  _plegctl = {2 5 6 5};
  n = rows(sym)*cols(sym);
  _psym = vec(Smile_K)~100*vec(Smile_ImpVol)~vec(sym)~ones(n,1).*(3.5~0~1~5);
  ytics(10,24,2,2);
  xtics(20,180,40,2);
  xy(x_K,100*S_K);

graphset;
  _pnum = 2; _pframe = 0; _pdate = "";
  _pzclr = seqa(1,1,7);
  xlabel("\216K");
  ylabel("\216T");
  xtics(0,200,50,2);
  ytics(0,2,0.5,2);
  ztics(10,24,2,2);
  surface(K',t,100*Implied_Volatility);

graphset;
  _pnum = 2; _pframe = 0; _pdate = "";
  _pzclr = seqa(1,1,7);
  xlabel("\216K");
  ylabel("\216T");
  xtics(0,200,50,2);
  ytics(0,2,0.5,2);
  ztics(5,30,5,2);
  surface(K',t,100*Local_Volatility);