/*
**> In this program, we compute the local volatility from the implied volatility
** 
*/

new;
library frm,optmum,pgraph;
frmset;

cls;

declare matrix _SVI_x;
declare matrix _SVI_ImpVol;
declare matrix _SVI_T;

proc (5) = SVI_Parameters(theta);
  local alpha,beta,sigma,rho,m;
  alpha = sqrt(theta[1]^2);
  beta = sqrt(theta[2]^2);
  sigma = theta[3];
  rho = 2*exp(-theta[4]^2) - 1.0;
  m = sqrt(theta[5]^2);
  retp(alpha,beta,sigma,rho,m);
endp;

proc (1) = rss(theta);
  local alpha,beta,sigma,rho,m;
  local v,u;
  {alpha,beta,sigma,rho,m} = SVI_Parameters(theta);
  v = SVI_Raw_Parametrization(_SVI_x,alpha,beta,sigma,rho,m);
  u = v - _SVI_T .* (_SVI_ImpVol^2);
  retp(u'u);
endp;

Smile_T = 1/12|1/4|1/2|1|2;
let Smile_K[9,5] = 
        87         77         66         53        37 
        92         85         78         69        56 
        96         93         89         86        80 
        98         97         96         96        96 
       100        101        102        104        103 
       103        106        111        119        137 
       106        111        119        133        163 
       110        121        136        166        229 
       116        134        161        217        347; 

let Smile_ImpVol[9,5] = 
      0.137       0.149       0.168       0.190       0.219 
      0.137       0.149       0.168       0.190       0.219 
      0.133       0.141       0.155       0.170       0.200 
      0.132       0.140       0.150       0.160       0.185 
      0.130       0.135       0.145       0.155       0.185 
      0.131       0.138       0.150       0.165       0.190 
      0.132       0.142       0.155       0.175       0.195 
      0.135       0.151       0.168       0.182       0.208 
      0.135       0.151       0.168       0.182       0.208;

Smile_K = trimr(Smile_K,1,1);
Smile_ImpVol = trimr(Smile_ImpVol,1,1);

S0 = 100;
b = 0.05;
r = 0.05;

Smile_x = ln(Smile_K/S0) + b .* Smile_T';

x_K = seqa(-1,0.025,81);
S_K = zeros(rows(x_K),rows(Smile_T));
_SVI_theta = zeros(5,rows(Smile_T));

i = 1;
do until i > rows(Smile_T);
  _SVI_T = Smile_T[i];
  _SVI_ImpVol = Smile_ImpVol[.,i]; 
  _SVI_x = Smile_x[.,i]; 
  sv = minc(_SVI_ImpVol)|0.0001|0.0001|-1.0|0.0001;
  {theta,fmin,grd,retcode} = optmum(&rss,sv);
  {alpha,beta,sigma,rho,m} = SVI_Parameters(theta);
  v = SVI_Raw_Parametrization(x_K,alpha,beta,sigma,rho,m);
  S_K[.,i] = sqrt(v/_SVI_T);
  _SVI_theta[.,i] = theta;
  i = i + 1;
endo;

sym = (8|9|10|12|13)' .* ones(rows(Smile_K),1);

proc SigmaProc(Tm,K);
  local theta,alpha,beta,sigma,rho,m,x,v,ImpVol;

  if Tm <= Smile_T[1];
    theta = _SVI_theta[.,1];
  elseif Tm <= Smile_T[2];
    theta = _SVI_theta[.,2];
  elseif Tm <= Smile_T[3];
    theta = _SVI_theta[.,3];
  elseif Tm <= Smile_T[4];
    theta = _SVI_theta[.,4];
  else;
    theta = _SVI_theta[.,5];
  endif;
  {alpha,beta,sigma,rho,m} = SVI_Parameters(theta);

  x = ln(K/S0) + b .* Tm;
  v = SVI_Raw_Parametrization(x,alpha,beta,sigma,rho,m);
  ImpVol = sqrt(v/Tm);

  retp(ImpVol);
endp;

proc ImpVolProc(T,K);
  local ImpVol;
  if rows(K) == 1;
    ImpVol = ImpVol_Time_Interpolation(&SigmaProc,Smile_T,T,K',b)';
  else;
    ImpVol = ImpVol_Time_Interpolation(&SigmaProc,Smile_T,T,K,b);
  endif;
  retp(ImpVol);
endp;

K = seqa(70,1,29) | seqa(102,1,30);

T = seqa(1/4,1/10,31);
ImpVol = ImpVolProc(T,K);
LocalVol = zeros(rows(T),rows(K));
i = 1;
do until i > rows(T);
  LocalVol[i,.] = ImpVol_to_LocalVol(&ImpVolProc,0,0,0,T[i],K',S0,b);
  i = i + 1;
endo;

graphset;
  window3u;

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3|2|5; _plwidth = 10; _pcolor = 1|4|2|3|5; _plctrl = 0|0|0|10|10; _pstype = 8|8|8|8|9; _psymsiz = 5;
  _pnumht = 0.225; _paxht = 0.225; _ptitlht = 0.275;

  setwind(1);
    xlabel("\216x");
    ylabel("\216S]m[\202\34\201K\202\118\201");
    _plegstr = "T = 1/12\000T = 3/12\000T = 6/12\000T = 1\000T = 2";
    _plegctl = {2 5 9.5 3};
    n = rows(sym)*cols(sym);
    _psym = vec(Smile_x)~100*vec(Smile_ImpVol)~vec(sym)~ones(n,1).*(3.5~0~1~5);
    ytics(12,24,2,2);
    xtics(-1,1,0.25,2);
    xy(x_K,100*S_K);

  _plegctl = 0;
  _pzclr = seqa(1,1,7);

  setwind(2);
    title("\216\202S\201(T,K)");
    xtics(70,130,10,2);
    ytics(0,4,1,2);
    ztics(12,24,2,2);
    xlabel("\216K");
    ylabel("\216T");
    surface(K',T,100*ImpVol');

  setwind(3);
    title("\216\202s\201(T,K)");
    ztics(14,24,2,2);
    surface(K',T,100*LocalVol);

  endwind;