/*
**> chap13-2-3-8
** 
*/

new;
library frm,optmum,pgraph;
frmset;
cls;

S0 = 100;
b = 0.05;
r = 0.05;

tau = 1.0;
K = 100;
L = 90;
H = 115;

sigma = 0.155;

let string OptionTypes[10,1] = "call" "put" "doc" "dop" "uoc" "uop" "koc" "kop" "bcc" "bcp";

iter = 1;
do until iter > 10;
  OptionType = OptionTypes[iter];

  proc aProc(t,x);
    retp( 0.5*(sigma^2) .* (x^2) );
  endp;

  proc bProc(t,x);
    retp( b .* x );
  endp;

  proc cProc(t,x);
    retp( r .* ones(rows(x),1) );
  endp;

  proc dProc(t,x);
    retp( zeros(rows(x),1) );
  endp;

  proc eProc(t,x,U);
    local e;
    if (OptionType $== "doc");
      e = (x .>= L);
    elseif (OptionType $== "dop");
      e = (x .>= L);
    elseif (OptionType $== "uoc");
      e = (x .<= H);
    elseif (OptionType $== "uop");
      e = (x .<= H);
    elseif (OptionType $== "koc");
      e = (x .>= L) .and (x .<= H);
    elseif (OptionType $== "kop");
      e = (x .>= L) .and (x .<= H);
    else;
      e = 1;
    endif;

    retp( e .* U );
  endp;

  proc tminBound(t,x);
    local Payoff;

    if (OptionType $== "call");
      Payoff = x - K;
      Payoff = Payoff .* (Payoff .> 0);
    elseif (OptionType $== "put");
      Payoff = K - x;
      Payoff = Payoff .* (Payoff .> 0);
    elseif (OptionType $== "doc");
      Payoff = x - K;
      Payoff = Payoff .* (Payoff .> 0) .* (x .>= L);
    elseif (OptionType $== "dop");
      Payoff = K - x;
      Payoff = Payoff .* (Payoff .> 0) .* (x .>= L);
    elseif (OptionType $== "uoc");
      Payoff = x - K;
      Payoff = Payoff .* (Payoff .> 0) .* (x .<= H);
    elseif (OptionType $== "uop");
      Payoff = K - x;
      Payoff = Payoff .* (Payoff .> 0) .* (x .<= H);
    elseif (OptionType $== "koc");
      Payoff = x - K;
      Payoff = Payoff .* (Payoff .> 0) .* (x .>= L) .* (x .<= H);
    elseif (OptionType $== "kop");
      Payoff = K - x;
      Payoff = Payoff .* (Payoff .> 0) .* (x .>= L) .* (x .<= H);
    elseif (OptionType $== "bcc");
      Payoff = x .>= K;
      Payoff = Payoff .* (Payoff .> 0);
    elseif (OptionType $== "bcp");
      Payoff = x .<= K;
      Payoff = Payoff .* (Payoff .> 0);
    endif;

    retp(Payoff);
  endp;

  // DxminBound

  proc xminBound_zero(t,x);
    retp( zeros(rows(x),1) );
  endp;

  // xmaxBound

  proc xmaxBound_zero(t,x);
    retp( zeros(rows(x),1) );
  endp;

  // DxminBound

  proc DxminBound_minus_one(t,x);
    retp( -ones(rows(x),1) );
  endp;

  // DxmaxBound

  proc DxmaxBound_one(t,x);
    retp( ones(rows(x),1) );
  endp;

  xmin = 0; xmax = 200; Nx = 2001;
  tmin = 0; tmax = tau; Nt = 2001;

  _PDE_SaveLastIter = 1;
  _PDE_PrintIters = 0;
  __output = 0;

  U = zeros(Nx,1);

  if (OptionType $== "call");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,0,0,&DxmaxBound_one);
  elseif (OptionType $== "put");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,0,&xmaxBound_zero,&DxminBound_minus_one,0);
  elseif (OptionType $== "doc");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,0,0,&DxmaxBound_one);
  elseif (OptionType $== "dop");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,&xmaxBound_zero,0,0);
  elseif (OptionType $== "uoc");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,&xmaxBound_zero,0,0);
  elseif (OptionType $== "uop");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,0,&xmaxBound_zero,&DxminBound_minus_one,0);
  elseif (OptionType $== "koc");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,&xmaxBound_zero,0,0);
  elseif (OptionType $== "kop");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,&xmaxBound_zero,0,0);
  elseif (OptionType $== "bcc");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,0,0,&DxmaxBound_one);
  elseif (OptionType $== "bcp");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,0,&xmaxBound_zero,&DxminBound_minus_one,0);
  endif;

  call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"pde",0.5);
  tau = PDE_readFile("pde","t");
  x = PDE_readFile("pde","x");
  U[.,1] = PDE_readFile("pde","t"|tau);
  indx = minindc(abs(S0-x));
  V1 = U[indx];

  if (OptionType $== "call");
    V2 = EuropeanBS_CALL(S0,K,sigma,tau,b,r);
  elseif (OptionType $== "put");
    V2 = EuropeanBS_PUT(S0,K,sigma,tau,b,r);
  elseif (OptionType $== "doc");
    V2 = EuropeanBS_DOC(S0,K,sigma,tau,b,r,tau,L,0);
  elseif (OptionType $== "dop");
    V2 = EuropeanBS_DOP(S0,K,sigma,tau,b,r,tau,L,0);
  elseif (OptionType $== "uoc");
    V2 = EuropeanBS_UOC(S0,K,sigma,tau,b,r,tau,H,0);
  elseif (OptionType $== "uop");
    V2 = EuropeanBS_UOP(S0,K,sigma,tau,b,r,tau,H,0);
  elseif (OptionType $== "koc");
    V2 = EuropeanBS_KOC(S0,K,sigma,tau,b,r,tau,L,H);
  elseif (OptionType $== "kop");
    V2 = EuropeanBS_KOP(S0,K,sigma,tau,b,r,tau,L,H);
  elseif (OptionType $== "bcc");
    V2 = EuropeanBS_BCC(S0,K,sigma,tau,b,r);
  elseif (OptionType $== "bcp");
    V2 = EuropeanBS_BCP(S0,K,sigma,tau,b,r);
  endif;

  V = V1~V2;
  results = OptionType $~ ftosa(V,"%lf",5,2);
  print /flush results;

  iter = iter + 1;
endo;

print;

iter = 1;
do until iter > 10;
  OptionType = OptionTypes[iter];

  proc aProc(t,x);
    retp( 0.5*(sigma^2) .* (x^2) );
  endp;

  proc bProc(t,x);
    retp( b .* x );
  endp;

  proc cProc(t,x);
    retp( r .* ones(rows(x),1) );
  endp;

  proc dProc(t,x);
    retp( zeros(rows(x),1) );
  endp;

  proc eProc(t,x,U);
    local e;
    if (OptionType $== "doc");
      e = (x .>= L);
    elseif (OptionType $== "dop");
      e = (x .>= L);
    elseif (OptionType $== "uoc");
      e = (x .<= H);
    elseif (OptionType $== "uop");
      e = (x .<= H);
    elseif (OptionType $== "koc");
      e = (x .>= L) .and (x .<= H);
    elseif (OptionType $== "kop");
      e = (x .>= L) .and (x .<= H);
    else;
      e = 1;
    endif;

    retp( e .* U );
  endp;

  proc tminBound(t,x);
    local Payoff;

    if (OptionType $== "call");
      Payoff = x - K;
      Payoff = Payoff .* (Payoff .> 0);
    elseif (OptionType $== "put");
      Payoff = K - x;
      Payoff = Payoff .* (Payoff .> 0);
    elseif (OptionType $== "doc");
      Payoff = x - K;
      Payoff = Payoff .* (Payoff .> 0) .* (x .>= L);
    elseif (OptionType $== "dop");
      Payoff = K - x;
      Payoff = Payoff .* (Payoff .> 0) .* (x .>= L);
    elseif (OptionType $== "uoc");
      Payoff = x - K;
      Payoff = Payoff .* (Payoff .> 0) .* (x .<= H);
    elseif (OptionType $== "uop");
      Payoff = K - x;
      Payoff = Payoff .* (Payoff .> 0) .* (x .<= H);
    elseif (OptionType $== "koc");
      Payoff = x - K;
      Payoff = Payoff .* (Payoff .> 0) .* (x .>= L) .* (x .<= H);
    elseif (OptionType $== "kop");
      Payoff = K - x;
      Payoff = Payoff .* (Payoff .> 0) .* (x .>= L) .* (x .<= H);
    elseif (OptionType $== "bcc");
      Payoff = x .>= K;
      Payoff = Payoff .* (Payoff .> 0);
    elseif (OptionType $== "bcp");
      Payoff = x .<= K;
      Payoff = Payoff .* (Payoff .> 0);
    endif;

    retp(Payoff);
  endp;

  // DxminBound

  proc xminBound_zero(t,x);
    retp( zeros(rows(x),1) );
  endp;

  // xmaxBound

  proc xmaxBound_zero(t,x);
    retp( zeros(rows(x),1) );
  endp;

  // DxminBound

  proc DxminBound_minus_one(t,x);
    retp( -ones(rows(x),1) );
  endp;

  // DxmaxBound

  proc DxmaxBound_one(t,x);
    retp( ones(rows(x),1) );
  endp;

  xmin = 0; xmax = 250; Nx = 2001;
  tmin = 0; tmax = tau; Nt = 2001;

  _PDE_SaveLastIter = 1;
  _PDE_PrintIters = 0;
  __output = 0;

  U = zeros(Nx,1);

  if (OptionType $== "call");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,0,0,&DxmaxBound_one);
  elseif (OptionType $== "put");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,0,&xmaxBound_zero,&DxminBound_minus_one,0);
  elseif (OptionType $== "doc");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,0,0,&DxmaxBound_one);
  elseif (OptionType $== "dop");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,&xmaxBound_zero,0,0);
  elseif (OptionType $== "uoc");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,&xmaxBound_zero,0,0);
  elseif (OptionType $== "uop");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,&xmaxBound_zero,0,0);
  elseif (OptionType $== "koc");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,&xmaxBound_zero,0,0);
  elseif (OptionType $== "kop");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,&xmaxBound_zero,0,0);
  elseif (OptionType $== "bcc");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,&xminBound_zero,0,0,&DxmaxBound_one);
  elseif (OptionType $== "bcp");
    call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
             &tminBound,0,&xmaxBound_zero,&DxminBound_minus_one,0);
  endif;

  call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"pde",0.5);
  tau = PDE_readFile("pde","t");
  x = PDE_readFile("pde","x");
  U[.,1] = PDE_readFile("pde","t"|tau);
  indx = minindc(abs(S0-x));
  V1 = U[indx];

  if (OptionType $== "call");
    V2 = EuropeanBS_CALL(S0,K,sigma,tau,b,r);
  elseif (OptionType $== "put");
    V2 = EuropeanBS_PUT(S0,K,sigma,tau,b,r);
  elseif (OptionType $== "doc");
    V2 = EuropeanBS_DOC(S0,K,sigma,tau,b,r,tau,L,0);
  elseif (OptionType $== "dop");
    V2 = EuropeanBS_DOP(S0,K,sigma,tau,b,r,tau,L,0);
  elseif (OptionType $== "uoc");
    V2 = EuropeanBS_UOC(S0,K,sigma,tau,b,r,tau,H,0);
  elseif (OptionType $== "uop");
    V2 = EuropeanBS_UOP(S0,K,sigma,tau,b,r,tau,H,0);
  elseif (OptionType $== "koc");
    V2 = EuropeanBS_KOC(S0,K,sigma,tau,b,r,tau,L,H);
  elseif (OptionType $== "kop");
    V2 = EuropeanBS_KOP(S0,K,sigma,tau,b,r,tau,L,H);
  elseif (OptionType $== "bcc");
    V2 = EuropeanBS_BCC(S0,K,sigma,tau,b,r);
  elseif (OptionType $== "bcp");
    V2 = EuropeanBS_BCP(S0,K,sigma,tau,b,r);
  endif;

  V = V1~V2;
  results = OptionType $~ ftosa(V,"%lf",5,2);
  print /flush results;

  iter = iter + 1;
endo;













