new;
library frm,pgraph;
frmset;

declare matrix Dividend;

//__output = 0;

S0 = 100;
K = 100;
sigma = 0.30;
r = 0.05;
b = r;
tau = 1.0;

D = (0.00|0.03|0.05|0.10)*S0;
t_D = 0.5*tau;

C = zeros(4,3);
P = zeros(4,3);

_option_type = "call";
C[.,1:2] = EuropeanBS(S0-D*exp(-r*t_D),K,sigma,tau,b,r)~EuropeanBS(S0,K+D*exp(r*(tau-t_D)),sigma,tau,b,r);

_option_type = "put";
P[.,1:2] = EuropeanBS(S0-D*exp(-r*t_D),K,sigma,tau,b,r)~EuropeanBS(S0,K+D*exp(r*(tau-t_D)),sigma,tau,b,r);

print ftosa(C~P,"%lf",3,2);

proc (1) = aProc(t,x);
  retp(0.5*(sigma^2) .* (x^2));
endp;

proc (1) = bProc(t,x);
  retp(b .* x);
endp;

proc (1) = cProc(t,x);
  retp(r .* ones(rows(x),1));
endp;

proc (1) = dProc(t,x);
  retp( zeros(rows(x),1) );
endp;

proc (1) = eProc(t,x,U);
  local dx,nLags,V;
  if abs(t-t_D) <= 0.00001;
    dx = maxc(x - lag1(x));
    nLags = trunc(Dividend/dx);
    V = lagn(U,nLags);
    if nLags > 0;
      V[1:nLags] = U[1]*ones(nLags,1);
    endif;
  else;
    V = U;
  endif;

  retp(V);
endp;

proc (1) = tminBound(t,x);
  local G;
  G = EuropeanPayoff(x,K);
  retp(G);
endp;

proc (1) = xminBound(t,x);
  retp(zeros(rows(x),1));
endp;

proc (1) = DxminBound(t,x);
  retp(-ones(rows(x),1));
endp;

proc (1) = xmaxBound(t,x);
  retp(zeros(rows(x),1));    
endp;

proc (1) = DxmaxBound(t,x);
  retp(ones(rows(x),1));    
endp;

// Parameters of the PDE

_PDE_SaveLastIter = 1;
_PDE_PrintIters = 50;

xmin = 0; xmax = 300; nX = 3001;
tmin = 0; tmax = tau; nT = 1001;

i = 1;
do until i > 4;

  Dividend = D[i];

  iter = 1;
  do until iter > 2;

    if iter == 1;
      _option_type = "call";
    else;
      _option_type = "put";
    endif;

    if _option_type $== "call";
      call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,&tminBound,&xminBound,0,0,&DxmaxBound);
    else;
      call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,&tminBound,0,&DxminBound,&xmaxBound,0);
    endif;

    fileName = "tmp";
    call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,fileName,0.5);

    tau = PDE_readFile(fileName,"t");
    x = PDE_readFile(fileName,"x");
    U = PDE_readFile(fileName,"t"|tau);
    indx = indnv1(S0,x);

    if _option_type $== "call";
      C[i,3] = U[indx];
    else;
      P[i,3] = U[indx];
    endif;

    iter = iter + 1;
  endo;

  i = i + 1;
endo;

print ftosa(C~P,"%lf",3,2);

