new;
library frm,pgraph,optmum;

n = 5;
let sigma = 0.20 0.20 0.20 0.20 0.20;
let S0 = 100 100 100 100 100;
r = 0.05;
tau = 3/12;

K = 5;
L = 105;

proc Payoff(S);
  local S1,S2,S3,S4,S5,payoff;
  S1 = S[.,1];
  S2 = S[.,2];
  S3 = S[.,3];
  S4 = S[.,4];
  S5 = S[.,5];

  payoff = (S1 - S2 + S3 - S4 - K) .* (S5 .> L);
  payoff = payoff .* (payoff .>= 0);

  retp( PayOff );
endp;

rndseed 123;
nS = 10000|100;

rho = seqa(-1/(n-1)+1e-3,0.05,26);
rho[26] = 1;
nRho = rows(rho);
option = miss(zeros(nRho,1),0);

i = 1;
do until i > nRho;

  C = Constant_Correlation_Matrix(rho[i],N);
  if C == error(0);
    i = i + 1;
    continue;
  endif;

  tmp = zeros(nS[2],1);

  j = 1;
  do until j > nS[2];
    S = simulate_mGBM_T(S0,r,sigma,C,0,tau,nS[1]);  
    tmp[j] = exp(-r*tau)*meanc(Payoff(S));
    j = j + 1;
  endo;

  option[i] = meanc(tmp);

  i = i + 1;
endo;

max = maxindc(option);
max_option = option[max];
max_rho = Constant_Correlation_Matrix(rho[max],N);

M = 25;
option2 = zeros(M,1);
rho2 = zeros(M,N*(N+1)/2);

rndseed 15699;
rndseed 955666115;
i = 1;
do until i > M;
  lambda = rndu(n,1);
  C = Simulate_Correlation_Matrix(n,lambda,0);
  rho2[i,.] = vech(C)';
  i = i + 1;
endo;

rndseed 123;
nS = 10000|50;
i = 1;
do until i > M;
  C = xpnd(rho2[i,.]');
  tmp = zeros(nS[2],1);
  j = 1;
  do until j > nS[2];
    S = simulate_mGBM_T(S0,r,sigma,C,0,tau,nS[1]);  
    tmp[j] = exp(-r*tau)*meanc(Payoff(S));
    j = j + 1;
  endo;
  option2[i] = meanc(tmp);
  i = i + 1;
endo;

max2_indx = maxindc(option2);
max2_option = option2[max2_indx];
max2_rho = xpnd(rho2[max2_indx,.]');

rndseed 123;
nS = 10000|500;
tmp = zeros(nS[2],1);
j = 1;
do until j > nS[2];
  S = simulate_mGBM_T(S0,r,sigma,max2_rho,0,tau,nS[1]);  
  tmp[j] = exp(-r*tau)*meanc(Payoff(S));
  j = j + 1;
endo;
max2_option = meanc(tmp);

cls;
print ftosa(max_rho,"%lf",1,4);
print; print ftos(max_option,"                 Option price = %lf",5,3);
print "------------------------------------------------------";
print ftosa(max2_rho,"%lf",1,4);
print; print ftos(max2_option,"                 Option price = %lf",5,3);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _pcolor = 1; _plwidth = 10;
  fonts("simplex simgrma");
  xlabel("\216\202r\201 (in %)");
  xtics(-25,100,25,5);
  ytics(0,2.5,0.5,5);
  xy(100*rho,option);

proc simulate_mGBM_T(S0,mu,sigma,rho,t0,TT,Ns);
  local ST,N,dt,dw,Pchol;

  S0 = S0'; mu = mu'; sigma = sigma';

  N = maxc(cols(S0)|cols(mu)|cols(sigma));
  dt = TT - t0;

  if rho == 1;
    dw = sqrt(dt) .* rndn(Ns,1) .* ones(1,N);
  elseif rho == -1 and N == 2;
    dw = sqrt(dt) .* (1~-1) .* rndn(Ns,1);
  else; 
    Pchol = chol(rho);
    dw = sqrt(dt) .* rndn(Ns,N)*Pchol;
  endif;

  ST = S0 .* exp( (mu-0.5*sigma^2) .* dt + sigma .* dw );

  retp(ST);
endp;
 
