new;
library frm,pgraph;

cls;

F1 = 0.05;
alpha1 = 8.944/100;
beta1 = 1.000;
nu1 = 0.322;
rho1 = -22.901/100;

F2 = 0.06;
alpha2 = 12.404/100;
beta2 = 1.000;
nu2 = 0.280;
rho2 = 16.974/100;

tau = 1;

K1 = F1;
K2 = F2;

ImpVol_ATM1 = SABR_ImpVol(F1,K1,alpha1,beta1,nu1,rho1,tau);
ImpVol_ATM2 = SABR_ImpVol(F2,K2,alpha2,beta2,nu2,rho2,tau);

print ftosa(100*(ImpVol_ATM1|ImpVol_ATM2),"%lf",3,2);

nS = 200000;

rhoVector = -0.999|seqa(-0.9,0.1,19)|0.999;
nIters = rows(rhoVector);
C_BS = zeros(nIters,1);
C_SABR = zeros(nIters,1);

K = 0.02;
t = seqa(0,1/260,tau*260+1);
nT = rows(T);

rndseed 123;

iter = 1;
do until iter > nIters;
  rho = rhoVector[iter];
  {F1_T,F2_T} = simulate_GBM2(F1,F2,0,0,ImpVol_ATM1,ImpVol_ATM2,rho,tau,nS);
  F1_T = F1_T';
  F2_T = F2_T';
  C_BS[iter] = meanc(_max_(_max_(F1_T/F1-1,F2_T/F2-1) - K,0));

  {F1_T,alpha1_t} = simulate_SABR(F1,alpha1,beta1,nu1,rho1,t,nS,1,1);
  {F2_T,alpha2_t} = simulate_SABR(F2,alpha2,beta2,nu2,rho2,t,nS,1,1);
  {u1,u2} = rndCopulaNormal2(rho,nS);
  F_T = rndCopulaEmpiricalQuantile(u1~u2,F1_T~F2_T);
  F1_T = F_T[.,1];
  F2_T = F_T[.,2];
  C_SABR[iter] = meanc(_max_(_max_(F1_T/F1-1,F2_T/F2-1) - K,0));

  print /flush ftosa(iter,"Iteration = %lf",3,0);
  iter = iter + 1;
endo;

x = rhoVector;
y = C_BS;
x = ones(rows(x),1)~x~x^2;
beta = y/x;
z1 = x*beta;
y = C_SABR;
beta = y/x;
z2 = x*beta;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0; 
  _pltype = 6|3|1; _plwidth = 10; _pcolor = 1|2|4;
  xlabel("\216\202r\201 (in %)"); 
  ylabel("\216Two-asset option price");
  _plegstr = "Black-Scholes\000SABR";
  _plegctl = {2 5 6 5.5};
  xtics(-100,100,25,2);
  ytics(3,7,0.5,2);
if 1;
  xy(100*rhoVector,100*(z1~z2));
else;
  xy(100*rhoVector,100*(C_BS~C_SABR));
endif;


/*
**> simulate_SABR
**
**
*/

proc (2) = simulate_SABR(F0,alpha,beta,nu,rho,t,nS,mtd,save_last);
  local nT,F_t,alpha_t,i,dt,sqrt_dt,dw1,dw2;

  nT = rows(t);
  if save_last;
    F_t = zeros(nT,nS);
    alpha_t = zeros(nT,nS);
  endif;
  F0 = F0 .* ones(1,nS);
  alpha = alpha .* ones(1,nS);

  i = 1;
  do until i > nT;
    if i == 1;
      dt = t[1];
    else;
      dt = t[i] - t[i-1];
    endif;

    sqrt_dt = sqrt(dt);
    dw1 = rndn(1,nS);
    dw2 = rho .* dw1 + sqrt(1-rho^2) .* rndn(1,nS);

    if mtd == 1;
      F0 = F0 + alpha .* (F0^beta) .* sqrt_dt .* dw1;
      alpha = alpha .* exp( -0.5*(nu^2)*sqrt_dt + nu * sqrt_dt .* dw2);
    else;
      F0 = F0 + alpha .* (F0^beta) .* sqrt_dt .* dw1;
      alpha = alpha + nu .* alpha .* sqrt_dt .* dw2;
    endif;

    if not save_last;
      F_t[i,.] = F0;
      alpha_t[i,.] = alpha;
    endif;
    i = i + 1;
  endo;

    if save_last;
      F_t = F0';
      alpha_t = alpha';
    endif;

  retp(F_t,alpha_t);
endp;


