new;
library frm,tsm,pgraph,optmum;

cls;

declare matrix _TV_Beta_KF_y;
declare matrix _TV_Beta_KF_x;
declare matrix _TV_Beta_KF_H;
declare matrix _TV_Beta_KF_Q;
declare matrix _TV_Beta_KF_K;

fn _TV_Beta_KF_Zproc(t) = _TV_Beta_KF_x[t,.];
fn _TV_Beta_KF_dproc(t) = 0;
fn _TV_Beta_KF_Hproc(t) = _TV_Beta_KF_H;
fn _TV_Beta_KF_Tproc(t) = eye(_TV_Beta_KF_K);
fn _TV_Beta_KF_cproc(t) = zeros(_TV_Beta_KF_K,1);
fn _TV_Beta_KF_Rproc(t) = eye(_TV_Beta_KF_K);
fn _TV_Beta_KF_Qproc(t) = _TV_Beta_KF_Q;

proc (0) = TV_Beta_SSM(y,x,a0,P0,H,Q);
  local K;

  K = cols(x);
  _TV_Beta_KF_y = y;
  _TV_Beta_KF_x = x;
  _TV_Beta_KF_H = H;
  _TV_Beta_KF_Q = Q;
  _TV_Beta_KF_K = K;

  call SSM_build(&_TV_Beta_KF_Zproc,&_TV_Beta_KF_dproc,&_TV_Beta_KF_Hproc,
                   &_TV_Beta_KF_Tproc,&_TV_Beta_KF_cproc,&_TV_Beta_KF_Rproc,&_TV_Beta_KF_Qproc,1);
  call KFiltering(y,a0,P0);

  retp;
endp;

datPathName = "..\\..\\data\\";
datFileName = datPathName $+ "equity-indices-2017-monthly"; 
let selected_Names = "MSUS" "MSEMU";

data = loadd(datFileName);
data = selif(data,(data[.,1] .>= 19900101) .and (data[.,1] .<= 20180101));
vnames = "" $+ getname(datFileName);
Dates = data[.,1];
indx_Selected_Names = indsav(Selected_Names,vnames);
Data = data[.,indx_Selected_Names];

Index_USA = Data[.,1];
Index_EMU = Data[.,2];
Index_Data = Index_USA~Index_EMU;
Index_Dates = Dates;
nDates = rows(Dates);
t = seqa(1,1,nDates);

R_Index = Index_Data ./ lag1(Index_Data) - 1;

w = reshape(0,5*12,1) | 
    reshape(0.5,3*12,1) | 
    reshape(0.25,12,1) | 
    reshape(0.75,6,1) | 
    reshape(1.0,20*12,1) ;
w = w[1:nDates];
w1 = w~(1-w);
Period = 12 * (2 .* (Dates .<= 19950601) + 5 .* (Dates .> 19950601) );
w = 0.5 * (1 + sin(2*pi*t./Period));
w2 = w~(1-w);
w = 0.10 + 0.30 * (1 + sin(2*pi*t./Period));
w3 = w~(1-w);
weights = w2 .* (Dates .< 19950601) +
          w1 .* ((Dates .>= 19950601) .and (Dates .<= 20010101)) +
          w3 .* (Dates .> 20010101) ;
R_Portfolio = sumr(R_Index .* weights);
Portfolio_Data = 100 * cumprodc(1 + missrv(R_Portfolio,0));
Data = Index_Data ~ Portfolio_Data;

Data = 100 * Data./Data[1,.];

pqgwin many;

graphset;
  begwind;
  window(2,2,0);

  _pdate = 0; _pframe = 0; _pnum = 2;
  _plwidth = 10; _pltype = 6;
  _paxht = 0.225; _pnumht = 0.225; _ptitlht = 0.30;
  xtics(1,12*28,12*5,1);
  let xlab = "90" "95" "2000" "05" 
             "10" "15" "20";
  asclabel(xlab,0);

  setwind(1);
    _pcolor = 1;
    ytics(0,1000,200,2);
    title("\216MSCI USA");
    xy(0,Data[.,1]);

  setwind(2);
    _pcolor = 2;
    ytics(0,500,100,2);
    title("\216MSCI EMU ($)");
    xy(0,Data[.,2]);

  setwind(3);
    ytics(0,100,20,2);
    _pcolor = 3;
    title("\216MSCI USA allocation (in %)");
    xy(0,100*weights[.,1]);

  setwind(4);
     ytics(0,600,100,2);
    _pcolor = 4;
    title("\216Cumulative performance\L\216of the portfolio");
    xy(0,Data[.,3]);

  endwind;

y = trimr(R_Portfolio,1,0);
x = trimr(R_Index,1,0);

// First estimation

dx = weights - lag1(weights);
dx = packr(dx);
Q = vcx(dx);
H = 0.0;
a0 = 0.5|0.5;
P0 = zeros(2,2);
call TV_Beta_SSM(y,x,a0,P0,H,Q);
x_hat1 = miss(zeros(1,2),0) | KF_matrix(3); 
Q1 = Q;

// Second estimation

Q = diagrv(eye(2),diag(Q));
call TV_Beta_SSM(y,x,a0,P0,H,Q);
x_hat2 = miss(zeros(1,2),0) | KF_matrix(3); 
Q2 = Q;

// Third estimation

proc mlProc3(theta);
  local H,Q,a0,P0,LogL;
  H = theta[1]^2;
  Q = diagrv(eye(2),theta[2:3]^2);
  a0 = 0.5|0.5;
  P0 = zeros(2,2);
  call TV_Beta_SSM(y,x,a0,P0,H,Q);
  LogL = KF_ml;
  retp(LogL);
endp;

sv = 0.1*ones(3,1);
fn mlNegSum3(theta) = -sumc(mlProc3(theta));
{theta,fmin,grd,retcode} = optmum(&mlNegSum3,sv);
H = theta[1]^2;
Q = diagrv(eye(2),theta[2:3]^2);
a0 = 0.5|0.5;
P0 = zeros(2,2);
call TV_Beta_SSM(y,x,a0,P0,H,Q);
x_hat3 = miss(zeros(1,2),0) | KF_matrix(3); 
Q3 = Q;

// Fourth estimation

proc mlProc4(theta);
  local H,Q,a0,P0,LogL;
  H = theta[1]^2;
  Q = theta[2]^2;
  a0 = 0.5;
  P0 = 0;
  call TV_Beta_SSM(y-x[.,2],x[.,1]-x[.,2],a0,P0,H,Q);
  LogL = KF_ml;
  retp(LogL);
endp;

sv = 0.1*ones(2,1);
fn mlNegSum4(theta) = -sumc(mlProc4(theta));
{theta,fmin,grd,retcode} = optmum(&mlNegSum4,sv);
H = theta[1]^2;
Q = theta[2]^2;
a0 = 0.5;
P0 = 0;
call TV_Beta_SSM(y-x[.,2],x[.,1]-x[.,2],a0,P0,H,Q);
x_hat4 = miss(0,0) | KF_matrix(3); 
Q4 = Q;

cls;

print ftosa(1e4*Q1,"%lf",3,5);
print ftosa(1e4*Q2,"%lf",3,5);
print ftosa(1e4*Q3,"%lf",3,5);
print ftosa(1e4*Q4,"%lf",3,5);

graphset;
  begwind;
  window(2,2,0);

  _pdate = 0; _pframe = 0; _pnum = 2;
  _plwidth = 10; _pltype = 6;
  _paxht = 0.225; _pnumht = 0.225; _ptitlht = 0.30;
  xtics(1,12*28,12*5,1);
  let xlab = "90" "95" "2000" "05" 
             "10" "15" "20";
  asclabel(xlab,0);
  ytics(0,500,100,2);

  setwind(1);
    ytics(0,100,20,2);
    _pcolor = 1;
    title("\216Empirical covariance matrix");
    xy(0,100*x_hat1[.,1]);

  setwind(2);
    ytics(0,100,20,2);
    _pcolor = 2;
    title("\216Diagonal covariance matrix");
    xy(0,100*x_hat2[.,1]);

  setwind(3);
    ytics(0,100,20,2);
    _pcolor = 3;
    title("\216Estimated model (2F)");
    xy(0,100*x_hat3[.,1]);

  setwind(4);
    ytics(0,100,20,2);
    _pcolor = 4;
    title("\216Estimated model (1F)");
    xy(0,100*x_hat4[.,1]);

  endwind;


