new;
library frm,optmum,pgraph;
cls;

datPathName = "..\\..\\data\\";
datFileName = datPathName $+ "equity-indices-2017-monthly";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

BeginDate = 19500101;
EndDate = 20191231;
Dates = data[.,1];
Index = data[.,2];

Rt = ln(Index) - ln(lag1(Index));
Rt = selif(Rt, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Dates = selif(Dates, (Dates .>= BeginDate) .and (Dates .<= EndDate));
T = rows(Dates);

/*
**> Ljung-Box test of randomness of residuals
**  H0: "residuals are random" vs. H1: "residuals are not random"
**
*/

print "***********************************";
print "*        Ljung-Box Test           *";
print "***********************************";

v = packr(Rt);
s = 100;
r_k = acf(v,s,0);
Q_Ljung_Box = T * (T+2) * sumc(r_k^2 ./ seqa(T,-1,s) );
Q_pvalue = cdfchic(Q_Ljung_Box,s);

Results = Q_Ljung_Box | Q_pvalue;

v = v^2;
r_k = acf(v,s,0);
Q_Ljung_Box = T * (T+2) * sumc(r_k^2 ./ seqa(T,-1,s) );
Q_pvalue = cdfchic(Q_Ljung_Box,s);

Results = Results ~ (Q_Ljung_Box | Q_pvalue);

print ftosa(Results,"%lf",3,3);

s = 50;
r_k = acf(Rt,s,0);
pr_k = pacf(Rt,s,0); 
r2_k = acf(Rt^2,s,0);
pr2_k = pacf(Rt^2,s,0);

alpha = 0.05;

graphset;
  begwind;
  window(2,2,1);
  
  _pdate = 0; _pframe = 0; _pnum = 2;   
  _pbarwid = 0.4; _pltype = 10; _plwidth = 10;
  _paxht = 0.25;  _ptitlht = 0.30;  _pnumht = 0.25; 

  ytics(-10,15,5,2);
  xtics(-5,55,5,2);
  let xlab = "0" "" "10" "" "20" "" "30" "" "40" "" "50" "";
  asclabel(xlab,0); 
 
  z_alpha = cdfni(1-alpha/2);
  e = 100*z_alpha/sqrt(T);
  _pline = 1~1~-10~e~60~e~1~0~10 |
           1~1~-10~-e~60~-e~1~0~10; 
  xlabel("\216Lag");

  setwind(1);
    title("\216ACF of R]t[ (in %)");
    _pbartyp = 6~1; 
    bar(0,100*r_k);

  setwind(2);
    title("\216PACF of R]t[ (in %)");
    _pbartyp = 6~2; 
    bar(0,100*pr_k);

  setwind(3);
    title("\216ACF of R]t[[2] (in %)");
    _pbartyp = 6~3; 
    bar(0,100*r2_k);

  setwind(4);
    title("\216PACF of R]t[[2] (in %)");
    _pbartyp = 6~4; 
    bar(0,100*pr2_k);

  endwind;
