new;
library cml,optmum,frm,pgraph;
cls;

datPathName = "..\\..\\data\\";
datFileName = datPathName $+ "equity-indices-2017-monthly";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

BeginDate = 19500101;
EndDate = 20191231;
Dates = data[.,1];
Index = data[.,2];

Rt = ln(Index) - ln(lag1(Index));
Rt = selif(Rt, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Dates = selif(Dates, (Dates .>= BeginDate) .and (Dates .<= EndDate));
nDates = rows(Dates);
T = nDates;

// Garch ML
p = 1; 
q = 1;
{theta,cl,cov,Logl} = regGarch(Rt,1,p,q,0);
stderr = sqrt(diag(cov));
tstudent = theta ./ stderr;
nParam = rows(theta);
df = T - nParam;
pvalue = 2*cdftc(abs(tstudent),df);

cls;

cn = T|0|T|nParam|nParam|df|Logl;
parnm = 0 $+ "P" $+ ftocv(seqa(1,1,nParam),2,0);
parnm = "Const" | 
        "alpha0" |
        (0 $+ "gamma" $+ ftocv(seqa(1,1,1),1,0)) |
        (0 $+ "alpha" $+ ftocv(seqa(1,1,q),1,0)) ;
call _reg_print(parnm,theta,stderr,tstudent,pvalue,cn,1,2);

h_t = _garch_h;
sigma_t = sqrt(h_t);
epsilon_t = Rt - theta[1];
e_t = epsilon_t ./ sigma_t;

/*
**> Ljung-Box test of randomness of residuals
**  H0: "residuals are random" vs. H1: "residuals are not random"
**
*/

alpha = 0.05;
order_max = 6;
v = packr(e_t);

Q_Ljung_Box = miss(zeros(order_max,1),0);
Q_pvalue = miss(zeros(order_max,1),0);
s = seqa(1,1,order_max);
i = 1;
do until i > order_max;
  r_k = acf(v,s[i],0);
  pr_k = pacf(v,s[i],0);
  Q_Ljung_Box[i] = T * (T+2) * sumc(r_k^2 ./ seqa(T,-1,s[i]) );
  Q_pvalue[i] = cdfchic(Q_Ljung_Box[i], s[i]);
  i = i + 1;  
endo;

results = s~Q_Ljung_Box~Q_pvalue;
print ftosa(results,"%lf",3,3);

v = v^2;

Q_Ljung_Box = miss(zeros(order_max,1),0);
Q_pvalue = miss(zeros(order_max,1),0);
s = seqa(1,1,order_max);
i = 1;
do until i > order_max;
  r_k = acf(v,s[i],0);
  Q_Ljung_Box[i] = T * (T+2) * sumc(r_k^2 ./ seqa(T,-1,s[i]) );
  Q_pvalue[i] = cdfchic(Q_Ljung_Box[i], s[i]);
  i = i + 1;  
endo;

results = s~Q_Ljung_Box~Q_pvalue;
print ftosa(results,"%lf",3,3);

s = 50;
r_k = acf(v,s,0);
pr_k = pacf(v,s,0);
vol = stdc(Rt)*sqrt(12);
vol_t = sigma_t*sqrt(12);

print;
call dstat(0,100*packr(vol_t));
print; print;
print ftos(100*vol,"Long-run vol = %lf",3,2);
print;

graphset;
  begwind;
  window(2,2,0);

  fonts("simplex simgrma");
  _pdate = 0; _pframe = 0; _pnum = 2;
  _plwidth = 10; _pltype = 6; _pcolor = 1;
  _paxht = 0.25; _ptitlht = 0.30; _pnumht = 0.25; 

  xtics(1,nDates,12*10,1);
  let xlab = "50" "60" "70" "80" "90" "2000" "10" "20";
  asclabel(xlab,0);

  _pcolor = 1;
  setwind(1);
    title("\216\202s\201]t[ (in %)");
    ytics(0,35,5,2);
    _pline = 1~3~0~100*vol~1000~100*vol~1~5~10;
    xy(0,100*vol_t);

  _pcolor = 2;
  nextwind;
    title("\216e]t[");
    ytics(-3,3,1,2);
    xy(0,e_t);

  z_alpha = cdfni(1-alpha/2);
  e = 100 * z_alpha/sqrt(T);
  _pline = 1~1~-10~e~60~e~1~0~10 |
           1~1~-10~-e~60~-e~1~0~10;    
  xlabel("\216Lag");

  ytics(-10,10,5,2);
  xtics(-5,55,5,2);
  let xlab = "0" "" "10" "" "20" "" "30" "" "40" "" "50" "";
  asclabel(xlab,0);
   _pbarwid = 0.4;  

  nextwind;
    _pbartyp = 6~3;
    title("\216ACF of e]t[[2] (in %)");
    bar(0,100*r_k);

  nextwind;
    _pbartyp = 6~4;
    title("\216PACF of e]t[[2] (in %)");
    bar(0,100*pr_k);

  endwind;

results = Dates~vol_t;
save vol_garch_data = results;
