new;
library tsm,frm,optmum,pgraph;

cls;

proc Historical_Volatility(x,Period);
  local n,hvol,i;

  n = rows(x);
  hvol = miss(zeros(n,1),0);
  i = Period + 1;
  do until i > n;
    hvol[i] = stdc(x[i-Period:i]);
    i = i + 1;
  endo;

  retp(hvol);
endp;

declare matrix model = 0;

load vol_garch_data;
vol_garch_dates = vol_garch_data[.,1];
vol_garch = vol_garch_data[.,2];

datPathName = "..\\..\\data\\";
datFileName = datPathName $+ "equity-indices-2017-monthly";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

BeginDate = 19500101;
EndDate = 20191231;
Dates = data[.,1];
Index = data[.,2];

Rt = ln(Index) - ln(lag1(Index));
Rt = selif(Rt, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Dates = selif(Dates, (Dates .>= BeginDate) .and (Dates .<= EndDate));
nDates = rows(Dates);
t_ = seqa(1,1,nDates);

mean_Rt = meanc(Rt);
Rt_c = Rt - mean_Rt;
x = Rt_c^2;
y = ln(x);

vol = sqrt(12) * stdc(Rt);
vol_R0 = sqrt(12) * stdc(Rt_c);
vol_R1 = sqrt(12) * sqrt(meanc(x));

mu_y = meanc(y);
sigma_y = stdc(y);
vol_R2 = sqrt(12) * exp(0.5*mu_y + 0.25*sigma_y^2);

hVol = Historical_Volatility(Rt,12);
hVol = sqrt(12) * hVol;

proc (9) = sv_model(theta);
  local alpha0,gamma1,sigma_eta,sigma2_eta,sigma_zeta;
  local a0,P0,Z,d,H,T,c,R,Q;

  if model == 1;

    alpha0 = theta[1];
    gamma1 = theta[2];
    sigma_eta = theta[3];
    sigma2_eta = sigma_eta^2;

    a0 = alpha0/(1 - gamma1);
    P0 = sigma2_eta/(1 - gamma1^2);
    P0 = 0;

    Z = 1;
    d = digamma(1) - ln(2);
    H = 0.5 * (pi^2);  

    T = gamma1;
    c = alpha0;   
    R = 1;
    Q = sigma2_eta;

  elseif model == 2;

    alpha0 = theta[1];
    gamma1 = theta[2];
    sigma_eta = theta[3];
    sigma2_eta = sigma_eta^2;

    a0 = 0;
    P0 = sigma2_eta/(1 - gamma1^2);
    P0 = 0;

    Z = 1;
    d = alpha0 + digamma(1) - ln(2);
    H = 0.5 * (pi^2);  

    T = gamma1;
    c = 0;   
    R = 1;
    Q = sigma2_eta;

  else;

    alpha0 = theta[1];
    gamma1 = theta[2];
    sigma_eta = theta[3];
    sigma2_eta = sigma_eta^2;
    sigma_zeta = theta[4];

    a0 = 0;
    P0 = sigma2_eta/(1 - gamma1^2);
    P0 = 0;

    Z = 1;
    d = alpha0;
    H = sigma_zeta^2;
  
    T = gamma1;
    c = 0;   
    R = 1;
    Q = sigma2_eta;

  endif;

  retp(a0,P0,Z,d,H,T,c,R,Q);
endp;

proc sv_ml(theta);
  local a0,P0,Z,d,H,T,c,R,Q,Logl;

  {a0,P0,Z,d,H,T,c,R,Q} = sv_model(theta);

  call SSM_build(Z,d,H,T,c,R,Q,0);
  call Kfiltering(y,a0,P0);
  LogL = KF_ml;

  retp(LogL);
endp;

if model == 0;
  sv = 0.1 * ones(4,1);
else;
  sv = 0.5 * ones(3,1);
endif;

{theta,stderr,Mcov,LogL} = TD_ml(&sv_ml,sv);

cls;

print ftosa(100*(meanc(packr(hVol))~vol_R0~vol_R1~vol_R2),"%lf",3,2);

alpha0 = theta[1];
gamma1 = theta[2];
sigma_eta = theta[3];
print ftos(alpha0,"alpha0 = %lf",3,2);
print ftos(gamma1,"gamma1 = %lf",3,2);
print ftos(sigma_eta,"sigma(eta) = %lf",3,2);


{a0,P0,Z,d,H,T,c,R,Q} = sv_model(theta);
call SSM_build(Z,d,H,T,c,R,Q,0);
call Kfiltering(y,a0,P0);

ht_KF = KF_matrix(3);
{ht_KS,ht_KS} = KSmoothing;

if (model == 0) or (model == 1);
  vol_KF = exp(0.5 * ht_KF); 
  vol_KS = exp(0.5 * ht_KS);    
else;
  vol_KF = exp(0.5 * ht_KF); 
  vol_KS = exp(0.5 * ht_KS);
endif;

scaling = vol_R0 / meanc(vol_KF);
vol_KF = scaling * vol_KF;
vol_KS = scaling * vol_KS;

graphset;
  begwind;
  window(2,2,0);

  _pdate = ""; _pframe = 0; _pnum = 2;
  _plwidth = 10; _pltype = 6; _pcolor = 1;
  _ptitlht = 0.30; _pnumht = 0.25; _paxht = 0.25;

  xtics(1,nDates,12*10,1);
  let xlab = "50" "60" "70" "80" "90" "2000" "10" "20";
  asclabel(xlab,0);

  ytics(5,25,5,2);
  _pline = 1~3~0~100*vol~1000~100*vol~1~5~10;

  setwind(1);
    _pcolor = 1;
    title("\216Kalman filtering");
    xy(t_,100*vol_KF);

  setwind(2);
    _pcolor = 2;
    title("\216Kalman smoothing");
    xy(t_,100*vol_KS);

  ytics(5,35,5,2);

  setwind(3);
    _pcolor = 3;
    title("\216GARCH(1,1)");
    xy(t_,100*vol_Garch);

  setwind(4);
    _pcolor = 4;
    title("\21612-month historical volatility");
    xy(t_,100*hVol);

 endwind;