/*
** Griddy Gibbs Sampling
**
*/

new;
library cml,frm,optmum,pgraph;

cls;

#include stochastic_volatility.src

datPathName = "..\\..\\data\\";
datFileName = datPathName $+ "equity-indices-2017-monthly";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

BeginDate = 19500101;
EndDate = 20191231;
Dates = data[.,1];
Index = data[.,2];

Rt = Index ./ lag1(Index) - 1.0;
Rt = selif(Rt, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Dates = selif(Dates, (Dates .>= BeginDate) .and (Dates .<= EndDate));

p = 1; 
q = 1;
{theta_Garch,cl_Garch,cov_Garch,logl_Garch} = regGarch(Rt,1,p,q,0);
h_Garch = _garch_h;

nR = 1000;       // Burn-in sample
nS = 10000;      // Number of simulations

// hyperparameters

// beta
beta0 = 0;
A0 = 9;
invA0 = invpd(A0);

// omega
omega0 = 0|0;
G0 = diagrv(eye(2),100|100);
invG0 = invpd(G0);
nu0 = 10; 
sigma2v = 0.15^2; 
s20 = (nu0-2.0)/nu0 * sigma2v;

// h0
m0 = 0; 
C0 = 10;
invC0 = invpd(C0);

// Storage
nobs = rows(Rt);
N = nR + nS;
h_stored = miss(zeros(nobs,N),0);
beta_stored = miss(zeros(N,1),0);
gamma_stored =  miss(zeros(N,1),0);
phi_stored =  miss(zeros(N,1),0);
sigma2v_stored = miss(zeros(N,1),0);

// initialization
rndseed 12345;
h =  log(packr(h_Garch));
h0 = 0; 
beta = 0;
omega = -0.5|0.8; 

// sampler parameters

nGrid = 500;
gridStep = 0.001;

cls;

t_start = hsec;

i = 1;
do until i > N;

  gamma_ = omega[1];
  phi = omega[2];

  // Sample from h --> Griddy Gibbs sampler
  theta = omega | sigma2v;
  h = SV_GriddyGibbs(h,h0,&condProb_GriddyGibbs,Rt,beta,theta,gridStep,nGrid);
  h1 = h[1];

  // Sample h0
  invC1 = invC0 + phi^2/sigma2v;
  C1 = invpd(invC1);
  m1 = C1 * (invC0 * m0 + phi*(h1-gamma_)/sigma2v);
  h0 = m1 + sqrt(C1) * rndn(1,1);

  // Sample omega and sigma_v^2
  {omega,sigma2v} = SV_Update_Parameters(h,h0,omega0,invG0,nu0,s20);

  gamma_ = omega[1];
  phi = omega[2];

  // sample from beta distribution
  X = exp(-0.5 .* h);
  Rt_star = Rt .* X;
  invA1 = invA0 + X'X;
  A1 = invpd(invA1);
  beta_hat = invpd(X'X)*X'Rt_star;
  beta1 = A1 * ( (X'X) * beta_hat + invA0 * beta0); // beta1 = A1 * ( sumc(exp(-h) .* Rdt) + invA0 * beta0);
  beta1 = A1 * ( sumc(exp(-0.5 * h) .* Rt) + invA0 * beta0); // Pourquoi exp(-0.5*h) ?

  beta = beta1 + sqrt(A1) * rndn(1,1);

  // storage
  h_stored[.,i] = h;
  beta_stored[i] = beta;
  gamma_stored[i] = gamma_;
  phi_stored[i] = phi;
  sigma2v_stored[i] = sigma2v;

  if i%50 == 0;
    t_end = hsec;
    elapsed_time = (t_end - t_start) / 100;
    remaining_time = ((N-i)/i) * elapsed_time;
    ct = ftosa(i,"Iteration = %lf",3,2) $+ ftos(remaining_time," / Computational Time = %lf ",3,2) $+ "seconds";
    print /flush ct;
  endif;

  i = i + 1;
endo;

all_results = vput(0,h_stored,"h");
/*
all_results = vput(all_results,beta_stored,"beta");
all_results = vput(all_results,gamma_stored,"gamma");
all_results = vput(all_results,phi_stored,"phi");
all_results = vput(all_results,sigma2v_stored,"sigma2v");
*/

save chap14-2-4-6a = all_results;

