/*
**> Results MCMC
*/

new;
library pgraph;

cls;

proc Historical_Volatility(x,Period);
  local n,hvol,i;

  n = rows(x);
  hvol = miss(zeros(n,1),0);
  i = Period + 1;
  do until i > n;
    hvol[i] = stdc(x[i-Period:i]);
    i = i + 1;
  endo;

  retp(hvol);
endp;

declare matrix model = 2;

load vol_garch_data;
vol_garch_dates = vol_garch_data[.,1];
vol_garch = vol_garch_data[.,2];

datPathName = "..\\..\\data\\";
datFileName = datPathName $+ "equity-indices-2017-monthly";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

BeginDate = 19500101;
EndDate = 20191231;
Dates = data[.,1];
Index = data[.,2];

Rt = ln(Index) - ln(lag1(Index));
Rt = selif(Rt, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Dates = selif(Dates, (Dates .>= BeginDate) .and (Dates .<= EndDate));
nDates = rows(Dates);

hVol = Historical_Volatility(Rt,12);
vol = sqrt(12) * stdc(Rt);

let string FileNames = "a" "b" "c" "d";

FileNames = "chap14-2-4-6" $+ FileNames;
nModels = rows(FileNames);

nR = 3000;
nS = 8000;

data = zeros(nDates,nModels);

i = 1;
do until i > nModels;
  FileName = FileNames[i];
  loadm all_results = ^FileName;

  h = vread(all_results,"h");
  h = h[.,nR+1:nR+nS];
  sVol = exp(0.5 * h);
  sVol_mean = meanc(sVol');

  data[.,i] = sVol_mean;
  i = i + 1;
endo;

data = hVol~data;

//data[.,1] = lagn(data[.,1],6);
data = 100 * sqrt(12) * data;

graphset;
  begwind;
  window(2,2,0);

  _pdate = ""; _pframe = 0; _pnum = 2;
  _plwidth = 10; _pltype = 6|3; _pcolor = 1|2;
  _ptitlht = 0.30; _pnumht = 0.25; _paxht = 0.25;
  _pstype = 9; _plctrl = 0;

  xtics(1,nDates,12*10,1);
  let xlab = "50" "60" "70" "80" "90" "2000" "10" "20";
  asclabel(xlab,0);

  ytics(5,25,5,2);
  _pline = 1~3~0~100*vol~1000~100*vol~1~5~10;

  setwind(1);
    _pcolor = 1;
    title("\216Griddy Gibbs sampler");
    xy(0,data[.,2]);

  setwind(2);
    _pcolor = 2;
    title("\216Random Walk Metropolis");
    xy(0,data[.,3]);

  setwind(3);
    _pcolor = 3;
    title("\216Metropolis-Hastings");
    xy(0,data[.,4]);

  setwind(4);
    _pcolor = 4;
    title("\216MH within Griddy Gibbs");
    xy(0,data[.,5]);

  endwind;