/*
**  TONG [1990], Non-linear Time Series, Oxford University Press
**
**  PRIESTLEY [1994], Spectral Analysis and Times Series,
**  Academic Press, London
**
**  Lynx data (1821-1934)
**  Tong, page 470
*/

new;
library frm,tsm,optmum,pgraph;

cls;

load lynx[] = ..\..\data\lynx.asc;

y = log(lynx);
Nobs = rows(y);

t = seqa(1821,1,1934-1821+1);

{lambda,I} = PDGM(y); I[1] = 0;
I = I / (2*pi);

q = trunc(rows(lambda)/2);

/*
**  Fisher's g-statistic
**
**  Priestley, page 407, formula 6.1.66
*/

g = maxc(I[2:57])/sumc(I[2:57]);
gstar = 2*Nobs*g;                    /* Priestley, page 407, formula 6.1.64 */
pvalue = Nobs*exp(-gstar/2);         /* Priestley, page 407, formula 6.1.65 */

print "Test for Periodogram Ordinates";
print chrs(45*ones(60,1)); print;
s = 1+maxindc(I[2:57]);
print ftos(lambda[s],"Test for periodic component at the frequency %lf",4,4);
print;
print ftos(g,     "Fisher's g-statistic : %lf",6,5);
print ftos(gstar, "g* :                   %lf",6,5);
print ftos(pvalue,"p-value :              %lf",6,5);

{s,g,pvalue_g,gstar,pvalue_gstar} = spectral_cycle_gtest(lambda,I);

print "Test for Periodogram Ordinates";
print chrs(45*ones(60,1)); print;
print ftos(lambda[s],"Test for periodic component at the frequency %lf",4,4);
print;
print ftos(g,     "Fisher's g-statistic : %lf",6,5);
print ftos(pvalue,"p-value (g) :          %lf",6,5);
print ftos(gstar, "g* :                   %lf",6,5);
print ftos(pvalue_gstar,"p-value (g*) :         %lf",6,5);

d = fourier(y);
n = rows(d);
d_cycle = zeros(n,1);
d_cycle[13] = d[13];
d_cycle[103] = d[103];
y_c = real(inverse_fourier(d_cycle)) + meanc(y);
p = 2*pi/lambda[s];

graphset;               /* Priestley, page 412 */
  begwind;
  window(2,2,0);
  
  _pdate = ""; _pnum = 2;
  fonts("simplex simgrma");
  _plwidth = 10; _pltype = 6; _pframe = 0;
  _pnumht = 0.225; _paxht = 0.25; _ptitlht = 0.30;

  _pcolor = 1;
  setwind(1);
    title("\216Time series y]t[");
    xlabel("\216Year");
    xtics(1820,1940,20,2);
    let xlab = "1820" "40" "60" "80" "1900" "20" "40";
    asclabel(xlab,0);
    ytics(1.5,4,0.5,2);
    xy(t,y);

  _pnumht = 0.25;
  _pcolor = 4;
  setwind(2);
    title("\216Periodogram");
    xtics(0,pi,pi/4,0);
    ytics(0,2,0.5,2);
    lab = " 0 \202p\201/4 \202p\201/2 \2013\202p\201/4 \202p\201";
    asclabel(lab,0);
    xlabel("\216\202l\201");
    xy(lambda[1:q],I[1:q]);

  xtics(1820,1940,20,2);
  let xlab = "1820" "40" "60" "80" "1900" "20" "40";
  asclabel(xlab,0);
  _pmsgctl = 0 ; 
  _pnumht = 0.225;

  _pcolor = 2; _pltype = 6;
  setwind(3);
    title("\216Cycle");
    xlabel("\216Year");
    ytics(2.2,3.7,0.3,2);
    xy(t,y_c);

  _pcolor = 3; _pltype = 3;
  setwind(4);
    title("\216Residual");
    xtics(1820,1940,20,2);
    ytics(-1,1,0.5,2);
    xlabel("\216Year");
    xy(t,y-y_c);

  endwind;

proc (1) = scr(theta);
  local m_z,omega,a,z;

  m_z = sqrt(theta[1]^2);
  omega = theta[2];
  a = sqrt(theta[3]^2);
  z = m_z + sin(2*pi*t/p + omega)*a;

  retp(sumc((z-y_c)^2));
endp;

__output = 0;
let sv = 3 -1 0.60;
{theta,fmin,grd,retcode} = optmum(&scr,sv);
m_z = sqrt(theta[1]^2);
omega = theta[2];
a = sqrt(theta[3]^2);
z = m_z + sin(2*pi*t/p + omega)*a;

print ftosa(m_z|p|omega|a,"%lf",3,3);
