new;
library frm,pgraph,optmum;

cls;

/*
**> Truncated Normal Distribution
**
*/

rndseed 123;

n = 1000000;
nS = 1;

mu = 2;
sigma = 3;
c = 1;

alpha = (c - mu) ./ sigma;
Phi = cdfn(alpha);
lambda = pdfn(alpha) ./ (1 - Phi);
delta = lambda .* (lambda - alpha);

U = rndu(n,nS);
X = mu + sigma .* cdfni(Phi + U .* (1 - Phi));

pqgwin many;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pbartyp = {6 1};
  xtics(0,15,1,0);
  xlabel(" "); ylabel(" ");
  call histp(X,50);

empirical_moments = meanc(X)~meanc(X^2)~stdc(X);

m1 = mu + sigma .* lambda;
m2 = mu^2 + 2 * mu .* sigma .* lambda + (sigma^2) .* (1 + alpha .* lambda);
m3 = sigma .* sqrt(1 - delta);

theoretical_moments = m1'~m2'~m3';

print "Truncated case:";
print ftosa(theoretical_moments,"%lf",4,5);
print ftosa(empirical_moments,"%lf",4,5);
print "--------------------------------------";

/*
**> Censured Normal Distribution
**
*/

alpha = (c - mu) ./ sigma;

X = mu + sigma .* rndn(n,ns);
cnd = (X .> c);
Y = X .* cnd + c .* (1 - cnd);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pbartyp = {6 4};
  xtics(0,15,1,0);
  xlabel(" "); ylabel(" ");
  call histp(Y,50);

empirical_moments = meanc(Y)~meanc(Y^2)~stdc(Y);

m1 = Phi .* c +  (1 - Phi) .* (mu + sigma .* lambda);
m2 = Phi .* (c^2) + (1 - Phi) .* mu^2 + 2 .* mu .* sigma .* pdfn(alpha) + 
     (sigma^2) .* (1 - Phi + alpha .* pdfn(alpha));
m1_sqr = Phi^2 .* c^2 +  (1 - Phi)^2 .* mu^2 + 
         2 * mu .* sigma .* (1 - Phi) .* pdfn(alpha) +
         sigma^2 .* pdfn(alpha)^2 + 2 .* Phi .* (1 - Phi) .* c .* mu + 
         2 .* Phi .* c .* sigma .* pdfn(alpha);

// First expression of the variance
m3 = sqrt(m2-m1^2);

// Second expression of the variance
m3 = Phi .* (1 - Phi) .* c^2 + Phi .* (1 - Phi) .* mu^2 + 
     2 * Phi .* pdfn(alpha) .* mu .* sigma + 
     pdfn(alpha) .* (alpha - pdfn(alpha)) .* sigma^2 + (1 - Phi) .* sigma^2 - 
     2 * Phi .* (1 - Phi) .* c .* mu - 2 * Phi .* pdfn(alpha) .* c .* sigma ;
m3 = sqrt(m3);

// third expression of the variance
m3 = Phi .* (1 - Phi) .* alpha^2  .* sigma^2 - 2 * Phi .* pdfn(alpha) .* alpha .* sigma^2 + 
     sigma^2 .* (1 - Phi) .* (1 + pdfn(alpha) .* (alpha - pdfn(alpha)) ./ (1 - Phi) );
m3 = Phi .* (1 - Phi) .* alpha^2  .* sigma^2 - 2 * Phi .* pdfn(alpha) .* alpha .* sigma^2 + 
     sigma^2 .* (1 - Phi) .* (1 + lambda .* (alpha - pdfn(alpha)) );
m3 = sigma^2 .* (1 - Phi) .* ( ( 1 -delta) + (alpha - lambda)^2 .* Phi );
m3 = sqrt(m3);

theoretical_moments = m1'~m2'~m3';

print "Censored case:";
print ftosa(theoretical_moments,"%lf",4,5);
print ftosa(empirical_moments,"%lf",4,5);
print "--------------------------------------";

/*
Truncated case:
         3.79547         18.38641          1.99520 
         3.79588         18.39650          1.99695 
--------------------------------------
Censored case:
         2.76271         11.96315          2.08101 
         2.76213         11.96248          2.08161 
--------------------------------------
*/

