new;
library frm,optmum,pgraph;
frmSet;

declare matrix _order_n,_order_i;
declare matrix _order_mu,_order_sigma,_order_nu,_order_data;

datFileName = "..\\..\\data\\benchmark";
Data = loadd(datFileName);
Dates = Data[.,1];
data = selif(data,(Dates .>= 19950101) .and (Dates .<= 20151231));

vnames = "" $+ getname(datFileName);
let string selected_names = "M2EF" "M2US";

indx = indsav(selected_names,vnames);
data = data[.,indx];

dt = 1;
Rt = packr(data ./ lagn(data,dt) - 1.0);

Rt = Rt[.,2];

proc cdfProc(x);
  local mu,sigma,nu,xc,cdf;
  mu = _order_mu;
  sigma = _order_sigma;
  nu = _order_nu; 
  xc = (x - mu) / sigma;
  if nu == 0;
    cdf = cdfn(xc);
  else;
    cdf = cdft(xc,nu);
  endif;
  retp(cdf);
endp;

proc pdfProc(x);
  local mu,sigma,nu,xc,pdf;
  mu = _order_mu;
  sigma = _order_sigma;
  nu = _order_nu; 
  xc = (x - mu) / sigma;
  if nu == 0;
    pdf = pdfn(xc)/sigma;
  else;
    pdf = pdft(xc,nu)/sigma;
  endif;
  retp(pdf);
endp;

proc mlProc(theta,data);
  local mu,sigma,pdf;
  _order_mu = theta[1];
  _order_sigma = sqrt(theta[2]^2);
  pdf = pdfOrderStatistic(data,_order_n,_order_i,&cdfProc,&pdfProc);
  retp( ln(missex(pdf,pdf .<= 0) ) );
endp;

fn negLogL(theta) = -sumc(mlProc(theta,_order_data));

let nu = 0 6 1;
nMax = 10;
mu = miss(zeros(nMax,nMax),0);
sigma = miss(zeros(nMax,nMax),0);

k = 1;
do until k > rows(nu);

  _order_nu = nu[k];
  if _order_nu == 0;
    sv = meanc(Rt)|stdc(Rt);
  elseif _order_nu <= 2;
    sv = meanc(Rt)|stdc(Rt)/sqrt(3); 
  else;
   sv = meanc(Rt)|stdc(Rt)/sqrt(_order_nu/(_order_nu-2));
  endif;
  
  n = 1;
  do until n > nMax;
    _order_n = n;
    i = 1;  
    do until i > n; 
      _order_i = i;
 
      str = ftos(_order_nu,"nu = %lf",1,0) $+
            ftos(_order_n,"   --- n = %lf",1,0) $+
            ftos(_order_i,",i = %lf",1,0);
      print /flush str;

      _order_data = Ordering_Data(Rt,i,n);
      {theta,fmin,grd,retcode} = optmum(&negLogL,sv);

      mu[n,i] = theta[1];
      sigma[n,i] = sqrt(theta[2]^2);

      i = i + 1;
    endo;
    n = n + 1;
  endo;
  call varput(mu,ftos(k,"mu%lf",1,0));
  call varput(sigma,ftos(k,"sigma%lf",1,0));
  k = k + 1;
endo;

cls;

print "----------------------------------------------------------------------";
print ftosa(1e4*varget("sigma1"),"%lf",3,0);
print "----------------------------------------------------------------------";
print ftosa(1e4*varget("sigma2"),"%lf",3,0);
print "----------------------------------------------------------------------";
print ftosa(1e4*varget("sigma3"),"%lf",3,0);
print "----------------------------------------------------------------------";
print ftosa(1e4*submat(varget("sigma1"),10,0),"%lf",3,1);
print ftosa(1e4*submat(varget("sigma2"),10,0),"%lf",3,1);
print ftosa(1e4*submat(varget("sigma3"),10,0),"%lf",3,1);

sigma = submat(varget("sigma3"),10,0) |
        submat(varget("sigma2"),10,0) |
        submat(varget("sigma1"),10,0) ;
sigma = sqrt(260) * sigma';
sigma[.,2] = sigma[.,2] * sqrt(6/4);
ratio = meanc(sigma[.,3])/meanc(sigma[.,1]);
sigma[.,1] = sigma[.,1] * ratio;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pframe = 0; _pnum = 2;
  _pltype = 6|1|3; _plwidth = 10; _pcolor = 1|2|4;
  _plctrl = 1; _pstype = 8|9|10; _psymsiz = 4.5;
  xtics(0,11,1,1);
  let lab = "" "1" "2" "3" "4" "5" "6" "7" "8" "9" "10" "";
  asclabel(lab,0);
  ytics(10,30,5,2);
  xlabel("\216i");
  _plegstr = "t]1[\000t]6[\000t]\202\56\201[";
  _plegctl = {2 5 2.25 1.30};
  xy(0,100*sigma);