new;
library frm,pgraph,optmum;

n = 5;
let sigma = 0.20 0.20 0.20 0.20 0.20;
let S0    = 100  100  100  100  100;
r = 0.05;
tau = 3/12;

K = 5; L = 105;

proc Payoff(S);
  local S1,S2,S3,S4,S5,payoff;

  S1 = S[.,1]; S2 = S[.,2]; S3 = S[.,3]; S4 = S[.,4]; S5 = S[.,5];
  payoff = (S1 - S2 + S3 - S4 - K) .* (S5 .> L);
  payoff = payoff .* (payoff .>= 0);

  retp( PayOff );
endp;

rndseed 123;
nS = 10000|50;
nC = 29;

dRho = -1/(n-1)+1;
rho = seqa(-1/(n-1)+1e-3,dRho/(nC-1),nC);
rho[nC] = 1;

nIters = 3;
option_price = miss(zeros(nC,nIters),0);
max_option_price = zeros(nIters,1);
all_C = zeros(nC,n*(n+1)/2);
max_C = zeros(nIters,n*(n+1)/2);

iter = 1;
do until iter > nIters;

  i = 1;
  do until i > nC;

    if iter == 1;
      C = Constant_Correlation_Matrix(n,rho[i]);
      if C == error(0);
        i = i + 1;
        continue;
      endif;
      all_C[i,.] = vech(C)';
    elseif iter == 2;
      lambda = Simulate_Eigenvalues(n,0,0,0);
      C = Simulate_Correlation_Matrix(n,lambda,0);
      all_C[i,.] = vech(C)';
    elseif iter == 3;
      lambda = Simulate_Eigenvalues(n,0,0,3);
      C = Simulate_Correlation_Matrix(n,lambda,0);
      all_C[i,.] = vech(C)';
    endif;

    tmp = zeros(nS[2],1);
    j = 1;
    do until j > nS[2];
      S = simulate_mGBM_T(S0,r,sigma,C,tau,nS[1]);  
      tmp[j] = exp(-r*tau)*meanc(Payoff(S));
      j = j + 1;
    endo;

    option_price[i,iter] = meanc(tmp);

    i = i + 1;
  endo;

  indx = maxindc(option_price[.,iter]);
  max_option_price[iter] = option_price[indx,iter];
  max_C[iter,.] = all_C[indx,.];

  iter = iter + 1;
endo;

cls;
iter = 1;
do until iter > nIters;
  print ftosa(xpnd(max_C[iter,.]'),"%lf",1,4);
  print; 
  print ftos(max_option_price[iter],"                 Option price = %lf",5,3);
  iter = iter + 1;
endo;

graphset;
  window3u;

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pcolor = 1; _plwidth = 10; _pltype = 6;
  _paxht = 0.225; _pnumht = 0.225; _ptitlht = 0.275;

  setwind(1);
   title("\216Constant correlation matrix");
    xlabel("\216Uniform correlation \202r\201 (in %)");
    xtics(-25,100,25,2);
    ytics(0,2.5,0.5,2);
    xy(100*rho,option_price[.,1]);

  _pline = 1~1~0~max_option_price[1]~nC~max_option_price[1]~1~0~10;

  setwind(2);
   title("\216Random correlation matrix\L\216Uniform distributed eigenvalues");
    _pbartyp = {6 4};
    xlabel("\216Simulation");
    xtics(0,nC,5,0);
    ytics(0,6,1.00,2);
    bar(0,option_price[.,2]);

  setwind(3);
   title("\216Random correlation matrix\L\216Geometrically distributed eigenvalues");
    _pbartyp = {6 4};
    xlabel("\216Simulation");
    xtics(0,nC,5,0);
    bar(0,option_price[.,3]);

  endwind;