new;
library frm,pgraph;

cls;

rndseed 123456;

fn pdf(x,mu,sigma) = pdfn((x-mu)./sigma)./(sigma .* x);

tau = 2;
S0 = 100;
r = 0.05;
sigma = 0.20;
K = 60;

nS = 1000;
nR = 50000;

_option_type = "put";
P_BS = EuropeanBS(S0,K,sigma,tau,r,r);
P_MC = zeros(nR,1);
P_IS1 = zeros(nR,1);
P_IS2 = zeros(nR,1);

mu_X = ln(S0) + (r - 0.5*sigma^2)*tau;
sigma_X = sigma*sqrt(tau);
theta = ln(K/S0) - (r - 0.5*sigma^2)*tau;
mu_Z1 = theta + mu_X;
sigma_Z1 = sigma*sqrt(tau);
S1 = S0 * exp(theta);

S2 = 80;
mu_Z2 = ln(S2) + (r - 0.5*sigma^2)*tau;
sigma_Z2 = sigma*sqrt(tau);

iter = 1;
do until iter > nR;

  X = simulate_GBM(S0,r,sigma,tau,nS)';
  Y = exp(-r*tau) * _max_(K - X,0);
  P_MC[iter] = meanc(Y);

  Z = simulate_GBM(S1,r,sigma,tau,nS)';
  Y = exp(-r*tau) * _max_(K - Z,0);
  W = pdf(ln(Z),mu_X,sigma_X) ./ pdf(ln(Z),mu_Z1,sigma_Z1);
  P_IS1[iter] = meanc(W .* Y);

  Z = simulate_GBM(S2,r,sigma,tau,nS)';
  Y = exp(-r*tau) * _max_(K - Z,0);
  W = pdf(ln(Z),mu_X,sigma_X) ./ pdf(ln(Z),mu_Z2,sigma_Z2);
  P_IS2[iter] = meanc(W .* Y);
 
  iter = iter + 1;
endo;

P = P_MC~P_IS1~P_IS2;
x_P = seqa(0,0.25/500,501);
d_P = zeros(rows(x_P),3);
i = 1;
do until i > 3;
  {d_P[.,i],F} = regKernelDensity(P[.,i],x_P);
  i = i + 1;
endo;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|3|1; _pcolor = 1|4|2; _plwidth = 10;
  xlabel("\216Option price");
  _pline = 1~1~P_BS~0~P_BS~100~1~0~10;
  xtics(0.04,0.20,0.02,2);
  ytics(0,100,25,2);
  _plegstr = "MC\000IS (S[']]0[ = S]0[e[\202t\201])\000IS (S[']]0[ = 80)";
  _plegctl = {2 5 6 5};
  xy(x_P,d_P);

ratio = stdc(P)/stdc(P[.,1]);
ratio = ratio^2;
print ftosa(100*ratio,"%lf",3,2);

