new;
library frm,pgraph;

cls;

fn phi1(x) = x;
fn phi2(x) = x^2;
fn phi3(x) = (1+cos(pi*x))/2;

x = seqa(0,1/100,101);
y = phi1(x)~phi2(x)~phi3(x);

m = 10;
j = seqa(1,1,m)/m;
e = ones(m,1);

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3; _pcolor = 1|2|4; _plwidth = 10;
  xtics(0,1,0.2,2);
  ytics(0,1,0.2,2);
  _plegstr = "\202f\201(x) = x\000\202f\201(x) = x[2]\000\202f\201(x) = (1+cos(\202p\201x))/2";
  _plegctl = {2 5 3.3 5.5};
  _pline = e.*(1~3)~j~e*0~j~e*(1~1~0~5);
  xy(x,y);

nS = 1e4;
nR = 10000;

I_MC = zeros(nR,3);
var_MC = zeros(nR,3);

iter = 1;
do until iter > nR;
  X = rndu(nS,1);

  Y1 = phi1(X);
  Y2 = phi2(X);
  Y3 = phi3(X);
  Y = Y1~Y2~Y3;

  I_MC[iter,.] = meanc(Y)';
  var_MC[iter,.] = stdc(Y)';
  
  iter = iter + 1;
endo;
  
var_MC = var_MC^2;
var2_MC = nS * (stdc(I_MC)^2);

results = meanc(I_MC)'|
          meanc(var_MC)'|
          var2_MC';
          
print ftosa(results,"%lf",3,5);
print /flush "---------------------------------------------------";

nS_j = nS * ones(m,1) / m;

mu1_MC = zeros(nR,m);
mu2_MC = zeros(nR,m);
mu3_MC = zeros(nR,m);

var1_strata = zeros(nR,m);
var2_strata = zeros(nR,m);
var3_strata = zeros(nR,m);

iter = 1;
do until iter > nR;    

  j = 1;
  do until j > m;
  
    X = (j-1)/m + rndu(nS_j[j],1)/m;
     
    Y1 = phi1(X);
    Y2 = phi2(X);
    Y3 = phi3(X);

    mu1_MC[iter,j] = meanc(Y1);
    mu2_MC[iter,j] = meanc(Y2);
    mu3_MC[iter,j] = meanc(Y3);      
    
    var1_strata[iter,j] = stdc(Y1)^2;
    var2_strata[iter,j] = stdc(Y2)^2;
    var3_strata[iter,j] = stdc(Y3)^2;

    j = j + 1;
  endo;
  
  iter = iter + 1;
endo;

I1_MC = meanc(mu1_MC');
I2_MC = meanc(mu2_MC');  
I3_MC = meanc(mu3_MC');  

I_MC = I1_MC~I2_MC~I3_MC;

var_MC = nS * (stdc(I_MC)^2);

results = meanc(I_MC)'|
          var_MC';

print ftosa(results,"%lf",3,5);
print /flush "---------------------------------------------------";

var_strata = meanc(var1_strata)~meanc(var2_strata)~meanc(var3_strata);
theoretical_sigma2 = ones(m,1)/(12*m^2);

results = seqa(1,1,m)~1e4*(var_strata~theoretical_sigma2);
print ftosa(results,"%lf",3,0~5~5~5~5);
print /flush "---------------------------------------------------";

graphset;
  window3u;

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pnumht = 0.225; _paxht = 0.225; _ptitlht = 0.30;
  xlabel("\216Stratum j");

  setwind(1);
    title("\216\202f\201(x) = x");
    ytics(0,10,2,2);
    _pbartyp = {6 1};
    bar(0,1e4*var_strata[.,1]);

  setwind(2);
    title("\216\202f\201(x) = x[2]");
    ytics(0,30,5,2);
    _pbartyp = {6 2};
    bar(0,1e4*var_strata[.,2]);

  setwind(3);
    title("\216\202f\201(x) = (1+cos(\202p\201x))/2");
    ytics(0,20,4,2);
    _pbartyp = {6 4};
    bar(0,1e4*var_strata[.,3]);

  endwind;

sigma = sqrt(var_strata);
p = ones(m,1)/m;
q = p .* sigma;
q = q ./ sumc(q)';

nS_j = trunc(nS * q); 

iter = 1;
do until iter > nR;    

  j = 1;
  do until j > m;
  
    X = (j-1)/m + rndu(nS_j[j,1],1)/m;
    Y1 = phi1(X);
    X = (j-1)/m + rndu(nS_j[j,2],1)/m;
    Y2 = phi2(X);
    X = (j-1)/m + rndu(nS_j[j,3],1)/m;
    Y3 = phi3(X);

    mu1_MC[iter,j] = meanc(Y1);
    mu2_MC[iter,j] = meanc(Y2);
    mu3_MC[iter,j] = meanc(Y3);      
    
    var1_strata[iter,j] = stdc(Y1)^2;
    var2_strata[iter,j] = stdc(Y2)^2;
    var3_strata[iter,j] = stdc(Y3)^2;

    j = j + 1;
  endo;
  
  iter = iter + 1;
endo;

I1_MC = meanc(mu1_MC');
I2_MC = meanc(mu2_MC');
I3_MC = meanc(mu3_MC');  

I_MC = I1_MC~I2_MC~I3_MC;

var_MC = nS * (stdc(I_MC)^2);

results = meanc(I_MC)'|
          var_MC';

print ftosa(results,"%lf",3,5);
print /flush "---------------------------------------------------";

var_strata = meanc(var1_strata)~meanc(var2_strata)~meanc(var3_strata);

results = seqa(1,1,m)~1e2*q;
print ftosa(results,"%lf",3,0~5~5~5);
print /flush "---------------------------------------------------";

graphset;
  window3u;

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pnumht = 0.225; _paxht = 0.225; _ptitlht = 0.30;
  xlabel("\216Stratum j");

  ytics(0,20,4,2);

  setwind(1);
    title("\216\202f\201(x) = x");
    _pbartyp = {6 1};
    bar(0,1e2*q[.,1]);

  setwind(2);
    title("\216\202f\201(x) = x[2]");
    _pbartyp = {6 2};
    bar(0,1e2*q[.,2]);

  setwind(3);
    title("\216\202f\201(x) = (1+cos(\202p\201x))/2");
    _pbartyp = {6 4};
    bar(0,1e2*q[.,3]);

  endwind;

end;


/*
         0.50002          0.33335          0.84146 
         0.08334          0.08889          0.01925 
         0.08407          0.08953          0.01939 
---------------------------------------------------

         0.50000          0.33334          0.84147 
         0.00084          0.00110          0.00023 
---------------------------------------------------

               1          8.33377          0.08889          0.02219          8.33333 
               2          8.33438          0.75565          0.18738          8.33333 
               3          8.33322          2.08890          0.51112          8.33333 
               4          8.33253          4.08852          0.98047          8.33333 
               5          8.33429          6.75635          1.57715          8.33333 
               6          8.33485         10.09083          2.27699          8.33333 
               7          8.33287         14.08816          3.05128          8.33333 
               8          8.33287         18.75459          3.87053          8.33333 
               9          8.33312         24.08856          4.70169          8.33333 
              10          8.33368         30.09002          5.51163          8.33333 
---------------------------------------------------

         0.50000          0.33333          0.84147 
         0.00084          0.00083          0.00018 
---------------------------------------------------

               1         10.00013          1.03207          1.12157 
               2         10.00049          3.00912          3.25915 
               3          9.99980          5.00310          5.38271 
               4          9.99938          6.99944          7.45513 
               5         10.00044          8.99779          9.45527 
               6         10.00077         10.99622         11.36104 
               7          9.99958         12.99293         13.15161 
               8          9.99959         14.99111         14.81232 
               9          9.99974         16.98969         16.32544 
              10         10.00007         18.98853         17.67575 
---------------------------------------------------
*/





