new;
library frm,pgraph;

cls;

rndseed 123;

mIters = 10;
var_STR = zeros(mIters,2);

S0 = 100;
K = 105;
sigma = 0.20;
tau = 1;
b = 0.05;
r = 0.05;
C0 = EuropeanBS(S0,K,sigma,tau,b,r);

print /flush "----------------------- C0 ----------------------------";
print ftosa(C0,"%lf",3,5);

print /flush "----------------------- MC ----------------------------";

// MC

nS = 1e4;
nR = 10000;

I_MC = zeros(nR,1);
var_MC = zeros(nR,1);

iter = 1;
do until iter > nR;
  X = rndn(nS,1);
  sTau = S0 * exp( (r-0.5*sigma^2)*tau + sigma * sqrt(tau) * X);
  Y = exp(-r*tau) * _max_(0,Stau-K);
  I_MC[iter] = meanc(Y); 
  iter = iter + 1;
endo;
  
var_I_MC = stdc(I_MC)^2;
var_STR[1,1] = var_I_MC;
var_STR[1,2] = var_I_MC;
results = meanc(I_MC)~var_I_MC;
          
print ftosa(results,"%lf",3,5);
print /flush "----------------------- STR (m = 2) ----------------------------";

x_1 = (ln(K/S0) - r*tau)/(sigma*sqrt(tau)) + 0.5 * sigma * sqrt(tau);
p_1 = cdfn(x_1);
p_2 = 1 - p_1;
nS_1 = trunc(nS * p_1);
nS_2 = trunc(nS * p_2);

mu_MC = zeros(nR,2);

iter = 1;
do until iter > nR;    

  // Case X <= x_1
  mu_MC[iter,1] = 0.0;

  // Case X >= x_1
  U = rndu(nS_2,1);
  X = cdfni(p_1 + p_2 * U);
  sTau = S0 * exp( (r-0.5*sigma^2)*tau + sigma * sqrt(tau) * X);
  Y = exp(-r*tau) * _max_(0,Stau-K);
  mu_MC[iter,2] = meanc(Y);
  
  iter = iter + 1;
endo;

I_STR = p_1 * mu_MC[.,1] + p_2 * mu_MC[.,2];
var_I_STR = stdc(I_STR)^2;
results = meanc(I_STR)~var_I_STR;
          
print ftosa(results,"%lf",3,5);
print /flush "----------------------- STR (m = 5) ----------------------------";

let d = 0.0 1.0 2.0 5.0 1000;
x_strata = (ln((K+d)/S0) - r*tau)/(sigma*sqrt(tau)) + 0.5 * sigma * sqrt(tau);
p_strata = cdfn(x_strata);
p_j = p_strata - lag1(p_strata);
p_j[1] = p_strata[1];
m = rows(p_j);
nS_j = trunc(nS * p_j);

mu_MC = zeros(NR,m);

iter = 1;
do until iter > nR;    

  j = 2;
  do until j > m;
  
    U = rndu(nS_j[j],1);
    X = cdfni(p_strata[j-1] + (p_strata[j]-p_strata[j-1]) * U);
    sTau = S0 * exp( (r-0.5*sigma^2)*tau + sigma * sqrt(tau) * X);
    Y = exp(-r*tau) * _max_(0,Stau-K);
    mu_MC[iter,j] = meanc(Y);

    j = j + 1;
  endo;
  
  iter = iter + 1;
endo;

I_STR = sumc(mu_MC' .* p_j);
var_I_STR = stdc(I_STR)^2;
results = meanc(I_STR)~var_I_STR;
          
print ftosa(results,"%lf",3,5);
print /flush "----------------------- STR (m) ----------------------------";

m = 2;
do until m > mIters;

  p_j = 1/m * ones(m,1);
  nS_j = trunc(nS * p_j);
  
  mu_MC = zeros(NR,m);

  iter = 1;
  do until iter > nR;    

    j = 1;
    do until j > m;
  
      U = rndu(nS_j[j],1);
      X = cdfni( (j-1)/m + U/m );
      sTau = S0 * exp( (r-0.5*sigma^2)*tau + sigma * sqrt(tau) * X);
      Y = exp(-r*tau) * _max_(0,Stau-K);
      mu_MC[iter,j] = meanc(Y);

      j = j + 1;
    endo;
  
    iter = iter + 1;
  endo;

  I_STR = sumc(mu_MC' .* p_j);
  var_I_STR = stdc(I_STR)^2;
  var_STR[m,1] = var_I_STR;
  results = m~meanc(I_STR)~var_I_STR;

  print /flush ftosa(results,"%lf",3,0~5~5);

  m = m + 1;
endo;

m = 2;
do until m > mIters;

  x_1 = (ln(K/S0) - r*tau)/(sigma*sqrt(tau)) + 0.5 * sigma * sqrt(tau);
  p_1 = cdfn(x_1);

  j = seqa(2,1,m-1);
  x_j = cdfni(p_1 + (j-1)/(m-1) * (1 - p_1));
  x_strata = x_1 | x_j;
  p_strata = cdfn(x_strata);
  p_j = p_strata - lag1(p_strata);
  p_j[1] = p_strata[1];
  nS_j = trunc(nS * p_j);
  
  mu_MC = zeros(NR,m);

  iter = 1;
  do until iter > nR;    

    j = 2;
    do until j > m;
  
      U = rndu(nS_j[j],1);
      X = cdfni(p_strata[j-1] + (p_strata[j]-p_strata[j-1]) * U);
      sTau = S0 * exp( (r-0.5*sigma^2)*tau + sigma * sqrt(tau) * X);
      Y = exp(-r*tau) * _max_(0,Stau-K);
      mu_MC[iter,j] = meanc(Y);

      j = j + 1;
    endo;
  
    iter = iter + 1;
  endo;

  I_STR = sumc(mu_MC' .* p_j);
  var_I_STR = stdc(I_STR)^2;
  var_STR[m,2] = var_I_STR;
  results = m~meanc(I_STR)~var_I_STR;

  print /flush ftosa(results,"%lf",3,0~5~5);

  m = m + 1;
endo;

save var_STR;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  _pbartyp = {6 1,6 4}; _pbarwid = -0.5;
  xtics(0,11,1,2);
  xlabel("\216m");
  ytics(0,180,20,2);
  _plegstr = "I[(1)]]STR[(m)\000I[(2)]]STR[(m)";
  _plegctl = {2 6 7 5};
  bar(seqa(1,1,mIters),1e4*var_STR);