new;
library frm,pgraph;

proc (1) = _GeometricAsianBS(t);
  local nt,m,t_sum,t_bar,v;

  nt = rows(t);
  m = seqa(1,1,nt);

  t_sum = sumc(t);
  t_bar = t_sum / nt;
  
  v = t_sum + 2 * sumc( (nt - m) .* t);
  v = v / (nt^2);

  retp(v);
endp;

proc GeometricAsianBS(S0,K,sigma,t,b,r);
  local tau,t_bar,upsilon,w,gamma_,m,d,d1,d2,CallPut;

  tau = maxc(t);
  t_bar = meanc(t);
  upsilon = _GeometricAsianBS(t);
  w = sigma .* sqrt(upsilon);

  gamma_ = b*t_bar + 0.5 * sigma^2 * (upsilon - t_bar);
  m = (b - 0.5*sigma^2) * t_bar;
  d = ( ln(S0 ./ K) + m ) ./ w;  
  d1 = d + w;
  d2 = d;
  _option_d1 = d1;
  _option_d2 = d2;

  if lower(_Option_Type) $== "put";
    CallPut = -S0 .* exp(gamma_ - r.*tau) .* cdfn(-d1) + K .*exp(-r.*tau) .* cdfn(-d2);
  else;
    CallPut = S0 .* exp(gamma_ - r.*tau) .* cdfn(d1) - K .* exp(-r.*tau) .* cdfn(d2);
  endif;

  retp(CallPut);
endp;

rndseed 123456;

tau = 5;
nT = tau;
t = seqa(1,1,nT);

S0 = 100;
r = 0.05;
sigma = 0.20;
K = 104;

nS = 1000;
nR = 10000;
Cst = ones(nS,1);

C_Asian_MC = zeros(nR,1);
C_Asian_CV1 = zeros(nR,1);
C_Asian_CV2 = zeros(nR,1);
C_Asian_CV3 = zeros(nR,1);
C_Asian_CV4 = zeros(nR,1);

iter = 1;
do until iter > nR;

  St = simulate_GBM(S0,r,sigma,t,nS);
  S_tau = St[nT,.]';
  S_bar = meanc(St);
  S_tilde = prodc(St)^(1/nT);
  t_bar = meanc(t);

  Y = exp(-r*tau) * _max_(S_bar - K,0);
  C_Asian_MC[iter] = meanc(Y);

  V1 = S_tau;
  E1 = S0*exp(r*tau);
  beta1 = Y / (Cst~V1);
  beta1 = beta1[2];
  Z1 = Y - beta1 * (V1 - E1);
  C_Asian_CV1[iter] = meanc(Z1);

  V2 = S_bar;
  E2 = S0*meanc(exp(r*t));
  beta2 = Y / (Cst~V2);
  beta2 = beta2[2];
  Z2 = Y - beta2 * (V2 - E2);
  C_Asian_CV2[iter] = meanc(Z2);

  V3 = exp(-r*tau) * _max_(S_tau - K,0);
  E3 = EuropeanBS(S0,K,sigma,tau,r,r);
  beta3 = Y / (Cst~V3);
  beta3 = beta3[2];
  Z3 = Y - beta3 * (V3 - E3);
  C_Asian_CV3[iter] = meanc(Z3);

  V4 = exp(-r*tau) * _max_(S_tilde - K,0);
  E4 = GeometricAsianBS(S0,K,sigma,t,r,r);
  beta4 = Y / (Cst~V4);
  beta4 = beta4[2];
  Z4 = Y - beta4 * (V4 - E4);
  C_Asian_CV4[iter] = meanc(Z4);

  iter = iter + 1;
endo;

C_Asian_CV = C_Asian_CV1~C_Asian_CV2~C_Asian_CV3~C_Asian_CV4;

x_C = seqa(14,4/255,256);
d_C_CV = zeros(256,4);
i = 1;
do until i > 4;
  {d_C_CV[.,i],F} = regKernelDensity(C_Asian_CV[.,i],x_C);
  i = i + 1;
endo;

{d_C_MC,F} = regKernelDensity(C_Asian_MC,x_C);

ratio = stdc(C_Asian_CV)/stdc(C_Asian_MC);
ratio = ratio^2;

stats = meanc(C_Asian_MC)~error(0) |
        meanc(C_Asian_CV)~100*ratio ;
print ftosa(stats,"%lf",3,2);

C = meanc(C_Asian_CV4);

graphset;
  begwind;
  window(2,2,0);

  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3|2; _pcolor = 1|2|3|4; _plwidth = 10;
  _paxht = 0.225; _pnumht = 0.225; _ptitlht = 0.275;
  xlabel("\216Option price");
  _pline = 1~1~C~0~C~10~1~0~10;
  xtics(14.0,18.0,1.0,2);

  setwind(1);
    ytics(0,2,0.5,2);
    _plegstr = "MC\000CV]1[\000CV]2[\000CV]3[\000CV]4[";
    _plegctl = {2 5 6 4};
    xy(x_C,d_C_MC~d_C_CV[.,1:3]);

  setwind(2);
    _pcolor = 1|5;
    ytics(0,8,2,2);
    _plegstr = "MC\000CV]4[";
    _plegctl = {2 5 6 4.75};
    xy(x_C,d_C_MC~d_C_CV[.,4]);

  _plctrl = -1; _pstype = 4;
  _pline = 0;
  _plegctl = 0;

  setwind(3);
    _pcolor = 2;
    xtics(0,400,100,2);
    ytics(0,100,20,2);
    ylabel("\216Y");
    xlabel("\216V]1[");
    xy(V1,Y);

  setwind(4);
    _pcolor = 5;
    xtics(0,100,20,2);
    ytics(0,100,20,2);
    ylabel("\216Y");
    xlabel("\216V]4[");
    xy(V4,Y);

  endwind;