new;
library frm,pgraph;

cls;

proc (1) = _GeometricAsianBS(t);
  local nt,m,t_sum,t_bar,v;

  nt = rows(t);
  m = seqa(1,1,nt);

  t_sum = sumc(t);
  t_bar = t_sum / nt;
  
  v = t_sum + 2 * sumc( (nt - m) .* t);
  v = v / (nt^2);

  retp(v);
endp;

proc GeometricAsianBS(S0,K,sigma,t,b,r);
  local tau,t_bar,upsilon,w,gamma_,m,d,d1,d2,CallPut;

  tau = maxc(t);
  t_bar = meanc(t);
  upsilon = _GeometricAsianBS(t);
  w = sigma .* sqrt(upsilon);

  gamma_ = b*t_bar + 0.5 * sigma^2 * (upsilon - t_bar);
  m = (b - 0.5*sigma^2) * t_bar;
  d = ( ln(S0 ./ K) + m ) ./ w;  
  d1 = d + w;
  d2 = d;
  _option_d1 = d1;
  _option_d2 = d2;

  if lower(_Option_Type) $== "put";
    CallPut = -S0 .* exp(gamma_ - r.*tau) .* cdfn(-d1) + K .*exp(-r.*tau) .* cdfn(-d2);
  else;
    CallPut = S0 .* exp(gamma_ - r.*tau) .* cdfn(d1) - K .* exp(-r.*tau) .* cdfn(d2);
  endif;

  retp(CallPut);
endp;

rndseed 123456;

tau = 5;
nT = tau;
t = seqa(1,1,nT);

S0 = 100;
r = 0.05;
sigma = 0.20;
K = 104;

nS = 1000000;
Cst = ones(nS,1);

St = simulate_GBM(S0,r,sigma,t,nS);
S_tau = St[nT,.]';
S_bar = meanc(St);
S_tilde = prodc(St)^(1/nT);
t_bar = meanc(t);

Y = exp(-r*tau) * _max_(S_bar - K,0);

V1 = S_tau;
E1 = S0*exp(r*tau);

V2 = S_bar;
E2 = S0*meanc(exp(r*t));

V3 = exp(-r*tau) * _max_(S_tau - K,0);
E3 = EuropeanBS(S0,K,sigma,tau,r,r);

V4 = exp(-r*tau) * _max_(S_tilde - K,0);
E4 = GeometricAsianBS(S0,K,sigma,t,r,r);

let all_indx[5,4] = 1 2 . .
                    1 2 3 .
                    1 . 3 .
                    . 2 3 .
                    1 2 3 4;
nIters = rows(all_indx);

iter = 1;
do until iter > nIters;

  indx = all_indx[iter,.]';
  indx = packr(indx);
  V = varget(itos("V",indx[1]));
  i = 2;
  do until i > rows(indx);
    V = V~varget(itos("V",indx[i]));
    i = i + 1;
  endo;

  beta = Y / (Cst~V);
  U = Y - (Cst~V)*beta;
  R2 = 1 - (stdc(U)/stdc(Y))^2;

  Big_Beta = miss(zeros(5,1),0);
  Big_Beta[1|(indx+1)] = Beta;

  results = Big_Beta'~100*R2~100*(1-R2);
  print ftosa(results,"%lf",3,3~3~3~3~3~1~1);

  iter = iter + 1;
endo;