new;
library frm,pgraph;

cls;

rndseed 123456;

proc (1) = MH_alpha(y,x,mu,Sigma);
  local f_x,f_y,alpha_xy;

  if cols(y) == 1;
    y = y';
    x = x';
  endif;

  f_x = pdfmvn(x,mu,Sigma);
  f_y = pdfmvn(y,mu,Sigma);

  alpha_xy = _min_(f_y./f_x,1.0);

  retp(alpha_xy);
endp;

proc (3) = MH_random(x1,x2,u,z1,z2,mu,Sigma);
  local y1,y2,alpha,e;

  y1 = x1 + z1;
  y2 = x2 + z2;
  alpha = MH_alpha(y1'~y2',x1'~x2',mu,Sigma)';
  e = (u .<= alpha);
  x1 = y1 .* e + x1 .* (1-e);
  x2 = y2 .* e + x2 .* (1-e);

  retp(x1,x2,alpha);
endp;

mu1 = 1; mu2 = -1; sigma1 = 2; sigma2 = 1; rho = 0.99;
mu = mu1|mu2;
sigma = sigma1|sigma2;
rho = xpnd(1|rho|1);
Sigma = rho .* sigma .* sigma';

n_burn = 500;
nS = 2000;
nT = n_burn + nS;

nModels = 4;
x1 = zeros(nT,nModels);
x2 = zeros(nT,nModels);
alpha = zeros(nT,nModels);

x1_t = zeros(1,nModels);
x2_t = zeros(1,nModels);

x1[1,.] = x1_t;
x2[1,.] = x2_t;

t = 2;
do until t > nT;
  u = rndu(1,1);

  Model = 1;
  do until Model > 4;
    if Model == 1;
      z1 = rndn(1,1);
      z2 = rndn(1,1);
    elseif Model == 2;
      z1 = sqrt(0.1)*rndn(1,1);
      z2 = sqrt(0.1)*rndn(1,1);
    elseif Model == 3;
      a = 2;
      z1 = 2*a*rndu(1,1) - a;
      z2 = 2*a*rndu(1,1) - a;
    elseif Model == 4;
      a = 0.2;
      z1 = 2*a*rndu(1,1) - a;
      z2 = 2*a*rndu(1,1) - a;
    endif;

    {x1_t[1,Model],x2_t[1,Model],alpha_t} = MH_random(x1_t[1,Model],x2_t[1,Model],u,z1,z2,mu,Sigma);

    x1[t,Model] = x1_t[1,Model];
    x2[t,Model] = x2_t[1,Model];
    alpha[t,Model] = alpha_t;

    Model = Model + 1;
  endo;

  t = t + 1;
endo;

x1 = trimr(x1,n_burn,0);
x2 = trimr(x2,n_burn,0);

rho_hat = zeros(nModels,1);
print "-----------------------------------------------------";
print ftosa(meanc(x1)~meanc(x2),"%lf",3,4);
print "-----------------------------------------------------";
print ftosa(stdc(x1)~stdc(x2),"%lf",3,4);
print "-----------------------------------------------------";
i = 1;
do until i > nModels;
  rho_hat[i] = submat(corrx(x1[.,i]~x2[.,i]),1,2);
  i = i + 1;
endo;
print ftosa(rho_hat,"%lf",3,4);
print "-----------------------------------------------------";
print ftosa(meanc(alpha),"%lf",3,4);
print "-----------------------------------------------------";

graphset;
  begwind;
  window(2,2,0);

  _pdate = ""; _pnum = 2; _pframe = 0;
  _paxht = 0.25;_pnumht = 0.20; _ptitlht = 0.275;
  fonts("simplex simgrma");
  _pcolor = 1; _plctrl = -1; _pstype = 4;
  xlabel("\216x]1[[(t)]");
  ylabel("\216x]2[[(t)]");
  xtics(-4,8,2,2);
  ytics(-3,2,1,2);

  i = 1;
  setwind(i);
    title("\216Proposal distribution (a)");
    _pcolor = i;
    xy(x1[.,i],x2[.,i]);

  i = 2;
  setwind(i);
    title("\216Proposal distribution (b)");
    _pcolor = i;
    xy(x1[.,i],x2[.,i]);

  i = 3;
  setwind(i);
    title("\216Proposal distribution (c)");
    _pcolor = i;
    xy(x1[.,i],x2[.,i]);

  i = 4;
  setwind(i);
    title("\216Proposal distribution (d)");
    _pcolor = i;
    xy(x1[.,i],x2[.,i]);

  endwind;