new;
library frm,pgraph;

cls;

rndseed 123456;

proc (1) = pdfCopulaClayton(u1,u2,theta);
  local a,b,c,d;
  a = 1 + theta;
  b = -(1 + 2*theta) ./ theta;
  c = (u1^(-theta) + u2^(-theta) -1)^b;
  d = (u1 .* u2)^(-a);
  retp(a .* c .* d);
endp;

proc (1) = Compute_PDF(x1,x2,model);
  local pdf,theta,lambda1,lambda2,u1,u2;

  if model == 1;
    pdf = exp(-(x1^2)-(x2^2)-x1);
  elseif model == 2;
    pdf = exp(-(x1^2)-(x2^2)) + exp(-(x1^2)-(x2^2)-1.5*x1.*x2);
  elseif model == 3;
    pdf = exp(-0.1 * cdfn(x1) .* cdfn(x2) .* x2^2);
    pdf = exp(-0.1 * exp(lncdfn(x1) + lncdfn(x2)) .* x2^2);
  elseif model == 4;
    theta = 2.5;
    lambda1 = 0.01;
    lambda2 = 0.05;
    u1 = 1 - exp(-lambda1 .* x1);
    u2 = 1 - exp(-lambda2 .* x2);
    pdf = pdfCopulaClayton(u1,u2,theta) .* lambda1 .* exp(-lambda1 .* x1) .* lambda2 .* exp(-lambda2 .* x2);
  endif;

  retp(pdf);
endp;

proc (1) = MH_alpha(y,x,model);
  local x1,x2,y1,y2,f_x,f_y,alpha_xy;

  x1 = x[1,.];
  x2 = x[2,.];

  y1 = y[1,.];
  y2 = y[2,.];

  f_x = compute_PDF(x1,x2,model);
  f_y = compute_PDF(y1,y2,model);

  alpha_xy = _min_(f_y./f_x,1.0);
  alpha_xy = missrv(alpha_xy,0);

  retp(alpha_xy);
endp;

proc (3) = MH_random(x1,x2,u,z1,z2,model);
  local y1,y2,alpha,e;

  y1 = x1 + z1;
  y2 = x2 + z2;
  alpha = MH_alpha(y1|y2,x1|x2,model);
  e = (u .<= alpha);
  x1 = y1 .* e + x1 .* (1-e);
  x2 = y2 .* e + x2 .* (1-e);

  retp(x1,x2,alpha);
endp;

n_burn = 500;
nS = 1500;
nT = n_burn + nS;

nModels = 4;
x1 = zeros(nT,nModels);
x2 = zeros(nT,nModels);
alpha = zeros(nT,nModels);

x1_t = rndu(1,nModels);
x2_t = rndu(1,nModels);

x1[1,.] = x1_t;
x2[1,.] = x2_t;

t = 2;
do until t > nT;

  u = rndu(1,1);
  z = rndmvn_chol(0|0,xpnd(1|0.5|1),1);
  z1 = z[.,1];
  z2 = z[.,2];

  Model = 1;
  do until Model > 4;
    {x1_t[1,Model],x2_t[1,Model],alpha_t} = MH_random(x1_t[1,Model],x2_t[1,Model],u,z1,z2,model);
    x1[t,Model] = x1_t[1,Model];
    x2[t,Model] = x2_t[1,Model];
    alpha[t,Model] = alpha_t;
    Model = Model + 1;
  endo;
  t = t + 1;
endo;

x1 = trimr(x1,n_burn,0);
x2 = trimr(x2,n_burn,0);
alpha = trimr(alpha,n_burn,0);

print "-----------------------------------------------------";
print ftosa(meanc(alpha),"%lf",3,4);
print "-----------------------------------------------------";

graphset;
  begwind;
  window(2,2,0);

  _pdate = ""; _pnum = 2; _pframe = 0; _pcross = 1;
  _paxht = 0.25;_pnumht = 0.20; _ptitlht = 0.275;
  fonts("simplex simgrma");
  _pcolor = 1; _plctrl = -1; _pstype = 4;
  xlabel("\216x]1[[(t)]");
  ylabel("\216x]2[[(t)]");

  i = 1;
  setwind(i);
    title("\216pdf (a)");
    _pcolor = i;
    xtics(-3,3,1,2);
    ytics(-2,3,1,2);
    xy(x1[.,i],x2[.,i]);

  i = 2;
  setwind(i);
    title("\216pdf (b)");
    _pcolor = i;
    xtics(-4,4,2,2);
    ytics(-4,4,2,2);
    xy(x1[.,i],x2[.,i]);

  i = 3;
  setwind(i);
    title("\216pdf (c)");
    _pcolor = i;
    xtics(-60,20,20,2);
    ytics(-20,40,20,2);
    xy(x1[.,i],x2[.,i]);

  i = 4;
  setwind(i);
    title("\216pdf (d)");
    _pcolor = i; _plctrl = -1;
    xtics(0,40,10,2);
    ytics(0,25,5,2);
    xy(x1[.,i],x2[.,i]);

  endwind;