new;
library frm,optmum,pgraph;
cls;

datPathName = "..\\..\\data\\";
datFileName = datPathName $+ "equity-indices-2017-daily";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

BeginDate = 19500101;
BeginDate = 19900101;
EndDate = 20191231;
Dates = data[.,1];
Index = data[.,3 4];

Rt = ln(Index) - ln(lag1(Index));
Rt = selif(Rt, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Index = selif(Index, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Dates = selif(Dates, (Dates .>= BeginDate) .and (Dates .<= EndDate));

rndseed 1234565;

n = 20;

Years = seqa(1,1,100);
nYears = rows(Years);
T = 260*Years; 
alpha = 1 - n/T;

__output = 0;

Short_Stress = zeros(nYears,2);
Long_Stress = zeros(nYears,2);

R_extremes = minc(Rt)|maxc(Rt);
T_extremes = zeros(4,1);

Short_theta = zeros(3,2);
Long_theta = zeros(3,2);

i = 1;
do until i > 2;

  R = Rt[.,i];
  R = packr(R);

  min = -Ordering_Data(R,1,n);
  {mu,sigma,xi} = regGEV(min,0);
  Long_Stress[.,i] = -cdfGEVi(alpha,mu,sigma,xi);
  Long_theta[.,i] = mu|sigma|xi;
  T_extremes[i] = n/(1 - cdfGEV(-R_extremes[i],mu,sigma,xi))/260;

  max = Ordering_Data(R,n,n);
  {mu,sigma,xi} = regGEV(max,0);
  Short_Stress[.,i] = cdfGEVi(alpha,mu,sigma,xi);
  Short_theta[.,i] = mu|sigma|xi;
  T_extremes[2+i] = n/(1 - cdfGEV(R_extremes[2+i],mu,sigma,xi))/260;

  i = i + 1;
endo;

let indx = 5 10 25 50 75 100;

Results = Long_theta~short_theta;
print ftosa(100*Results,"%lf",10,3); 

Results = 100 * (Long_Stress[indx,.] ~ Short_Stress[indx,.]);
print ftosa(Years[indx]~Results,"%lf",10,0~2~2~2~2); 
print ftosa(error(0)~100*R_extremes',"%lf",10,0~2~2~2~2);
print ftosa(error(0)~T_extremes',"%lf",10,0~2~2~2~2);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0; _pcross = 0; _pgrid = {3,1};
  _pltype = 6|3; _pstype = 8|8|9|9; _plctrl = 10; _pcolor = 1|2|3|4; 
  _plwidth = 10;
  fonts("simplex simgrma");
  xtics(0,50,10,2);
  ytics(-15,15,5,2);
  xlabel("\216Return time (in years)");
  _plegstr = "MSCI USA (long)\000MSCI EMU (long)\000MSCI USA (short)\000MSCI EMU (short)";
  _plegctl = {2 5 5.75 3.15};
  xy(Years,100*(Long_Stress~Short_Stress));

