new;
library frm,optmum,pgraph;
cls;

datPathName = "..\\..\\data\\";
datFileName = datPathName $+ "equity-indices-2017-daily";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

BeginDate = 19500101;
BeginDate = 19900101;
EndDate = 20191231;
Dates = data[.,1];
Index = data[.,3 4];

Rt = ln(Index) - ln(lag1(Index));
Rt = selif(Rt, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Index = selif(Index, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Dates = selif(Dates, (Dates .>= BeginDate) .and (Dates .<= EndDate));

u_Rt = dependogram(Rt);
u_Rt = u_Rt * rows(u_Rt) / (rows(u_Rt) + 1);

rndseed 1234565;

n = 20;

Years = 5;
T = 260*Years; 
alpha = 1 - n/T;

__output = 0;

Short_stress = zeros(2,1);
Long_stress = zeros(2,1);

Short_theta = zeros(3,2);
Long_theta = zeros(3,2);

nBlocks = rows(Rt)/n;
Long_cdf = zeros(nBlocks,2);
Short_cdf = zeros(nBlocks,2);

i = 1;
do until i > 2;

  R = Rt[.,i];
  R = packr(R);

  min = -Ordering_Data(R,1,n);
  {mu,sigma,xi} = regGEV(min,0);
  Long_stress[i] = -cdfGEVi(alpha,mu,sigma,xi);
  Long_theta[.,i] = mu|sigma|xi;
  Long_cdf[.,i] = cdfGEV(min,mu,sigma,xi);

  max = Ordering_Data(R,n,n);
  {mu,sigma,xi} = regGEV(max,0);
  Short_stress[i] = cdfGEVi(alpha,mu,sigma,xi);
  Short_theta[.,i] = mu|sigma|xi;
  Short_cdf[.,i] = cdfGEV(max,mu,sigma,xi);

  i = i + 1;
endo;

proc AlphaParametrization(theta,Family);
  local alpha;

  if Family == 1;                   /* Gumbel */
    alpha = 1 + sqrt(theta^2);  
  elseif Family == 2;               /* Galambos */
    alpha = sqrt(theta^2);  
  endif;

  retp(alpha);
endp;

Family = 1;                         /* Gumbel */
Parametrization = 1;

declare matrix u_IFM; 

proc procML(theta);
  local u1,u2,alpha,lnpdf;

  u1 = u_IFM[.,1];
  u2 = u_IFM[.,2];

  alpha = theta[1];

  if Parametrization;
    alpha = AlphaParametrization(theta,Family);
  else;
    alpha = theta;
  endif;

  if family == 1;                   /* Gumbel */
    lnpdf = ln(pdfCopulaGumbel(u1,u2,alpha));
  elseif family == 2;               /* Galambos */
    lnpdf = ln(pdfCopulaGalambos(u1,u2,alpha));
  endif;

  retp(lnpdf);
endp;

fn NegSumLL(theta) = -sumc(procML(theta));

data = Short_cdf~Long_cdf;

let pairs[4,2] = 1 2
                 1 4
                 3 4
                 3 2;
let Portfolio[4,2] =  1  1
                      1 -1
                     -1 -1
                     -1  1;

theta = zeros(4,1);
__output = 0;
i = 1;
do until i > 4; 
  u_IFM = data[.,pairs[i,.]];
  sv = 0.5;
  {theta[i],fmin,grd,retcode} = optmum(&NegSumLL,sv);
  i = i + 1;
endo;

if Parametrization;
  alpha = AlphaParametrization(theta,Family);
else;
  alpha = theta;
endif;

iter = 1;
do until iter > 4;

  if iter == 1;
    stress1 = 0.10;  stress2 = 0.10;
  elseif iter == 2;
    stress1 = -0.10;  stress2 = 0.10;
  elseif iter == 3;
    stress1 = 0.10;  stress2 = -0.10;
  elseif iter == 4;
    stress1 = -0.10;  stress2 = -0.10;
  endif;

  if (stress1 > 0) and (stress2 > 0);
    indx_alpha = 1;
  elseif (stress1 > 0) and (stress2 < 0);
    indx_alpha = 2;
  elseif (stress1 < 0) and (stress2 < 0);
    indx_alpha = 3;
  elseif (stress1 < 0) and (stress2 > 0);
    indx_alpha = 4;
  endif;

  if stress1 < 0;
    u1 = cdfGEV(-stress1,Long_theta[1,1],Long_theta[2,1],Long_theta[3,1]);
  else;
    u1 = cdfGEV(stress1,Short_theta[1,1],Short_theta[2,1],Short_theta[3,1]);
  endif;

  if stress2 < 0;
    u2 = cdfGEV(-stress2,Long_theta[1,2],Long_theta[2,2],Long_theta[3,2]);
  else;
    u2 = cdfGEV(stress2,Short_theta[1,2],Short_theta[2,2],Short_theta[3,2]);
  endif;

  T1 = (n / 260) ./ (1 - u1);
  T2 = (n / 260) ./ (1 - u2);

  if Family == 1;
    T12 = (n / 260) ./ (1 - u1 - u2 + cdfCopulaGumbel(u1,u2,alpha[indx_alpha]));
  elseif Family == 2;
    T12 = (n / 260) ./ (1 - u1 - u2 + cdfCopulaGalambos(u1,u2,alpha[indx_alpha]));
  endif;
  T12_min = (n / 260) ./ (1 - u1 - u2 + cdfCopulaUpper2(u1,u2)); 
  T12_max = (n / 260) ./ (1 - u1 - u2 + cdfCopulaProduct2(u1,u2)); 

  print "-------------------------------------------------------------------";
  Results = (Stress1~T1~Stress2~T2);
  print ftosa(Results,"%lf",5,3);
  Results = T12_min~T12~T12_max;
  print ftosa(Results,"%lf",5,3);
  print "-------------------------------------------------------------------";

  iter = iter + 1;
endo;

let n = 1 5 20 260 260 260;

let T1 = 5 5 5 5 10 1;
let T2 = 5 5 5 5 5  1;

u1 = 1 - n ./ (T1 .* 260);
u2 = 1 - n ./ (T2 .* 260);

T12_min = (n / 260) ./ (1 - u1 - u2 + cdfCopulaUpper2(u1,u2)); 
T12_max = (n / 260) ./ (1 - u1 - u2 + cdfCopulaProduct2(u1,u2)); 

Results = n~T1~T2~T12_min~T12_max;
print ftosa(Results,"%lf",3,0);






