new;
library frm,optmum,pgraph;
cls;

datPathName = "..\\..\\data\\";
datFileName = datPathName $+ "equity-indices-2017-daily";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

BeginDate = 19500101;
BeginDate = 19900101;
EndDate = 20191231;
Dates = data[.,1];
Index = data[.,3 4];

Rt = ln(Index) - ln(lag1(Index));
Rt = selif(Rt, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Index = selif(Index, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Dates = selif(Dates, (Dates .>= BeginDate) .and (Dates .<= EndDate));

extremes = Rt~Dates;

rndseed 1234565;

n = 20;

Year = 5;
T = 260*Year; 
alpha = 1 - n/T;

__output = 0;

Short_stress = zeros(2,1);
Long_stress = zeros(2,1);

Short_theta = zeros(3,2);
Long_theta = zeros(3,2);

nBlocks = rows(Rt)/n;
Long_cdf = zeros(nBlocks,2);
Short_cdf = zeros(nBlocks,2);

i = 1;
do until i > 2;

  R = Rt[.,i];

  min = -Ordering_Data(R,1,n);
  {mu,sigma,xi} = regGEV(min,0);
  Long_stress[i] = cdfGEVi(alpha,mu,sigma,xi);
  Long_theta[.,i] = mu|sigma|xi;
  Long_cdf[.,i] = cdfGEV(min,mu,sigma,xi);

  max = Ordering_Data(R,n,n);
  {mu,sigma,xi} = regGEV(max,0);
  Short_stress[i] = cdfGEVi(alpha,mu,sigma,xi);
  Short_theta[.,i] = mu|sigma|xi;
  Short_cdf[.,i] = cdfGEV(max,mu,sigma,xi);

  i = i + 1;
endo;

proc AlphaParametrization(theta,Family);
  local alpha;

  if Family == 1;                   /* Gumbel */
    alpha = 1 + sqrt(theta^2);  
  elseif Family == 2;               /* Galambos */
    alpha = sqrt(theta^2);  
  endif;

  retp(alpha);
endp;

Family = 1;                         /* Gumbel */
Parametrization = 1;

declare matrix u_IFM; 

proc procML(theta);
  local u1,u2,alpha,lnpdf;

  u1 = u_IFM[.,1];
  u2 = u_IFM[.,2];

  alpha = theta[1];

  if Parametrization;
    alpha = AlphaParametrization(theta,Family);
  else;
    alpha = theta;
  endif;

  if family == 1;                   /* Gumbel */
    lnpdf = ln(pdfCopulaGumbel(u1,u2,alpha));
  elseif family == 2;               /* Galambos */
    lnpdf = ln(pdfCopulaGalambos(u1,u2,alpha));
  endif;

  retp(lnpdf);
endp;

fn NegSumLL(theta) = -sumc(procML(theta));

data = Short_cdf~Long_cdf;

let pairs[4,2] = 1 2
                 1 4
                 3 4
                 3 2;
let Portfolio[4,2] =  1  1
                      1 -1
                     -1 -1
                     -1  1;

theta = zeros(4,1);
__output = 0;
i = 1;
do until i > 4; 
  u_IFM = selif(Rt, ((Rt[.,1] * Portfolio[i,1]) .> 0) .and ((Rt[.,2] * Portfolio[i,2]) .> 0));
  u_IFM = dependogram(u_IFM);
  u_IFM = u_IFM ./ (rows(u_IFM) +1) .* rows(u_IFM);
  sv = 0.5;
  {theta[i],fmin,grd,retcode} = optmum(&NegSumLL,sv);
  i = i + 1;
endo;

if Parametrization;
  alpha = AlphaParametrization(theta,Family);
else;
  alpha = theta;
endif;

nX = 100;
stress1 = seqa(0,Short_stress[1]/(nX-1),nX)~seqa(0,Short_stress[1]/(nX-1),nX)~seqa(0,Long_stress[1]/(nX-1),nX)~seqa(0,Long_stress[1]/(nX-1),nX);
stress2 = seqa(0,Short_stress[2]/(nX-1),nX)~seqa(0,Long_stress[2]/(nX-1),nX)~seqa(0,Long_Stress[2]/(nX-1),nX)~seqa(0,Short_Stress[2]/(nX-1),nX);

Bivariate_Stress_Independant_1 = zeros(nX,4);
Bivariate_Stress_Independant_2 = zeros(nX,4);
Bivariate_Stress_Copula_1 = zeros(nX,4);
Bivariate_Stress_Copula_2 = zeros(nX,4);
u1 = zeros(nX,4);
u2 = zeros(nX,4);

_stress1_ = seqa(0,0.003,100);
_u1_ = zeros(rows(_stress1_),4);
pline_x = zeros(rows(_stress1_),4);
pline_y = zeros(rows(_stress1_),4);

i = 1;
do until i > 4;

  if Portfolio[i,1] == 1;
    u1[.,i] = cdfGEV(stress1[.,i],Short_theta[1,1],Short_theta[2,1],Short_theta[3,1]);
    _u1_[.,i] = cdfGEV(_stress1_,Short_theta[1,1],Short_theta[2,1],Short_theta[3,1]);
  elseif Portfolio[i,1] == -1;
    u1[.,i] = cdfGEV(stress1[.,i],Long_theta[1,1],Long_theta[2,1],Long_theta[3,1]);
    _u1_[.,i] = cdfGEV(_stress1_,Long_theta[1,1],Long_theta[2,1],Long_theta[3,1]);
  endif;

  if Portfolio[i,2] == 1;
    u2[.,i] = cdfGEV(stress2[.,i],Short_theta[1,2],Short_theta[2,2],Short_theta[3,2]);
  elseif Portfolio[i,2] == -1;
    u2[.,i] = cdfGEV(stress2[.,i],Long_theta[1,2],Long_theta[2,2],Long_theta[3,2]);
  endif;

  T_tilde = (n / 260) ./ (1 - u1[.,i] - u2[.,i]' + cdfCopulaProduct2(miss(u1[.,i],0),miss(u2[.,i]',0)));
  indx = missrv(miss(sumc(T_tilde' .<= year),0),1);
  Bivariate_Stress_Independant_1[.,i] = stress1[.,i];
  Bivariate_Stress_Independant_2[.,i] = stress2[indx,i];

  if Family == 1;
    T_tilde = (n / 260) ./ (1 - u1[.,i] - u2[.,i]' + cdfCopulaGumbel(miss(u1[.,i],0),miss(u2[.,i]',0),alpha[i]));
  elseif Family == 2;
    T_tilde = (n / 260) ./ (1 - u1[.,i] - u2[.,i]' + cdfCopulaGalambos(miss(u1[.,i],0),miss(u2[.,i]',0),alpha[i]));
  endif;
  indx = missrv(miss(sumc(T_tilde' .<= year),0),1);
  Bivariate_Stress_Copula_1[.,i] = stress1[.,i];
  Bivariate_Stress_Copula_2[.,i] = stress2[indx,i];

  if Family == 1;
    T_tilde = (n / 260) ./ (1 - _u1_[.,i] - u2[.,i]' + cdfCopulaGumbel(miss(_u1_[.,i],0),miss(u2[.,i]',0),alpha[i]));
  elseif Family == 2;
    T_tilde = (n / 260) ./ (1 - _u1_[.,i] - u2[.,i]' + cdfCopulaGalambos(miss(_u1_[.,i],0),miss(u2[.,i]',0),alpha[i]));
  endif;
  indx = missrv(miss(sumc(T_tilde' .<= year),0),1);
  pline_x[.,i] = _stress1_;
  pline_y[.,i] = stress2[indx,i];

  i = i + 1;
endo;

Short_Stress = 100 * Short_Stress;
Long_Stress = -100 * Long_Stress;
Bivariate_Stress_Independant_1 = 100 * Bivariate_Stress_Independant_1 .* Portfolio[.,1]';
Bivariate_Stress_Independant_2 = 100 * Bivariate_Stress_Independant_2 .* Portfolio[.,2]';
Bivariate_Stress_Copula_1 = 100 * Bivariate_Stress_Copula_1 .* Portfolio[.,1]';
Bivariate_Stress_Copula_2 = 100 * Bivariate_Stress_Copula_2 .* Portfolio[.,2]';
pline_x = 100 * pline_x .* Portfolio[.,1]';
pline_y = 100 * pline_y .* Portfolio[.,2]';
e = ones(rows(pline_x),1);
end_pline = 15 * e; 
pline = e.*(1~6)~pline_x[.,1]~pline_y[.,1]~pline_x[.,1]~end_pline~e.*(1~0~0) |
        e.*(1~6)~pline_x[.,2]~pline_y[.,2]~pline_x[.,1]~-end_pline~e.*(1~0~0) |
        e.*(1~6)~pline_x[.,3]~pline_y[.,3]~pline_x[.,3]~-end_pline~e.*(1~0~0) |
        e.*(1~6)~pline_x[.,4]~pline_y[.,4]~pline_x[.,4]~end_pline~e.*(1~0~0) ;

sens = Rt .> 0;
u1 = abs(Rt[.,1]); 
u2 = abs(Rt[.,2]);
u1 = sens[.,1] .* cdfGEV(u1,Short_theta[1,1],Short_theta[2,1],Short_theta[3,1]) + 
     (1-sens[.,1]) .* cdfGEV(u1,Long_theta[1,1],Long_theta[2,1],Long_theta[3,1]);
u2 = sens[.,2] .* cdfGEV(u2,Short_theta[1,2],Short_theta[2,2],Short_theta[3,2]) +  
     (1-sens[.,2]) .* cdfGEV(u2,Long_theta[1,2],Long_theta[2,2],Long_theta[3,2]);
param = sens[.,1] + 10*sens[.,2];
param = recode(param,param .== (11~1~0~10),1|2|3|4); 
if Family == 1;
  T_tilde = (n / 260) ./ (1 - u1 - u2 + cdfCopulaGumbel(miss(u1,0),miss(u2,0),alpha[param]));
elseif Family == 2;
  T_tilde = (n / 260) ./ (1 - u1 - u2 + cdfCopulaGalambos(miss(u1,0),miss(u2,0),alpha[param]));
endif;
indx = indexcat((T_tilde .> year) .and (sens[.,1] .== sens[.,2]),1);
extremes = extremes[indx,.];
e = ones(rows(extremes),1);
xe = 100*extremes[.,1];
ye = 100*extremes[.,2];

graphset;
  begwind;
  makewind(9,6.855,0,0,1);
  makewind(9,6.855,0,0,1);
  fonts("simplex simgrma");
  axmargin(0,0,0,0);

  setwind(1);
    _pcross = 1; _pdate = ""; _pframe = 0; _plwidth = 10;
    _pnum = 2; _pcolor = 1|1|1|1|4|4|4|4; _pltype = 3|3|3|3|6|6|6|6;
    xtics(-10,12,2,1);
    ytics(-10,12,2,1);
    
    xlabel("\216MSCI USA");
    ylabel("\216MSCI EMU");

    _pline = 1~1~Long_Stress[1]~Long_Stress[2]~Long_Stress[1]~Short_Stress[2]~1~2~12 |
             1~1~Long_Stress[1]~Short_Stress[2]~Short_Stress[1]~Short_Stress[2]~1~2~12 |
             1~1~Short_Stress[1]~Short_Stress[2]~Short_Stress[1]~Long_Stress[2]~1~2~12 |
             1~1~Short_Stress[1]~Long_Stress[2]~Long_Stress[1]~Long_Stress[2]~1~2~12 |
             pline;

    xy(Bivariate_Stress_Independant_1~Bivariate_Stress_Copula_1,
       Bivariate_Stress_Independant_2~Bivariate_Stress_Copula_2);

  setwind(2);
    _pline = 0;

    _pmsgstr = "C[+]\000C[\202\74\201]\000C";

    _pmsgctl = -7.0~5.7~0.20~0~1~2~10 |
                3.0~3.4~0.20~0~1~1~10 |
                2.9~-2.5~0.20~0~1~4~10 ; 

    // _psym = (xe~ye)~e.*(4~5~0~1~10);
    xy(Bivariate_Stress_Independant_1~Bivariate_Stress_Copula_1,
       Bivariate_Stress_Independant_2~Bivariate_Stress_Copula_2);

   
  endwind;

