new;
library frm,optmum,pgraph;
cls;

proc (1) = Compute_Inertia(y);
  local dy,inertia;

  dy = y - lag1(y);
  dy = packr(dy);
  inertia = sumc(stdc(dy)^2);

  retp(inertia);
endp;

rndseed 123456;

declare matrix B1,B2,B3,B4;
declare matrix C1,C2,C3,C4;

n = 4;

let rho = 1.0 
          0.7  1.0
          0.2  0.2 1.0
          0.0  0.2 0.5 1.0;
let sigma = 0.20 0.20 0.20 0.20;
let mu  = 0.05 0.10 0.05 0.00;

rho = xpnd(rho);
call invpd(rho);

x0 = 100 * ones(n,1);
t = seqa(0,1/50,50*5);
nT = rows(t);
x = matrix_simulate_mGBM(x0,mu,sigma,rho,t);

ln_x = ln(x);
r = ln_x - lag(ln_x);
r[1,.] = zeros(1,n);

cov = vcx(r);
cov = sigma .* sigma' .* rho;
{lambda,B0,PCAmatrix} = regPCA(cov,4,0);

acp_factor = r*B0;
acp_price = ln(100*cumprodc(1 + cumsumc(acp_factor')'));

_nmf_maxIters = 10000;
_nmf_PrintIters = 0;

A = ln_x';
m = rows(A);
n = 1;
p = cols(A);

_nmf_maxIters = 10000;
_nmf_PrintIters = 0;

if n == 1;
  _nmf_B0 = ones(m,1);
  _nmf_C0 = meanc(A)';
endif;

do until n > 4; 
  {B,C,retcode} = regNMF(A,n,1);
  {B_star,C_star} = NMF_scaling(B,C,maxc(B));

  call varput(B_star,itos("B",n));
  call varput(C_star,itos("C",n));

  n = n + 1;
  if n <= 4;
    _nmf_B0 = _nmf_B0 ~ rndu(m,1);
    _nmf_C0 = _nmf_C0 | rndu(1,nT);
  endif;
endo; 
 
A1 = B1*C1;
A2 = B2*C2;
A3 = B3*C3;
A4 = B4*C4;

I0 = Compute_Inertia(ln_x);
I1 = Compute_Inertia(A1');
I2 = Compute_Inertia(A2');
I3 = Compute_Inertia(A3');
I4 = Compute_Inertia(A4');

lambda_nmf = (I1|I2|I3|I4)/I0;
lambda_pca = cumsumc(lambda)/sumc(lambda);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1; _plwidth = 10; _pcolor = 1|2; _plctrl = 1; _pstype = 8|9;
  xtics(0.5,4.5,0.5,1);
  let xlab = "" "1" "" "2" "" "3" "" "4" "";
  asclabel(xlab,0);
  _plegstr = "NMF\000PCA";
  _plegctl = {2 5 5 2};
  xlabel("\216Number of factors");
  ylabel("\216Quality of representation (in %)");
  xy(0,100*(lambda_nmf~lambda_pca));
