new;
library frm,pgraph,optmum;

cls;

rndseed 123;

declare matrix x1,x2;

pi1 = 0.50;
pi2 = 1 - pi1;

mu1 = 1 | 3;
mu2 = 4 | 1;

Sigma1 = eye(2);
Sigma2 = 1.5*eye(2);
Sigma = 0.5 * (Sigma1 + Sigma2);

nS = 50;
x1_rnd = rndmvn_chol(mu1,Sigma1,nS);
x2_rnd = rndmvn_chol(mu2,Sigma2,nS);
xs = x1_rnd[.,1]~x2_rnd[.,1];
ys = x1_rnd[.,2]~x2_rnd[.,2];

proc QDA_function(x);
  local inv_Sigma1,inv_Sigma2,p;

  inv_Sigma1 = invpd(Sigma1);
  inv_Sigma2 = invpd(Sigma2);

  p = 0.5 * ln( det(Sigma2) / det(Sigma1)) - 0.5 * (x - mu1)'inv_Sigma1*(x -mu1) + 0.5 * (x - mu2)'inv_Sigma2*(x -mu2) + ln(pi1 / pi2);
  retp(p);
endp;

proc LDA_function(x);
  local inv_Sigma,p;

  inv_Sigma = invpd(Sigma);
  p = (mu2 - mu1)'inv_Sigma*x - 0.5 * (mu2'inv_Sigma*mu2 - mu1'inv_Sigma*mu1) - ln(pi1 / pi2);
  retp(p);
endp;

proc LDA_boundary(x2);
  local x,p;

  x = x1 | x2;
  p = LDA_function(x);

  retp(p^2);
endp;

proc QDA_boundary(x2);
  local x,p;

  x = x1 | x2;
  p = QDA_function(x);

  retp(p^2);
endp;

x = seqa(-3,9/100,101);
nX = rows(x);
y = miss(zeros(nX,2),0);
p = miss(zeros(nX,2),0);

i = 1;
do until i > nX;
  x1 = x[i];
  {x2,fmin,grd,retcode} = optmum(&LDA_boundary,x1);
  p[i,1] = LDA_function(x1|x2);
  if abs(p[i,1]) <= 1e-5;
    y[i,1] = x2;
  endif;
  {x2,fmin,grd,retcode} = optmum(&QDA_boundary,x1);
  p[i,2] = QDA_function(x1|x2);
  if abs(p[i,2]) <= 1e-5;
    y[i,2] = x2;
  endif;
  i = i + 1;
endo;

graphset;
  begwind;
  makewind(9,6.855,0,0,1);
  makewind(9,6.855,0,0,1);

  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10; _pcolor = 2|3; _pltype = 6|3;
  xtics(-3,6,1,0);
  ytics(-3,6,1,0);
  _psym = mu1[1]~mu1[2]~11~7.5~1~1~10 |
          mu2[1]~mu2[2]~11~7.5~4~1~10 ;
  _pmsgstr = "C]1[\000C]2[";
  _pmsgctl = -2~5~0.30~0~1~1~10 |
              5~-2~0.30~0~1~4~10 ;
  _pline = 1~3~mu1[1]~1~mu1[1]~5~1~1~10 |
           1~3~-1~mu1[2]~3~mu1[2]~1~1~10 |
           1~3~mu2[1]~-1~mu2[1]~3~1~4~10 |
           1~3~2~mu2[2]~6~mu2[2]~1~4~10 ;
  _plegstr = "LDA\000QDA";
  _plegctl = {2 5 1.3 2.5};
  xlabel("\216X]1[");
  ylabel("\216X]2[");

  setwind(1);
    xy(x,y);

  _plegctl = 0;

  setwind(2);
    _plctrl = -1; _pstype = 9|10; _psymsiz = 3.5; _pcolor = 1|4;
    xy(xs,ys);

  endwind;  


