new;
library frm,optmum,pgraph;

cls;

rndseed 123;

n = 1000;
n_x = 4;
n_z = 3;
n_y = 3;

x = 2*rndu(n,n_x)-1;
x = 10*x;

y = 2*rndu(n,n_y)-1;
y = nn_fn(y,5);

f_xz = 0;
f_zy = 5;
f_loss = 3;

constant_xz = 1;
constant_zy = 1;
directlink_xy = 1;

row_beta = n_z;
col_beta = n_x;
if constant_xz == 1;
  col_beta = col_beta + 1;
endif;

row_gamma = n_y;
col_gamma = n_z;
if constant_zy == 1;
  col_gamma = col_gamma + 1;
endif;
if directlink_xy == 1;
  col_gamma = col_gamma + n_x;
endif;

beta_matrix = rndn(row_beta,col_beta);
gamma_matrix = rndn(row_gamma,col_gamma);

theta = vecr(beta_matrix) | vecr(gamma_matrix);

proc (1) = loss_function(theta);
  local col_beta,beta_matrix,col_gamma,gamma_matrix;
  local beta_x,beta_c,gamma_z,gamma_c,gamma_x;
  local u,z,v,y_hat,loss;

  col_beta = n_x;
  if constant_xz == 1;
    col_beta = col_beta + 1;
  endif;
  beta_matrix = reshape(theta[1:n_z*col_beta],n_z,col_beta);
  theta = theta[1+n_z*col_beta:rows(theta)];
  col_gamma = n_z;
  if constant_zy == 1;
    col_gamma = col_gamma + 1;
  endif;
  if directlink_xy == 1;
    col_gamma = col_gamma + n_x;
  endif;
  gamma_matrix = reshape(theta,n_y,col_gamma);

  {u,z,v,y_hat} = nn_forward_propagation(x,beta_matrix,f_xz,gamma_matrix,f_zy);
  loss = nn_error_loss(y_hat,y,f_loss);
  loss = sumc(vecr(loss));

  retp(loss);
endp;

grad_theta = gradp(&loss_function,theta)';

grad_beta = reshape(grad_theta[1:n_z*col_beta],n_z,col_beta);
grad_theta = grad_theta[1+n_z*col_beta:rows(grad_theta)];
grad_gamma = reshape(grad_theta,n_y,col_gamma);
grad_beta_x = grad_beta[.,1:n_x];
grad_gamma_z = grad_gamma[.,1:n_z];

if constant_xz == 1;
  grad_beta_c = grad_beta[.,n_x+1];
else;
  grad_beta_c = zeros(n_z,1);
endif;

if constant_zy == 1;
  grad_gamma_c = grad_gamma[.,n_z+1];
else;
  grad_gamma_c = zeros(n_y,1);
endif;

if (directlink_xy == 1) and (constant_zy == 1);
  grad_gamma_x = grad_gamma[.,n_z+1+1:n_z+1+n_x];
elseif (directlink_xy == 1) and (constant_zy == 0);
  grad_gamma_x = grad_gamma[.,n_z+1:n_z+n_x];
else;
  grad_gamma_x = zeros(n_y,n_x);
endif;

print "====================================================================================================================";
print grad_beta_x;
print "-----------------------------------------------";
print grad_beta_c;
print "-----------------------------------------------";
print grad_gamma_z;
print "-----------------------------------------------";
print grad_gamma_c;
print "-----------------------------------------------";
print grad_gamma_x;

{beta_x,beta_c,constant_xz,gamma_z,gamma_c,gamma_x,constant_zy,directlink_xy} = nn_split_parameters(n_x,n_z,n_y,beta_matrix,gamma_matrix);
{u,z,v,y_hat} = nn_forward_propagation(x,beta_matrix,f_xz,gamma_matrix,f_zy);

D1_beta_x = (((y_hat - y) * gamma_z))' * x;
D1_beta_c = (((y_hat - y) * gamma_z))' * ones(n,1);
D1_gamma_z = (y_hat - y)' * z;
D1_gamma_c = (y_hat - y)' * ones(n,1);
D1_gamma_x = (y_hat - y)' * x;


print "====================================================================================================================";
print D1_beta_x;
print "-----------------------------------------------";
print D1_beta_c;
print "-----------------------------------------------";
print D1_gamma_z;
print "-----------------------------------------------";
print D1_gamma_c;
print "-----------------------------------------------";
print D1_gamma_x;

G_gamma_z = zeros(n_y,n_z);
G_gamma_c = zeros(n_y,1);
G_gamma_x = zeros(n_y,n_x);
G_beta_x = zeros(n_z,n_x);
G_beta_c = zeros(n_z,1);

i = 1;
do until i > n;
  j = 1;
  do until j > n_y;
    h = 1;
    do until h > n_z;
      G_gamma_z[j,h] = G_gamma_z[j,h] + (y_hat[i,j] - y[i,j]) * z[i,h];
      h = h + 1;
    endo;
    j = j + 1;
  endo;
  i = i + 1;
endo;

i = 1;
do until i > n;
  j = 1;
  do until j > n_y;
    h = 1;
    do until h > n_z;
      k = 1;
      do until k > n_x;
        G_beta_x[h,k] = G_beta_x[h,k] + (y_hat[i,j] - y[i,j]) * gamma_z[j,h] * x[i,k];
        k = k + 1;
      endo;
      h = h + 1;
    endo;
    j = j + 1;
  endo;
  i = i + 1;
endo;

print "====================================================================================================================";
print G_beta_x;
print "-----------------------------------------------";
print G_gamma_z;
