/*
**> Thierry Roncalli
**
**> Lecture Notes on Risk Management & Financial Regulation
**
**> Page ??, Figure ??
**
*/

new;
library frm,pgraph;

cls;

x = seqa(-0.40,0.80/250,251);
nX = rows(x);

let xi     = 0.01 -0.02   0.15;
let omega  = 0.05  0.10   0.20;
let eta    = 0.00  2.00  -5.00;
nAssets = rows(eta);

nu1 = 1e10; nu2 = 1; nu3 = 4;
pdf1 = zeros(nX,nAssets);
pdf2 = zeros(nX,nAssets);
pdf3 = zeros(nX,nAssets);

i = 1;
do until i > nAssets;
  pdf1[.,i] = pdfSN(x,xi[i],omega[i],eta[i]);
  pdf2[.,i] = pdfST(x,xi[i],omega[i],eta[i],nu2);
  pdf3[.,i] = pdfST(x,xi[i],omega[i],eta[i],nu3);
  i = i + 1;
endo;

{mu,sigma,gamma1,gamma2} = momSN(xi,omega,eta);
M1 = 100*mu ~ 100*sigma ~ gamma1 ~ gamma2;

{mu,sigma,gamma1,gamma2} = momST(xi,omega,eta,nu2);
M2 = 100*mu ~ 100*sigma ~ gamma1 ~ gamma2;

{mu,sigma,gamma1,gamma2} = momST(xi,omega,eta,nu3);
M3 = 100*mu ~ 100*sigma ~ gamma1 ~ gamma2;

print "-------------------------------------------------------------";
print "SN";
print ftosa(M1,"%lf",3,2);
print "-------------------------------------------------------------";
print "ST (nu = 1)";
print ftosa(M2,"%lf",3,5);
print "-------------------------------------------------------------";
print "ST (nu = 4)";
print ftosa(M3,"%lf",3,5);
print "-------------------------------------------------------------";

graphset;
  window3u;
  fonts("simplex simgrma");
  _pdate = ""; _pframe = 0; _pnum = 2;
  _pltype = 6|1|3; _pcolor = 1|2|4; _plwidth = 10;
  _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.275;

  xtics(-40,40,10,2);
  xlabel("\216Return (in %)");
  _plegstr = "\202x\201=1%, \202w\201=5%, \202a\201=0\000"\
             "\202x\201=-2%, \202w\201=10%, \202a\201=2\000"\
             "\202x\201=15%, \202w\201=20%, \202a\201=-5";

  setwind(1);
    title("\216SN");
    _plegctl = {2 6.5 7.5 3.5};
    xy(100*x,pdf1);

  _plegctl = 0;

  setwind(2);
    title("\216ST (\202n\201 = 1)");
    xy(100*x,pdf2);

  setwind(3);
    title("\216ST (\202n\201 = 4)");
    xy(100*x,pdf3);

  endwind;