/*
**> Thierry Roncalli
**
**> Lecture Notes on Risk Management & Financial Regulation
**
**> Page ??, Figure ??
**
*/

new;
library frm,pgraph;

cls;

rndseed 123;

nS = 50000;

x = seqa(-0.40,0.70/250,251);
nX = rows(x);

nu = 5;
let xi     = 0.01 -0.02   0.15;
let omega  = 0.05  0.10   0.20;
let eta    = 0.00  2.00  -5.00;
nAssets = rows(eta);

pdf = zeros(nX,nAssets);
Simulated_Moments = zeros(nAssets,4);

i = 1;
do until i > nAssets;
  pdf[.,i] = pdfST(x,xi[i],omega[i],eta[i],nu);
  u = rndST(nS,1,xi[i],omega[i],eta[i],nu);
  Simulated_Moments[i,1] = meanc(u);
  Simulated_Moments[i,2] = sqrt((nS-1)/nS) * stdc(u);
  Simulated_Moments[i,3] = Estimate_Skewness(u);
  Simulated_Moments[i,4] = Estimate_Kurtosis(u) - 3;
  i = i + 1;
endo;

{mu,sigma,gamma1,gamma2} = momST(xi,omega,eta,nu);
ST_Moments = mu ~ sigma ~ gamma1 ~ gamma2;
diff = Simulated_Moments - ST_moments;
ratio = Simulated_Moments ./ miss(ST_moments,0.00);

print "-------------------------------------------------------------";
print "ST moments:";
print ftosa(ST_moments,"%lf",3,5);
print "Simulated moments:";
print ftosa(Simulated_moments,"%lf",3,5);
print "-------------------------------------------------------------";
print "Difference:";
print ftosa(diff,"%lf",3,4);
print "Ratio:";
print ftosa(ratio,"%lf",3,4);
print "-------------------------------------------------------------";

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pframe = 0; _pnum = 2;
  _pltype = 6|1|3; _pcolor = 1|2|4; _plwidth = 10;
  xtics(-40,30,10,2);
  xlabel("\216Return (in %)");
  _plegstr = "\202x\201=1%, \202w\201=5%, \202a\201=0\000"\
             "\202x\201=-2%, \202w\201=10%, \202a\201=2\000"\
             "\202x\201=15%, \202w\201=20%, \202a\201=-5";
  _plegctl = {2 5 1.0 5};
  xy(100*x,pdf);