new;
library frm,pgraph;

cls;

rndseed 123;

proc (1) = Simplified_Bond_Price(R,Spread,RR,h);
  local Ct_m,t_m,Bt_m,St_m,P;

  if cols(RR) == 1;
    RR = RR';
  endif;

  if cols(Spread) == 1;
    Spread = Spread';
  endif;

  if (rows(R) /= 5) and (cols(R) == 5);
    R = R';
  endif;

  let Ct_m = 5 5 5 5 105;
  let t_m  = 1 2 3 4 5;

  t_m = t_m - h/260;

  Bt_m = exp(-R .* t_m);
  St_m = exp(-Spread .* t_m);
  P = sumc(Ct_m .* Bt_m .* St_m) + 100 * RR' .* (1 - St_m[5,.])' .* Bt_m[5,.]';

  retp(P);
endp;

datPathName = "..\\..\\Data\\";
datFileName = datPathName $+ "zc";
loadm maturity = ^datFileName;

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

data = selif(data,(data[.,1] .>= 20140101) .and (data[.,1] .<= 20141231));
R = data[.,2:cols(data)];
nR = cols(R);
cnd = sumr(R .== lag1(R)) .== nR;
data = delif(data,cnd);

Dates = data[.,1];
nDates = rows(Dates);
R = data[.,2:cols(data)]/100;

tau = seqa(1,1,5);
indx = indnv(tau,Maturity);
R = R[.,indx];

Notional = 100;
Coupon = 5;
Maturity = 5;
Ct_m = Coupon * ones(Maturity,1);
Ct_m[Maturity] = Ct_m[Maturity] + Notional;
t_m = seqa(1,1,Maturity);

Current_Date = Dates[nDates];

nB = 10000;

R0 = R[nDates,.]';
S0 = 0.02;
RR0 = 0.50;
P0 = Simplified_Bond_Price(R0,S0,RR0,0);
print ftosa(P0,"P0 (with credit risk) = %lf",3,2);

alpha = 0.99;

dR = R - lag1(R);
dR = packr(dR);
Sigma_dR = vcx(dR);

Sigma_Spread = 20/1e4;

RR_min = 0.40;
RR_max = 0.60;

nR = 100;
nS = 5000;
VaR = zeros(nR,3);

iter = 1;
do until iter > nR;
  if iter == nR;
    nS = 100000;
  endif;
  dR = rndmvn_chol(0,Sigma_dR,nS);
  dSpread = sigma_Spread * rndn(nS,1);
  RR = RR_min + (RR_max - RR_min) * rndu(nS,1);

  P1 = Simplified_Bond_Price(R0' + dR,S0,RR0,1/260); 
  P2 = Simplified_Bond_Price(R0' + dR,S0+dSpread,RR0,1/260); 
  P3 = Simplified_Bond_Price(R0' + dR,S0+dSpread,RR,1/260); 

  PnL1 = nB * (P1 - P0);
  PnL2 = nB * (P2 - P0);
  PnL3 = nB * (P3 - P0);

  VaR[iter,1] = -quantile(PnL1,1-alpha);
  VaR[iter,2] = -quantile(PnL2,1-alpha);
  VaR[iter,3] = -quantile(PnL3,1-alpha);

  iter = iter + 1;
endo;

print meanc(VaR);

