new;
library frm,pgraph;

cls;

rndseed 123;

// Historical scenarios for the asset return

datPathName = "..\\..\\Data\\";
datFileName = datPathName $+ "var-option";
data = loadd(datFileName);
vnames = "" $+ getname(datFileName);
R_S = data[.,2];
nS = rows(R_S);

nC = 100;
S0 = 100;
K = 100;
Sigma = 0.20;
tau = 52/252;
b = 0.05;
r = 0.05;

// Option price

C0 = EuropeanBS(S0,K,Sigma,tau,r,r);
IV0 = EuropeanBS_ImpVol(S0,K,tau,r,r,4.14);
Sigma = 0.20;
C0 = 4.14;

print ftos(C0,"C_t = %lf",10,3);
print ftos(100*IV0,"Sigma_t = %lf",10,3);

// Value-at-risk

h = 1;
tau = tau - h/252;

S1 = S0 .* (1 + R_S);
C1 = EuropeanBS(S1,K,Sigma,tau,b,r);
PnL = nC * (C1 - C0);

R_C = (C1 - C0) / C0;
rho = submat(corrx(R_C~R_S),1,2);
print "-----------------------------------------------------------";
print ftosa(100*rho,"rho = %lf",10,2);

Cst = ones(nS,1);
beta = R_C/(R_S~Cst);

sc = seqa(1,1,nS);
Results = sc~100*R_S~S1~C1~PnL;

print "-----------------------------------------------------------";
print ftosa(Results[1:9,.],"%lf",10,2);
print "-----------------------------------------------------------";
indx = sortind(PnL);
print ftosa(Results[indx[1:10],.],"%lf",10,2);
print "-----------------------------------------------------------";

alpha = 0.99;
VaR = -quantile(PnL,1-alpha);
print ftos(VaR,"VaR = %lf",10,2);

alpha = 0.975;
ES = Compute_ES(PnL,alpha);
print ftos(ES,"ES = %lf",10,2);

x = seqa(-0.05,0.10/nS,nS);
y = EuropeanBS(S0 .* (1 + x),K,Sigma,tau,b,r);
y = (y - C0)/C0;
y = y~(x~Cst)*beta;
x = x~x;

graphset;
  _pdate = ""; _pnum =2; _pframe = 0;
  _pltype = 6|6|3; _plwidth = 10; _pcolor = 4|1|2; _pcross = 1;
  _plctrl = -1|0|0; _pstype = 11; _psymsiz = 3.5;
  xlabel("\216R]S[ (in %)");
  ylabel("\216R]C[ (in %)");
  xtics(-5,5,1,0);
  ytics(-50,75,25,2);
  _plegstr = "Historical scenarios\000Black-Scholes formula\000OLS regression";
  _plegctl = {2 5 5 1};
  xy(100*(R_S~x),100*(R_C~y));

xls_results = sc~R_S~S1~C1~PnL;
xlsFileName = "..\\results\\chap1-var.xlsx";
call SpreadSheetWrite(xls_Results,xlsFileName,"A3",10); 

