/*
**> Thierry Roncalli
**
**> Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??
**
*/

new;
library frm,pgraph;

cls;

rndseed 123;

// Historical scenarios for the asset return

datPathName = "..\\..\\Data\\";
datFileName = datPathName $+ "var-option";
data = loadd(datFileName);
vnames = "" $+ getname(datFileName);
R_S = data[.,2];
Delta_Sigma = data[.,3];
nS = rows(R_S);

nC = 100;
S0 = 100;
K = 100;
Sigma0 = 0.20;
tau = 52/252;
b = 0.05;
r = 0.05;
C0 = EuropeanBS(S0,K,Sigma0,tau,b,r);
d1 = _option_d1; d2 = _option_d2;
cd1 = cdfn(d1); cd2 = cdfn(-d2);
pd1 = pdfn(d1); pd2 = pdfn(d2);
print ftosa((d1~cd1~pd1)|(d2~cd2~pd2),"%lf",3,4);
C0 = 4.14;

fn Call_Option(S) = EuropeanBS(S,K,Sigma0,tau,b,r);
fn Call_Option2(tau) = EuropeanBS(S0,K,Sigma0,tau,b,r);
fn Call_Option3(Sigma) = EuropeanBS(S0,K,Sigma,tau,b,r);

Delta0 = EuropeanBS_Delta(S0,K,Sigma0,tau,b,r);
D0 = gradp(&Call_Option,S0);
Gamma0 = EuropeanBS_Gamma(S0,K,Sigma0,tau,b,r);
G0 = hessp(&Call_Option,S0);
Theta0 = EuropeanBS_Theta(S0,K,Sigma0,tau,b,r);
T0 = gradp(&Call_Option2,tau);
Vega0 = EuropeanBS_Vega(S0,K,Sigma0,tau,b,r);
V0 = gradp(&Call_Option3,Sigma0);

print ftos(C0,"C0 = %lf",10,3);
print ftos(Delta0,"Delta0 = %lf",10,4);
print ftos(Gamma0,"Gamma0 = %lf",10,4);
print ftos(Theta0,"Theta0 = %lf",10,4);
print ftos(Vega0,"Vega0 = %lf",10,4);

print ftos(D0,"Delta0 = %lf",10,4);
print ftos(G0,"Gamma0 = %lf",10,4);
print ftos(T0,"Theta0 = %lf",10,4);
print ftos(V0,"Vega0 = %lf",10,4);

// Value-at-risk

h = 1/252;
tau = tau - h;

S1 = S0 .* (1 + R_S);
Sigma1 = Sigma0 + Delta_Sigma;
C1 = EuropeanBS(S1,K,Sigma1,tau,b,r);
PnL1 = nC * (C1 - C0);
PnL2 = nc * (Vega0 * (Sigma1 - Sigma0));
PnL3 = nC * (Delta0 * (S1 - S0) + Vega0 * (Sigma1 - Sigma0));
PnL4 = nC * (Delta0 * (S1 - S0) + 0.5 * Gamma0 * (S1 - S0)^2 + Vega0 * (Sigma1 - Sigma0));
PnL5 = nc * (Delta0 * (S1 - S0) + 0.5 * Gamma0 * (S1 - S0)^2 + Theta0 * h + Vega0 * (Sigma1 - Sigma0));

sc = seqa(1,1,nS);
Results = sc~S1~100*Sigma1~PnL1~PnL2~PnL3~PnL4~PnL5;

print "-----------------------------------------------------------";
print ftosa(Results[1:9,.],"%lf",10,2);
print "-----------------------------------------------------------";
indx = sortind(PnL1);
print ftosa(Results[indx[1:10],.],"%lf",10,2);
print "-----------------------------------------------------------";

alpha = 0.99;
VaR1 = -quantile(PnL1,1-alpha);
VaR2 = -quantile(PnL2,1-alpha);
VaR3 = -quantile(PnL3,1-alpha);
VaR4 = -quantile(PnL4,1-alpha);
VaR5 = -quantile(PnL5,1-alpha);

VaR = VaR1|VaR2|VaR3|VaR4|VaR5;

print "VaR = ";
print ftosa(VaR,"%lf",10,2);

alpha = 0.975;
ES1 = compute_ES(PnL1,alpha);
ES2 = compute_ES(PnL2,alpha);
ES3 = compute_ES(PnL3,alpha);
ES4 = compute_ES(PnL4,alpha);
ES5 = compute_ES(PnL5,alpha);
ES = ES1|ES2|ES3|ES4|ES5;

print "ES = ";
print ftosa(ES,"%lf",10,2);

xls_results = sc~S1~Sigma1~PnL1~PnL2~PnL3~PnL4~PnL5;
xlsFileName = "..\\results\\chap1-var.xlsx";
call SpreadSheetWrite(xls_Results,xlsFileName,"A3",13); 
