/*
**> Thierry Roncalli
**
**> Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??
**
*/

new;
library frm,pgraph;

cls;

rndseed 123;

// Historical scenarios for the asset return

datPathName = "..\\..\\Data\\";
datFileName = datPathName $+ "var-option";
data = loadd(datFileName);
vnames = "" $+ getname(datFileName);
R_S = data[.,2];
Delta_Sigma = data[.,3];
nS = rows(R_S);

nC = 100;
S0 = 100;
K = 100;
Sigma0 = 0.20;
tau = 52/252;
b = 0.05;
r = 0.05;
C0 = 4.14;

Delta0 = EuropeanBS_Delta(S0,K,Sigma0,tau,b,r);
Gamma0 = EuropeanBS_Gamma(S0,K,Sigma0,tau,b,r);
Theta0 = EuropeanBS_Theta(S0,K,Sigma0,tau,b,r);
Vega0 = EuropeanBS_Vega(S0,K,Sigma0,tau,b,r);

// Value-at-risk

h = 1/252;
tau = tau - h;

S1 = S0 .* (1 + R_S);
Sigma1 = Sigma0 + Delta_Sigma;
C1 = EuropeanBS(S1,K,Sigma1,tau,b,r);
PnL1 = nC * (C1 - C0);
PnL2 = nc * (Delta0 * (S1 - S0) + 0.5 * Gamma0 * (S1 - S0)^2 + Theta0 * h + Vega0 * (Sigma1 - Sigma0) );
PnL3 = nc * (Delta0 * (S1 - S0) + Theta0 * h);
PnL4 = nc * (Delta0 * (S1 - S0) + Theta0 * h + Vega0 * (Sigma1 - Sigma0));

sc = seqa(1,1,nS);
Results = sc~PnL1~PnL2~PnL3~PnL4;

indx1 = sortind(PnL1);
indx2 = sortind(PnL2);
indx3 = sortind(PnL3);
indx4 = sortind(PnL4);

Results = sc~Results[indx1,1 2]~Results[indx2,1 3]~Results[indx3,1 4]~Results[indx4,1 5];

print "-----------------------------------------------------------";
print ftosa(Results[1:10,.],"%lf",10,2);
print "-----------------------------------------------------------";