/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Exercise 2.4.5
**
*/

new;
library frm,pgraph;

cls;

print "=====================  Question 1.  =======================================";

let beta = 0.5 1.5 1.0;
sigma_I = 0.20;
var_tilde = 0.03|0.07|0.00;
sigma_tilde = sqrt(var_tilde);
D = diagrv(eye(3),sigma_tilde^2);
cov = beta*beta' * (sigma_I^2) + D;
sigma = sqrt(diag(cov));
rho = cov ./ sigma ./ sigma';
cov = rho .* sigma .* sigma';

print ftosa(100*sigma,"%lf",3,3);
print ftosa(100*rho,"%lf",3,3);

rho = beta .* sigma_I ./ sigma;
print ftosa(100*rho,"%lf",3,3);

rho = cov ./ sigma ./ sigma';
print ftosa(100*cov,"%lf",3,3);

print;
print "=====================  Question 2.a ======================================";

Let S0 = 100 50 50;
let w  =   4 10  5;
W = w .* S0;
Pt = sumc(W);

print ftosa(W,"%lf",3,3); print;

sigma = sqrt(W'cov*W);
VaR1 = cdfni(0.99) * sigma * sqrt(2/52); 

print ftos(sigma,"sigma(w) = %lf",3,2);
print ftos(VaR1,"10-day VaR = %lf",3,2);

print;
print "=====================  Question 2.b ======================================";

rndseed 123;
u = rndu(260,1);
PnL = trunc(100 * sigma/sqrt(260) * cdfti(u,5))/100;
PnL = sortc(PnL,1);
print ftosa(PnL[1:5],"%lf",3,2); print;

VaR2 = -quantile(PnL,0.01);
print ftosa(VaR2,"1-day VaR = %lf",3,2);
VaR2 = -(PnL[2] + (2.6 - 2) * (PnL[3] - PnL[2]));
print ftosa(VaR2,"1-day VaR = %lf",3,2);
VaR2 = sqrt(10) * VaR2;
print ftosa(VaR2,"10-day VaR = %lf",3,2); 

print;
print "=====================  Question 2.c ======================================";

K_IMA = 3 * (VaR1|VaR2);
K_Sp = sumc(W[1:2]) * 0.08 + W[3] * 0.04;
K_Gen = 0.08 * Pt;
K_SMM = K_Sp + K_Gen;
K = K_IMA|K_Sp|K_Gen|K_SMM;
print ftosa(K,"%lf",3,0);

print;
print "=====================  Question 3.a  =======================================";

Delta_P = -0.25;
n_P = 2 * W[3] / S0[3];
new_W = W;
new_W[3] = new_W[3] + n_P * Delta_P * S0[3];
print ftos(n_P,"Number of put options = %lf",3,2);
print ftos(new_W[3],"Exposure on the index = %lf",3,2);
sigma = sqrt(new_W'cov*new_W);
VaR3 = cdfni(0.99) * sigma * sqrt(2/52); 
print ftos(sigma,"sigma(w) = %lf",3,2);
print ftos(VaR3,"VaR = %lf",3,2);

print;
print "=====================  Question 3.b  =======================================";

Beta_Exposure = sumc(W .* beta);
print ftos(Beta_exposure,"Beta = %lf",3,2);
n_P = Beta_Exposure ./ S0[3] ./ abs(delta_P);
new_W = W;
new_W[3] = new_W[3] + n_P * Delta_P * S0[3];
new_Beta_Exposure = sumc(new_W .* beta);
print ftos(New_Beta_exposure,"New Beta = %lf",3,2);
print ftos(n_P,"Number of put options = %lf",3,2);
print ftos(new_W[3],"Exposure on the index = %lf",3,2);
sigma = sqrt(new_W'cov*new_W);
VaR3 = cdfni(0.99) * sigma * sqrt(2/52); 
print ftos(sigma,"sigma(w) = %lf",3,2);
print ftos(VaR3,"VaR = %lf",3,2);

print;
print "=====================  Question 3.c  =======================================";

nPut = seqa(0,1,201);
nMax = rows(nPut);
gVaR = zeros(nMax,1);
W_I = zeros(nMax,1);

i = 1;
do until i > nMax;
  n_P = nPut[i];
  new_W = W;
  new_W[3] = new_W[3] + n_P * Delta_P * S0[3];
  sigma = sqrt(new_W'cov*new_W);
  gVaR[i] = cdfni(0.99) * sigma * sqrt(2/52);  
  W_I[i] = new_W[3];   
  i = i + 1;
endo;

indx = seqa(1,10,20); 

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1; _pgrid = {3,1};
  xtics(0,200,25,2);
  ytics(50,150,10,5);
//  _psym = submat(nPut~gVaR~ones(nMax,1).*(8~2.5~4~1~10),indx,0);
  _psym = 96~VaR3~8~2.5~0~1~10;
  _pline = 1~3~96~0~96~VaR3~1~0~10 |
           1~3~0~VaR3~96~VaR3~1~0~10 ;
  xlabel("\216Number of put options");
  ylabel("\216VaR (in $)");
  xy(nPut,gVaR);
  





