new;
library frm,pgraph,optmum;

dlibrary frm-tridiag; // Don't miss to download the tridiag solver

cls;

declare matrix Use_Fokker_Planck,Use_Density;

Use_Density = 1;

_fcmptol = 1e-12;

mu = 0.10;
sigma = 0.20;

declare matrix _x_end,x_begin;

proc aProc(t,x);
  retp( 0.5*(sigma^2) * (x^2) .* ones(rows(x),1) ); 
endp;

proc bProc(t,x);
  if Use_Fokker_Planck;
    retp( -mu .* x + 2 * (sigma^2) * x);
  else;
    retp( mu .* x );
  endif;
endp;

proc cProc(t,x);
  if Use_Fokker_Planck;
    retp( (mu - sigma^2) .* ones(rows(x),1) );
  else;
    retp( 0 * ones(rows(x),1) );
  endif;
endp;

proc dProc(t,x);  
  retp( zeros(rows(x),1) );
endp;

proc tminBound(t,x);
  local cn;

  if Use_Fokker_Planck;
    cn = ones(rows(x),1) .* dotfeq(x,x_begin);
    if sumc(cn) /= 1;
      errorlog "error"; 
      end;
    endif;
  else;
    if Use_Density;
      cn = ones(rows(x),1) .* dotfeq(x,_x_end);
      if sumc(cn) == 0;
        errorlog "error"; 
      endif;
    else;
      cn = ones(rows(x),1) .* (x .<= _x_end);
    endif;
  endif;

  retp(cn);
endp;

call PDE(&aProc,&bProc,&cProc,&dProc,0,
         &tminBound,0,0,0,0);

tmin = 0;
tmax = 1;
Nt = 101;
xmin = 0;
xmax = 300;
Nx = 301;

_pde_PrintIters = 0;
_pde_SaveLastIter = 1;

x_end = seqa(0,1,301);
nx = rows(x_end);

x_begin = 100;

cdf = cdfn((ln(x_end) - ln( x_begin) - (mu - 0.5*sigma^2)*tmax)/(sigma*sqrt(tmax)));
pdf = pdfn((ln(x_end) - ln(x_begin) - (mu - 0.5*sigma^2)*tmax)/(sigma*sqrt(tmax)))/(sigma*sqrt(tmax))./x_end;

Use_Fokker_Planck = 1;

call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"pde",1);
t = PDE_readFile("pde","t");
x = PDE_readFile("pde","x");
U = PDE_readFile("pde","t"|t);
sol_Fokker_Planck = u;

Use_Fokker_Planck = 0;
Use_Density = 1;

sol = zeros(nx,1);

i = 1;
do until i > nx;

  _x_end = x_end[i];

  call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"pde",0.5);
  t = PDE_readFile("pde","t");
  x = PDE_readFile("pde","x");
  U = PDE_readFile("pde","t"|t);
  sol[i] = u[101];
  
  i = i + 1;
endo;

dx = (xmax - xmin)/(Nx - 1);
sol_Fokker_Planck = sol_Fokker_Planck/dx;

if Use_Density;
  sol_Feynmann_Kac = sol/dx;
else;
  sol_Feynmann_Kac = (sol - lag1(sol)) / dx;
endif;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 1|3|4; _plwidth = 10; _pcolor = 1|2|3;
  ytics(0,0.020,0.005,2);
  xtics(25,225,25,2);
  _plegstr = "Analytical solution\000Feynman-Kac solution\000Fokker-Planck solution";
  _plegctl = {2 5 5.5 4};
  xy(x,pdf~sol_Feynmann_Kac~sol_Fokker_Planck);
