new;
library frm,pgraph;

dlibrary frm-tridiag; // Don't miss to download the tridiag solver

cls;

_fcmptol = 1e-12;

nu = 0.50;
rho = -0.75;
beta = 1.00;

declare matrix x_0,y_0;

proc aProc(t,x,y);
  retp( 0.5 * x^(2*beta) .* y^2 );
endp;

proc bProc(t,x,y);
  retp( 0.5 * rho * nu * x^beta .* y^2 );
endp;

proc cProc(t,x,y);
  retp( 0.5 * nu^2 .* y^2 .* ones(rows(x),1) );
endp;

proc dProc(t,x,y);
  retp( 2 * beta .* x^(2*beta-1) .* y^2 + 2 * rho * nu .* x^beta .* y );
endp;

proc eProc(t,x,y);
  retp( (2 * nu^2 .* y + beta * rho * nu .* x^(beta-1) .* y^2) );
endp;

proc fProc(t,x,y);
  retp( -(beta  .* (2*beta - 1) .* x^(2*beta-2) .* y^2 + nu^2 + 2 * beta * rho * nu .* x^(beta-1) .* y) );
endp;

proc gProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc tminBound(t,x,y);
  local cn;
  cn = ones(rows(x),cols(y)) .* dotfeq(x,x_0) .* dotfeq(y,y_0);
  if sumc(sumc(cn)) /= 1;
    errorlog "error"; 
    end;
  endif;
  retp( cn );
endp;

proc DxminBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc DxmaxBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc DyminBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc DymaxBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

call PDE2D(&aProc,&bProc,&cProc,&dProc,&eProc,&fProc,&gProc,0,
                  &tminBound,0,0,0,0,
                  &DxminBound,&DxmaxBound,&DyminBound,&DymaxBound);

xmin = 0.70; xmax = 1.30; Nx = 121;
ymin = 0.000; ymax = 0.20; Ny = 101;
tmin = 0; tmax = 0.50; Nt = 51;

x_0 = 1.00; y_0 = 0.06;

_Hopscotch_SaveLastIter = 1;
_Hopscotch_PrintIters = 1;
_Hopscotch_Methods = {1, 0.5};
call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"tmp");

x = ExtractSolution("tmp","x");
y = ExtractSolution("tmp","y");
tmax = ExtractSolution("tmp","t");
sol = ExtractSolution("tmp","t"|tmax);

dx = (xmax - xmin)/(Nx - 1);
dy = (ymax - ymin)/(Ny - 1);
sol_Fokker_Planck = sol/dx/dy;

graphset;
  begwind;
  makewind(9,6.855,0,0,1);
  makewind(9/3,6.855/3,5,4,1);

  setwind(1);
    _pdate = ""; _pnum = 2; _pframe = 0; _paxht = 0.25;
    ztics(0,300,100,2);
    xtics(0,0.20,0.05,2);
    ytics(0.70,1.30,0.10,2);
    xlabel("\216X]2[");
    ylabel("\216X]1[");
    _pzclr = 1|2|3|4|5;
    surface(y,x,sol_Fokker_Planck);

  setwind(2);
    _paxht = 0.30; _pnumht = 0.30; _paxht = 0.40;
    _plev = 0.2|1|2|5|20|50|100|200|300|500|700;

  endwind;


