new;
library frm,pgraph;

cls;

datPathName = "..\\..\\data\\";
datFileName = datPathName $+ "bis-credit-2019b";
dollar_adjustment = 1;

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

let string selected_names = 
                    "cn-5" "cn-3" "cn-4"
                    "gb-5" "gb-3" "gb-4"
                    "jp-5" "jp-3" "jp-4"
                    "us-5" "us-3" "us-4"
                    "eu-5" "eu-3" "eu-4";
indx = indsav(selected_names,vnames);

dates = data[.,1];
yyyy = trunc(dates/10000);
mm = trunc((dates - yyyy*10000 - 1)/100);
t = yyyy + mm/12;

data = data[.,indx] / 1000;

if dollar_adjustment == 1;
  let fx = 7.381624219	0.60210469	103.4912705	1.000000	0.809288474;
  fx = fx .*. ones(3,1);
  data_usd = miss(data ./ fx',0);
else;
  data_usd = data;
endif;

growth_rate = data ./ lagn(data,4) - 1.0;
results = selif(dates~100*growth_rate,dates .>= 20100101);
print ftosa(results,"%lf",3,2);

graphset;
  window5d;

  _pdate = ""; _pframe = 0; _pnum = 2;
  _plwidth = 10; _pltype = 6|1|3|2; _pcolor = 1|4|2|3;
  _paxht = 0.22; _pnumht = 0.275; _ptitlht = 0.35;
  _plctrl = 0|0|0|1; _pstype = 8; _psymsiz = 2;

  xtics(1980,2020,5,2);
  let xlab = "80" "85" "90" "95" "00" "05" "10" "15" "20";
  asclabel(xlab,0);

  setwind(1);
    title("\216China");
    ytics(0,24,4,4);
    xy(clean_miss(t,data_usd[.,1:3],1));

  setwind(2);
    title("\216United Kingdom");
    ytics(0,6,1,1);
    xy(clean_miss(t,data_usd[.,4:6],0.999));

  setwind(3);
    title("\216Japan");
    ytics(0,12,2,2);
    xy(clean_miss(t,data_usd[.,7:9],0.999));

  _paxht = 0.22; _pnumht = 0.20; _ptitlht = 0.275;

  setwind(4);
    title("\216United States");
    ytics(0,27,3,3);
    xy(clean_miss(t,data_usd[.,10:12],1));

  setwind(5);
    title("\216Euro area");
    _plegctl = {2 5 1.05 4.25};
    _plegstr = "Private non-financial\000Households\000Non-financial corporations";
    xy(clean_miss(t,data_usd[.,13:15],0.999));

  endwind;

proc (2) = clean_miss(x,y,smooth);
  local n,e,p;

  n = sumr(y .== error(0));
  e = n .== cols(y);
  x = delif(x,e);
  y = delif(y,e);

  x = x .* ones(1,cols(y));

  if smooth < 1;
    p = smooth;
    y = spline(x,y,p,x);
  endif;

  retp(x,y);
endp;