/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 3.1.2
**
*/

new;
library frm,pgraph;
frmSet;

cls;

_Continuous_Time = 1;

declare matrix lambda;

let params = 0.05 -0.05 0.06 10;
fn Survival_Function(t) = exp(-lambda*t);
fn Density_Function(t) = lambda * exp(-lambda*t);
fn YieldCurve_Function(t) = NelsonSiegel_Spot_Rate(params,t);

Coupon = 4.5;
Maturity = 10;
Notional = 100;
t = seqa(1,1,Maturity);
Ct = Coupon * ones(Maturity,1);

dTau = 10/260;
nTau = 1 + Maturity/dTau;
tau = seqa(0,dTau,nTau);

Recovery_Rate = 0.40;

rndseed 123;
u = 15/1e5 + 0.004*rndn(nTau,1);
lambda_tau = recserar(u,0.02,1);

P = zeros(nTau,1);
P[nTau] = Notional;

default_time = 6.25;
DP = zeros(nTau,1);
DP[nTau] = Recovery_rate * Notional;
cnd = 1;

i = 1;
do until i > nTau-1;
  lambda = lambda_tau[i];
  data = (t-tau[i])~CT;
  data = selif(data,data[.,1] .>= 0.0);
  AC = Coupon * (tau[i] - trunc(tau[i]));
  P[i] = Risky_Bond_Price(data[.,1],data[.,2],Notional,Maturity-tau[i],Recovery_Rate,&YieldCurve_Function,&Survival_Function,&Density_Function) - AC;
  if tau[i] <= default_time;
    DP[i] = P[i];
  else;
    if cnd;
      DP[i] = Recovery_rate * Notional;
      tau[i] = tau[i-1];
    else;
      DP[i] = Recovery_rate * Notional;
    endif;
    cnd = 0;
  endif;
  i = i + 1;
endo;

graphset;
 window3u;

  fonts("simplex simgrma");
  _pdate = ""; _pframe = 0; _pnum = 2;
  _plwidth = 10; _pltype = 6|1|3|2; _pcolor = 1|4|2|3;
  _paxht = 0.22; _pnumht = 0.22; _ptitlht = 0.275;

  xtics(0,10,2,2);
  xlabel("\216t");

  setwind(1);
   title("\216\202l\201]t[ (in %)");
   _pcolor = 1;
   ytics(0,10,2,2);
   _pline = 1~3~default_time~0~default_time~10~1~0~6;
   _pmsgstr = "Default time";
   _pmsgctl = default_time+0.25~2~0.22~0~1~0~7;
   xy(tau,100*lambda_tau);

  setwind(2);
   title("\216Clean price P]t[\L\216Without default");
   _pcolor = 2;
   ytics(90,110,5,2);
   xy(tau,P);

  setwind(3);
   title("\216Clean price P]t[\L\216With default");
   _pcolor = 4;
   _parrow = 6.5~90~6.5~110~5~.15~32~0~1~6~10 |
             6.075~93~6.075~40~5~.15~32~0~1~6~10;
   _pline = 1~1~0~112~10~112~1~0~6 |
            1~1~0~88~10~88~1~0~6 ;
   _pmsgstr = "Market risk\000Credit risk";
   _pmsgctl = 6.75~99~0.22~0~1~0~7 |
              3.5~65~0.22~0~1~0~7 ;

   ytics(20,120,20,2);
   xy(tau,DP);

 endwind;