/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 3.1.3
**
*/

new;
library frm,pgraph,optmum;
frmSet;

cls;

// Nelson-Siegel parameters

let params = 0.05 -0.05 0.06 10;

// CDS parameters

lambda = 50/1e4;
Recovery = 0.40;
Notional = 1e6;
let T = 0.5 1 2 3 5 7 10;
nT = rows(T);

_Continuous_Time = 1;

proc DF(t);
  retp( NelsonSiegel_ZC_Bond(params,t) );
endp;

proc Survival_Fn(t);
  retp( exp(-lambda * t) );
endp;

proc Density_Fn(t);
  retp( lambda .* exp(-lambda * t) );
endp;

_intord = 128;

Price = zeros(nT,1);
Spread = zeros(nT,1);
RPV01 = zeros(nT,1);
Results_WAP = zeros(nT,6);

c = 10/1e4;
i = 1;
do until i > nT;
  Maturity = T[i];
  t_m = seqa(0.25,0.25,4*Maturity);
  {Price[i],Spread[i],RPV01[i]} = CDS_Price(&DF,&Survival_Fn,&Density_Fn,Notional,Recovery,c,Maturity,t_m);
  Results_WAP[i,1 2 3] = _CDS_Price_WAP~1e4*_CDS_Spread_WAP~_CDS_RPV01_WAP;
  i = i + 1;
endo;
MtM = Notional .* (Spread - c) .* RPV01;
Spread = 1e4 * Spread;

Results = Price~Spread~RPV01~MtM;

c = 100/1e4;
i = 1;
do until i > nT;
  Maturity = T[i];
  t_m = seqa(0.25,0.25,4*Maturity);
  {Price[i],Spread[i],RPV01[i]} = CDS_Price(&DF,&Survival_Fn,&Density_Fn,Notional,Recovery,c,Maturity,t_m);
  Results_WAP[i,4 5 6] = _CDS_Price_WAP~1e4*_CDS_Spread_WAP~_CDS_RPV01_WAP;
  i = i + 1;
endo;
MtM = Notional .* (Spread - c) .* RPV01;
Spread = 1e4 * Spread;

Results = Results~Price~Spread~RPV01~MtM;

print ftosa(T~Results,"%lf",3,2);
print ftosa(T~Results_WAP,"%lf",3,2);

print ftosa(T~Results[.,1 5 2 3],"%lf",3,0~0~0~2~3);
print ftosa(T~Results_WAP[.,1 4 2 3],"%lf",3,0~0~0~2~3);


