/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 3.1.3
**
*/

new;
library frm,pgraph,optmum;
frmSet;

cls;

declare matrix lambda,phi,gamma_;

// Nelson-Siegel parameters

let params = 0.05 -0.05 0.06 10;

// CDS parameters

phi = 0.05;
gamma_ = 0.10;
Recovery = 0.40;
Notional = 1e6;
let T = 0.5 1 2 3 5 7 10;
nT = rows(T);

let params = 0.05 -0.05 0.06 10;

_Continuous_Time = 1;

proc DF(t);
  retp( NelsonSiegel_ZC_Bond(params,t) );
endp;

proc Hazard_Rate(t);
  retp( phi .* gamma_ .* exp(gamma_ .* t) );
endp;

proc Survival_Fn(t);
  retp( exp(phi .* (1 - exp(gamma_ .* t)) ) );
endp;

proc Density_Fn(t);
  retp( phi .* gamma_ .* exp(gamma_ .* t) .* exp(phi .* (1 - exp(gamma_ .* t)) ) );
endp;

u = seqa(0,10/100,101);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10; _pcolor = 1; _pltype = 6;
  xy(u,1e4*Hazard_Rate(u));

_intord = 128;

Results = zeros(nT,4);
i = 1;
do until i > nT;
  Maturity = T[i];
  t_m = seqa(0.25,0.25,4*Maturity);
  c = 10/1e4;
  {Price,Spread,RPV01} = CDS_Price(&DF,&Survival_Fn,&Density_Fn,Notional,Recovery,c,Maturity,t_m);
  Results[i,1 3 4] = Price~1e4*Spread~RPV01;
  c = 100/1e4;
  {Price,Spread,RPV01} = CDS_Price(&DF,&Survival_Fn,&Density_Fn,Notional,Recovery,c,Maturity,t_m);
  Results[i,2] = Price;  
  i = i + 1;
endo;

print ftosa(T~Results,"%lf",3,0~0~0~2~3);

t_m = CDS_Make_TmTable(T);
Spread = Results[.,3]/1e4;

lambda = CDS_Calibrate_Lambda(&DF,Notional,Recovery,T,t_m,Spread);

print ftosa(T~Results~1e4*lambda,"%lf",3,0~0~0~2~3~2);



