/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 3.1.3
**
*/

new;
library frm,pgraph,optmum;
frmSet;

declare matrix lambda,gamma_;

let params = 0.05 -0.05 0.06 10;

_Continuous_Time = 1;

proc DF(t);
  retp( NelsonSiegel_ZC_Bond(params,t) );
endp;

lambda = 0.05;
gamma_ = 0.10;

proc hazard(t);
  retp( lambda .* gamma_ .* exp(gamma_ .* t) );
endp;

proc grad_hazard(t);
  retp( lambda .* gamma_^2 .* exp(gamma_ .* t) );
endp;

proc Survival_Fn(t);
  retp( exp(-hazard(t) .* t) );
endp;

proc Density_Fn(t);
  retp( (hazard(t) + (grad_hazard(t) .* t))  .* exp(-hazard(t) .* t) );
endp;

_intord = 128;

Notional = 1e6;
Recovery = 0.40;
let T = 0.5 1 2 3 5 7 10;
nT = rows(T);

Price = zeros(nT,1);
Fair_Spread = zeros(nT,1);
RPV01 = zeros(nT,1);

Spread = 10/1e4;
i = 1;
do until i > nT;
  Maturity = T[i];
  t_m = seqa(0.25,0.25,4*Maturity);
  {Price[i],Fair_Spread[i],RPV01[i]} = CDS_Price(&DF,&Survival_Fn,&Density_Fn,Notional,Recovery,Spread,Maturity,t_m);
  i = i + 1;
endo;
MtM = Notional .* (Fair_Spread - Spread) .* RPV01;
Fair_Spread = 1e4 * Fair_Spread;

Results = Price~Fair_Spread~RPV01~MtM;

Spread = 100/1e4;
i = 1;
do until i > nT;
  Maturity = T[i];
  t_m = seqa(0.25,0.25,4*Maturity);
  {Price[i],Fair_Spread[i],RPV01[i]} = CDS_Price(&DF,&Survival_Fn,&Density_Fn,Notional,Recovery,Spread,Maturity,t_m);
  i = i + 1;
endo;
MtM = Notional .* (Fair_Spread - Spread) .* RPV01;
Fair_Spread = 1e4 * Fair_Spread;

Results = Results~Price~Fair_Spread~RPV01~MtM;

print ftosa(T~Results,"%lf",3,2);

t = seqa(0,10/100,101);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10; _pcolor = 1; _pltype = 6;
  xy(t,1e4*hazard(t));
