/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 3.3.1
**
*/

new;
library frm,optmum,pgraph;

cls;

__output = 0;

n = 1000;
let dt   =    3    6   9  12  15  18  21  24;  
let n_dt =    2    5   9  12  16  20  25  29;
S_hat = 1 - n_dt/n;

t = dt/12;

print ftosa(t~S_hat,"%lf",3,3);

proc rss1(theta);
  local lambda,S,u;
  lambda = sqrt(theta^2);
  S = survivalExponential(t,lambda);
  u = S - S_hat;
  retp(u'u);
endp;

{theta,fmin,grd,retcode} = optmum(&rss1,0.1);
lambda1 = sqrt(theta^2);
PD1 = 1 - exp(-lambda1);

print;
print ftos(100*lambda1,"lambda = %lf",3,3);

proc Hazard_Function2(t,lambda,gamma_);
  retp( lambda .* gamma_ .* exp(gamma_ .* t) );
endp;

proc Survival_Function2(t,lambda,gamma_);
  retp( exp(lambda .* (1 - exp(gamma_ .* t))) );
endp;

proc rss2(theta);
  local lambda,gamma_,S,u;
  lambda = sqrt(theta[1]^2);
  gamma_ = sqrt(theta[2]^2);
  S = Survival_Function2(t,lambda,gamma_);
  u = S - S_hat;
  retp(u'u);
endp;

{theta,fmin,grd,retcode} = optmum(&rss2,0.1|1);
lambda2 = sqrt(theta[1]^2);
gamma2 = sqrt(theta[2]^2);
PD2 = 1 - Survival_Function2(1,lambda2,gamma2);

print;
print ftos(100*lambda2,"lambda = %lf",3,3);
print ftos(gamma2,"gamma = %lf",3,3);

proc rss3(theta);
  local lambda,S,u;
  lambda = sqrt(theta^2);
  S = survivalExponential(t,t~lambda);
  u = S - S_hat;
  retp(u'u);
endp;

{theta,fmin,grd,retcode} = optmum(&rss3,1-S_hat);
lambda3 = sqrt(theta^2);
PD3 = 1 - survivalExponential(1,t~lambda3);
print;
print ftosa(100*lambda3,"%lf",3,3);

PD = PD1 | PD2 | PD3;
print ftosa(100*PD,"%lf",3,3);

x = seqa(0,1/100,301);
nx = rows(x);
h1 = lambda1 .* ones(nx,1);
h2 = Hazard_Function2(x,lambda2,gamma2);
pline3 = xy_piecewise_constant_function(t,100*lambda3,6,2,10);


graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10; _pcolor = 1|4|2; _pltype = 3|1|6;
  _pline = pline3;
  ytics(0.5,2.5,0.5,2);
  xtics(0,2.5,0.5,2);
  xlabel("\216t");
  ylabel("\216\202l\201(t) (in %)");
  _plegstr = "Exponential\000Gompertz\000Piecewise exponential";
  _plegctl = {2 5 5 1.5};
  xy(x,100*(h1~h2~-h2));



