/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 3.3.3
**
*/

new;
library frm,pgraph,optmum;
frmSet;

cls;

declare matrix _param_lambda,Spread;

// Nelson-Siegel parameters

let params = 0.05 -0.05 0.06 10;

// CDS parameters

Recovery = 0.40;
Notional = 1e6;

let Maturity = 1 3 5 7 10;
M = rows(Maturity);

_Continuous_Time = 1;

proc DF(t);
  retp( NelsonSiegel_ZC_Bond(params,t) );
endp;

proc Survival_Fn(t);
  retp( survivalExponential(t,_param_lambda) );
endp;

proc Density_Fn(t);
  retp( pdfExponential(t,_param_lambda) );
endp;

proc (1) = BootProc(theta);
  local lambda,nCDS,c,i,t_m,price,RPV01,diff;

  lambda = sqrt(theta^2);
  _param_lambda = Maturity~lambda;

  nCDS = rows(Spread);
  c = zeros(nCDS,1);

  i = 1;
  do until i > nCDS;
    t_m = seqa(0.25,0.25,4*Maturity[i]);
    {price,c[i],RPV01} = CDS_Price(&DF,&Survival_Fn,&Density_Fn,Notional,Recovery,0,Maturity[i],t_m);
    i = i + 1;
  endo;

  diff = c - Spread;

  retp(sumc(diff^2));
endp;

t = seqa(0,1/10,101);

_opgtol = 1e-7;

let Spread = 50 60 70 80 90;
Spread = Spread/1e4;
sv = Spread ./ (1 - Recovery);
{theta,fmin,grd,retcode} = optmum(&BootProc,sv);
lambda = sqrt(theta^2);
_param_lambda = Maturity~lambda;
S1 = Survival_Fn(t);
lambda1 = lambda;
Spread1 = Spread;

let Spread = 50 60 90 115 125;
Spread = Spread/1e4;
sv = Spread ./ (1 - Recovery);
{theta,fmin,grd,retcode} = optmum(&BootProc,sv);
lambda = sqrt(theta^2);
_param_lambda = Maturity~lambda;
S2 = Survival_Fn(t);
lambda2 = lambda;
Spread2 = Spread;

let Spread = 350 370 390 385 370;
Spread = Spread/1e4;
sv = Spread ./ (1 - Recovery);
{theta,fmin,grd,retcode} = optmum(&BootProc,sv);
lambda = sqrt(theta^2);
_param_lambda = Maturity~lambda;
S3 = Survival_Fn(t);
lambda3 = lambda;
Spread3 = Spread;

print ftosa(Maturity~1e4*(Spread1~Spread2~Spread3),"%lf",3,1);

print ftosa(Maturity~1e4*(lambda1~lambda2~lambda3),"%lf",3,1);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3; _pcolor = 1|4|2; _plwidth = 10;
  xlabel("\216t (in years)");
  ylabel("\216S(t)");
  ytics(0.50,1.00,0.10,2);
  _plegstr = "#1\000#2\000#3";
  _plegctl = {2 5 2 1.5};
  xy(t,S1~S2~S3);

