/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Exercise 3.4.4
**
*/

new;
library frm,pgraph,optmum;
frmSet;

cls;

declare matrix r,lambda1,lambda2,lambda3,Recovery,Sp3,Sp5,Sp7;

print;
print "=====================  Question 4.b  =======================================";

proc pdf(t);
  local e1,e2,e3,f;
  e1 = (t .<= 3);
  e2 = (t .> 3) .and (t .<= 5);
  e3 = (t .> 5);
  
  f = lambda1 .* exp(-lambda1 .* t) .* e1 +
      lambda2 .* exp(-3*lambda1 - lambda2 .* (t - 3)) .* e2 +
      lambda3 .* exp(-3*lambda1 - 2*lambda2 - lambda3 .* (t - 5)) .* e3;

  retp(f);
endp;

proc sdf(t);
  local e1,e2,e3,S;
  e1 = (t .<= 3);
  e2 = (t .> 3) .and (t .<= 5);
  e3 = (t .> 5);
  
  S = exp(-lambda1 .* t) .* e1 +
      exp(-3*lambda1 - lambda2 .* (t - 3)) .* e2 +
      exp(-3*lambda1 - 2*lambda2 - lambda3 .* (t - 5)) .* e3;

  retp(S);
endp;

proc intgr(t_star,T);
  local lambda,I;

  if t_star == 0;
    lambda = lambda1;
  elseif t_star == 3;
    lambda = lambda2;
  elseif t_star == 5;
    lambda = lambda3;
  endif;

  I = sdf(t_star) .* (exp(-r .* t_star) - exp(-r .* T) .* exp(-lambda .* (T - t_star))) ./ (r + lambda); 

  retp(I);
endp;

r = 0.05;
lambda1 = 0.0167;
lambda2 = 0.0401;
lambda3 = 0.0322;

/*
lambda1 = 0.0100;
lambda2 = 0.0200;
lambda3 = 0.0300;
*/

Recovery = 0.40;

I_03 = intgr(0,3);
I_35 = intgr(3,5);
I_57 = intgr(5,7);

proc procF(t);
  retp( exp(-r .*t) .* pdf(t));
endp;

proc procS(t);
  retp( exp(-r .*t) .* sdf(t));
endp;

results = (I_03~quadLegendre1(&procS,3|0,128)) |
          (I_35~quadLegendre1(&procS,5|3,128)) |
          (I_57~quadLegendre1(&procS,7|5,128)) ;

print ftosa(results,"%lf",3,5);

S3 = (1 - Recovery) * lambda1;
S5 = (1 - Recovery) * (lambda1 * I_03 + lambda2 * I_35) / (I_03 + I_35);
S7 = (1 - Recovery) * (lambda1 * I_03 + lambda2 * I_35 + lambda3 * I_57) / (I_03 + I_35 + I_57);

Sp3 = (1 - Recovery) * quadLegendre1(&procF,3|0,128) / quadLegendre1(&procS,3|0,128);
Sp5 = (1 - Recovery) * (quadLegendre1(&procF,3|0,128) + quadLegendre1(&procF,5|3,128)) / 
                       (quadLegendre1(&procS,3|0,128) + quadLegendre1(&procS,5|3,128));
Sp7 = (1 - Recovery) * (quadLegendre1(&procF,3|0,128) + quadLegendre1(&procF,5|3,128) + quadLegendre1(&procF,7|5,128)) / 
                       (quadLegendre1(&procS,3|0,128) + quadLegendre1(&procS,5|3,128) + quadLegendre1(&procS,7|5,128));

Results = (S3~Sp3) |
          (S5~Sp5) |
          (S7~Sp7) ;

print ftosa(1e4*results,"%lf",3,5);

print;
print "=====================  Question 4.e  =======================================";

proc minProc(lambda);
  local I_03,I_35,I_57;
  local S3,S5,S7,S,Sp,u;

  lambda1 = lambda[1];
  lambda2 = lambda[2];
  lambda3 = lambda[3];

  I_03 = intgr(0,3);
  I_35 = intgr(3,5);
  I_57 = intgr(5,7);

  S3 = (1 - Recovery) * lambda1;
  S5 = (1 - Recovery) * (lambda1 * I_03 + lambda2 * I_35) / (I_03 + I_35);
  S7 = (1 - Recovery) * (lambda1 * I_03 + lambda2 * I_35 + lambda3 * I_57) / (I_03 + I_35 + I_57);

  S = S3|S5|S7;
  Sp = Sp3|Sp5|Sp7;
  u = 1e4 * (S - Sp);

  retp(sumc(u^2));
endp;

r = 0.05;
Recovery = 0.40;
Sp3 = 0.0100;
Sp5 = 0.0150;
Sp7 = 0.0160;

sv = (Sp3|Sp5|Sp7)/(1 - recovery);

__output = 0;
_opgtol = 1e-6;
{lambda,fmin,grd,retcode} = optmum(&minProc,sv);

print ftosa(1e4 * lambda,"%lf",3,2);

lambda1 = Sp3/(1 - Recovery);
lambda2 = (5 * Sp5/(1 - Recovery) - 3 * Sp3/(1 - Recovery))/2;
lambda3 = (7 * Sp7/(1 - Recovery) - 5 * Sp5/(1 - Recovery))/2;

Results = lambda ~ (lambda1|lambda2|lambda3);
print ftosa(1e4 * Results,"%lf",3,1);

print;
print "=====================  Question 4.f  =======================================";

t_star = 3|5|1000;
params = t_star~lambda;

S_star = survivalExponential(t_star,params);
print ftosa(S_star,"%lf",3,3);

let u = 0.96 0.23 0.90 0.80;
t = invExponential(1-u,params);
S = survivalExponential(t,params);
print ftosa(u~t~S,"%lf",3,5);

t1 = -ln(u[1])/lambda[1];
t2 = 5 + (ln(S_star[2]) - ln(u[2]))/lambda[3];
t3 = 3 + (ln(S_star[1]) - ln(u[3]))/lambda[2];
t4 = 5 + (ln(S_star[2]) - ln(u[1]))/lambda[3];
t = t1|t2|t3|t4;
print ftosa(u~t~S,"%lf",3,5);



