/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 4.1.2
**
*/

new;
library frm,pgraph,option,optmum;
frmSet;

cls;

rndseed 123456;

nb = 1000;

S0 = 100;
K = 100;
tau = 1;
mu = 0.15;
sigma = 0.20;
r = 0.05;

C0 = EuropeanBS(S0,K,sigma,Tau,r,r);

nt = 12*20;
t = seqa(0,1/nt,nt+1);
nT = rows(t);

St = simulate_GBM(S0,mu,sigma,t,1);
Ct = EuropeanBS(St,K,sigma,Tau-t,r,r);
MtM = nb * (Ct - C0);
EAD = _max_(MtM,0);

Indx_M1 = 6*20+1;
Indx_M2 = 9*20+1;
Indx_Months = Indx_M1 | Indx_M2;

Results = t[Indx_Months]~St[Indx_Months]~Ct[Indx_Months]~MtM[Indx_Months]~EAD[Indx_Months];
print ftosa(Results,"%lf",3,2~2~2~0~0);

t_0_M1 = t[1:indx_M1];
S_0_M1 = St[1:Indx_M1];
S_M1 = St[Indx_M1];
t1 = t[indx_M1:nT];

t_0_M2 = t[1:indx_M2];
S_0_M2 = St[1:Indx_M2];
S_M2 = St[Indx_M2];
t2 = t[indx_M2:nT];

nS = 10000;
S1 = simulate_GBM(S_M1,r,sigma,t1-t1[1],nS);
{x1_t,y1_S} = Merge_Time_Series(t_0_M1,S_0_M1,t1,S1,0);

ST = S1[rows(S1),.]';
C1 = EuropeanBS(S_M1,K,sigma,Tau-t1[1],r,r);
MtM1 = nb * (C1 - C0);
Ct = _max_(ST - K,0);
MtM_Future = nb * (Ct - C1);
MtM1s = MtM1 + MtM_Future;

min_MtM = -nb * C0 / 1e4;
MtM1s = MtM1s/1e4;
x1_MtM = seqa(-2.0,12/100,101);
{y1_MtM,F_MtM} = regKernelDensity2(MtM1s,x1_MtM,min_MtM,error(0));

e1s = _max_(MtM1s,0);
x1_e = seqa(-2,12/100,101);
{y1_e,F_e} = regKernelDensity2(e1s,x1_e,0,error(0));

S2 = simulate_GBM(S_M2,r,sigma,t2-t2[1],nS);
{x2_t,y2_S} = Merge_Time_Series(t_0_M2,S_0_M2,t2,S2,0);

ST = S2[rows(S2),.]';
C2 = EuropeanBS(S_M2,K,sigma,Tau-t2[1],r,r);
MtM2 = nb * (C2 - C0);
Ct = _max_(ST - K,0);
MtM_Future = nb * (Ct - C1);
MtM2s = MtM2 + MtM_Future;

MtM2s = MtM2s/1e4;
x2_MtM = seqa(-2.0,12/100,101);
{y2_MtM,F_MtM} = regKernelDensity2(MtM2s,x2_MtM,min_MtM,error(0));

e2s = _max_(MtM2s,0);
x2_e = seqa(-2,12/100,101);
{y2_e,F_e} = regKernelDensity2(e2s,x2_e,0,error(0));

graphset;
  window3u;

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10 | 5 * ones(nS,1);
  _pcolor = 1|reshape(2|3|4|5,nS,1);
  _paxht = 0.225; _pnumht = 0.225; _ptitlht = 0.275;

  setwind(1);
    title("\216Simulated scenarios");
    xlabel("\216t");
    ylabel("\216S(t)");
    xtics(0,1,0.2,2);
    ytics(50,200,50,2);
    _pline = 1~1~0.50~0~0.50~200~1~0~10 |
             1~1~0.75~0~0.75~200~1~0~10 ;
    xy(x2_t[.,1:100],y2_S[.,1:100]);

  _pcross = 1; _pline = 0;

  setwind(2);
    title("\216Density of the mark-to-market");
    xlabel("\216MtM(0;1) (\202\42\201 10[4])");
    ylabel("");
    _pltype = 1; _pcolor = 4;
    xtics(-2,10,2,2);
    ytics(0,0.30,0.10,2);
    xy(x2_MtM,missrv(y2_MtM,0));

  setwind(3);
    title("\216Density of the credit exposure");
    xlabel("\216e(1) (\202\42\201 10[4])");
    _pcolor = 2; 
    xy(x2_e,missrv(y2_e,0));

  endwind;

