/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 4.1.2
**
*/

new;
library frm,pgraph,option,optmum;
frmSet;

cls;

nb = 1000;

S0 = 100;
K = 100;
tau = 1;
mu = 0.15;
sigma = 0.20;
r = 0.05;

C0 = EuropeanBS(S0,K,sigma,Tau,r,r);

nt = 12*20;
t = seqa(0,1/nt,nt+1);
nT = rows(t);

nS = 10000;

rndseed 123456;
St = simulate_GBM(S0,mu,sigma,t,1);
Ct = EuropeanBS(St,K,sigma,Tau-t,r,r);
MtM = nb * (Ct - C0);
EAD = _max_(MtM,0);

results = zeros(nT,3);
results[.,1] = EAD;

i = 1;
do until i > nT;
  dt = tau - t[i];
  u = rndn(nS,1);
  S_tau = St[i] .* exp( (r-0.5*sigma^2)*dt + sigma*sqrt(dt) .* u);
  C_tau = EuropeanBS(S_tau,K,sigma,dt,r,r);
  MtM = nb * (C_tau - C0);
  e = _max_(MtM,0.0);
  results[i,2] = meanc(e);
  results[i,3] = quantile(e,0.95);

  i = i + 1;
endo;

S1 = St; result1 = results;

rndseed 12456;
St = simulate_GBM(S0,-mu,sigma,t,1);
Ct = EuropeanBS(St,K,sigma,Tau-t,r,r);
MtM = nb * (Ct - C0);
EAD = _max_(MtM,0);

results = zeros(nT,3);
results[.,1] = EAD;

i = 1;
do until i > nT;
  dt = tau - t[i];
  u = rndn(nS,1);
  S_tau = St[i] .* exp( (r-0.5*sigma^2)*dt + sigma*sqrt(dt) .* u);
  C_tau = EuropeanBS(S_tau,K,sigma,dt,r,r);
  MtM = nb * (C_tau - C0);
  e = _max_(MtM,0.0);
  results[i,2] = meanc(e);
  results[i,3] = quantile(e,0.95);

  i = i + 1;
endo;

S2 = St; result2 = results;

graphset;
  begwind;
  window(2,2,0);

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  _paxht = 0.225; _pnumht = 0.225; _ptitlht = 0.275;

  setwind(1);
    title("\216Evolution of the asset price");
    xlabel("\216t");
    ylabel("\216S(t)");
    xtics(0,1,0.2,2);
    ytics(70,140,10,2);
    xy(t,S1);

  setwind(2);
    xy(t,S2);

  setwind(3);
    _pysci = 1;
    _pcolor = 2|3|4;
    title("\216Evolution of the credit exposure");
    xlabel("\216t");
    ylabel("\216e(t)");
    xtics(0,1,0.2,2);
    ytics(0,60000,10000,2);
    xy(t,result1);

  setwind(4);
    _plegstr = "Current\000Expected\00095% quantile";
    _plegctl = {2 5 5 4};
    xy(t,result2);

  endwind;

 
 

