/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 4.2.4
**
*/

new;
library frm,pgraph,optmum;

cls;

proc (2) = counterparty_collateral(MtM,delta_C,H);
  local nDates,nCols,e,C,t;

  nDates = rows(MtM);
  nCols = cols(MtM);
  e = zeros(nDates,nCols);
  C = zeros(nDates,nCols);

  t = 1;
  do until t > nDates;
    if t > delta_C;
      C[t,.] = _max_(MtM[t-delta_C,.] - H,0);
    endif;
    e[t,.] = _max_(MtM[t,.] - C[t,.],0);
    t = t + 1;
  endo;

  retp(e,C);
endp;

rndseed 123;
t = seqa(0,1/260,2*260);
MtM = 20 * simulate_BM(t,1);

H = 5;
delta_C = 20;
{e1,C1} = counterparty_collateral(MtM,0,1000);
{e2,C2} = counterparty_collateral(MtM,delta_C,H);
{e3,C3} = counterparty_collateral(MtM,0,H);
{e4,C4} = counterparty_collateral(MtM,delta_C,0);

graphset;
  begwind;
  window(4,2,0);

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  _paxht = 0.45; _pnumht = 0.35; _ptitlht = 0.55;
  xtics(0,2,0.5,2);
  ytics(-10,40,10,2);

  setwind(1);
    _pcolor = 1;
    title("\216MtM(t)");
    ylabel("\216MtM(t)");
    ytics(-10,40,10,2);
    xy(t,MtM);

  nextwind;
    _pcolor = 4;
    title("\216No collateral");
    ylabel("\216e[+](t)");
    ytics(0,40,10,2);
    xy(t,e1);

  nextwind;
    _pcolor = 2;
    title("\216\202d\201]C[=0, H=5");
    ylabel("\216C(t)");
    ytics(0,40,10,2);
    xy(t,C3);

  nextwind;
    _pcolor = 3;
    ylabel("\216e[+](t)");
    ytics(0,15,5,2);
    xy(t,e3);

  nextwind;
    _pcolor = 5;
    title("\216\202d\201]C[=20, H=0");
    ylabel("\216C(t)");
    ytics(0,40,10,2);
    xy(t,C4);

  nextwind;
    _pcolor = 6;
    ylabel("\216e[+](t)");
    ytics(0,15,5,2);
    xy(t,e4);

  nextwind;
    _pcolor = 7;
    title("\216\202d\201]C[=20, H=5");
    ylabel("\216C(t)");
    ytics(0,40,10,2);
    xy(t,C2);

  nextwind;
    _pcolor = 8;
    ylabel("\216e[+](t)");
    ytics(0,15,5,2);
    xy(t,e2);


  endwind;