/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 4.2.4
**
*/

new;
library frm,pgraph,optmum;

cls;

let mu_t    = 1 1;
let sigma_t = 1 1;
H = 0;

n = rows(mu_t);
w = ones(n,1);
params = seqa(-4,0.1,80);
nIters = rows(params);

rho1 = 0.00;
results = zeros(nIters,2+1+n+3);

iter = 1;
do until iter > nIters;
  mu_t[2] = params[iter];

  rho = diagrv(rho1*ones(n,n),1.0);
  {mu_w,sigma_w,gamma_t,e_t,ec_mu,ec_sigma,ec_H,ec_t} = counterparty_collateral_EpE(w,mu_t,sigma_t,rho,H);
  ec_t = ec_t/e_t;
  results[iter,.] = mu_w~sigma_w~e_t~100*ec_t'~ec_mu~ec_sigma~ec_H;

  iter = iter + 1;
endo;

graphset;
  begwind;
  window(2,2,0);

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0; _pcross = 1;
  _plwidth = 10; _pcolor = 1|4|2; _pltype = 6|1|3;
  _pnumht = 0.225; _paxht = 0.225; _ptitlht = 0.275;
  xtics(-4,4,2,1);
  xlabel("\216\202m\201]2[(t)");

  indx = 3|(3+n+1)|(3+n+2);
  setwind(1);
    title("\216Counterparty exposure");
    _plegstr = "EpE(t;w)\000EpE]\202m\201[(t;w)\000EpE]\202s\201[(t;w)";
    _plegctl = {2 5 1 4};
    xy(params,results[.,indx]);

  indx = (3+1)|(3+2);
  setwind(2);
    title("\216Risk contribution (in %)");
    ytics(-200,200,100,2);
    _plegstr = "EpE]1[(t;w)\000EpE]2[(t;w)";
    _plegctl = {2 5 5.5 1.25};
    xy(params,results[.,indx]);

  _plegctl = 0;

  indx = 1;
  setwind(3);
    title("\216Portfolio mean");
    ylabel("\216\202m\201]w[(t)");
    ytics(-3,6,3,2);
    xy(params,results[.,indx]);

  indx = 2;
  setwind(4);
    title("\216Portfolio volatility");
    ylabel("\216\202s\201]w[(t)");
    ytics(0,2,0.5,2);
    xy(params,results[.,indx]);

  endwind;