new;
library frm,pgraph,optmum;
frmSet;

declare matrix t;

Notional = 1000;
F0 = 0.05;
mu = 0.02;
sigma = 0.25;
r = 0.05;
Maturity = 10;

lambda = 0.01;
Recovery = 0.50;
Spread = (1 - Recovery) * lambda;

proc CVA_integral1(u);
  local psi_u,EpE,g_u;
  
  psi_u = (1 - exp(-r*(Maturity - u))) ./ r;
  EpE = Notional .* F0 .* psi_u .* (exp(mu .* u) .* cdfn( (mu./sigma + 0.5 * sigma) .* sqrt(u)) - 
        cdfn( (mu./sigma - 0.5 * sigma) .* sqrt(u)));
  g_u = exp(-(r+lambda) .* (u-t)) .* EpE; 
  
  retp(g_u);
endp;

proc CVA_Approximation(t);
  local A1,A2,CVA;
  
  A1 = pdfn(mu ./ sigma .* sqrt(t)) .* (Maturity/3);
  A2 = cdfn(mu ./ sigma .* sqrt(t)) .* (Maturity/2);

  CVA = Spread .* Notional .* F0 .* (A1 .* sigma .* (4/15) .* (Maturity^2.5) + A2 .* mu .* (1/6) .* (Maturity .^3));
  
  retp(CVA);
endp;

all_t = seqa(0,Maturity/200,201);
nT = rows(all_t);
CVA = zeros(nT,2);

i = 1;
do until i > nT;
  t = all_t[i];
  CVA[i,1] = quadLegendre1(&CVA_integral1,Maturity|t,128);  
  CVA[i,2] = CVA_Approximation(t);    
  i = i + 1;
endo;

CVA = Spread .* CVA;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|3; _plwidth = 10; _pcolor = 1|4;
  xlabel("\216t (in years)");
  ylabel("\216CVA(t)");
  xtics(0,10,1,2);  
  ytics(0,2,0.5,2);
  _plegstr = "Exact solution\000Approximation";
  _plegctl = {2 5 6 5};
  xy(all_t,CVA);




  
  
  
