/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 5.3.2
**
*/

new;
library frm,pgraph,optmum;
frmSet;

cls;

declare matrix method;

let loss = 10.1 12.5 14 25 317.3 353 1200 1254 52000 251000;
loss = 1e3 * loss;

proc mlProc(theta);
  local alpha,beta,mu,sigma,pdf,LogL;

  theta = sqrt(theta^2);

  if method == 0;          // Gamma
    alpha = theta[1]; 
    beta = theta[2];
    pdf = pdfGamma(loss,alpha,beta);
  elseif method == 1;     // Log-gamma
    alpha = theta[1]; 
    beta = theta[2];
    pdf = pdfLogGamma(loss,alpha,beta);
  elseif method == 2;     // Log-logistic
    alpha = theta[1]; 
    beta = theta[2];
    pdf = pdfLogLogistic(loss,alpha,beta);
  elseif method == 3;     // Log-normal
    mu = theta[1]; 
    sigma = theta[2];
    pdf = pdfLN(loss,mu,sigma);
  elseif method == 4;     // Pareto
    alpha = theta[1]; 
    beta = theta[2];
    pdf = pdfPareto(loss,alpha,beta);
  endif;

  LogL = sumc(ln(pdf));

  retp(-logl);
endp;

proc gmmProc(theta);
  local alpha,beta,mu,sigma,m1,m2,h1,h2;

  theta = sqrt(theta^2);

  if method == 0;          // Gamma
    alpha = theta[1]; 
    beta = theta[2];
    {m1,m2} = momentGamma(alpha,beta);
  elseif method == 1;     // Log-gamma
    alpha = theta[1]; 
    beta = theta[2];
    {m1,m2} = momentLogGamma(alpha,beta);
  elseif method == 2;     // Log-logistic
    alpha = theta[1]; 
    beta = theta[2];
    {m1,m2} = momentLogLogistic(alpha,beta);
  elseif method == 3;     // Log-normal
    mu = theta[1]; 
    sigma = theta[2];
    {m1,m2} = momentLN(mu,sigma);
  elseif method == 4;     // Pareto
    alpha = theta[1]; 
    beta = theta[2];
    {m1,m2} = momentPareto(alpha,beta);
  endif;

  h1 = loss - m1;
  h2 = (loss - m1)^2 - m2;

  retp(h1~h2);
endp;

let sv[2,4] = 2 2 2  3 
              2 2 2  12;
theta_ml = miss(zeros(2,4),0);
theta_gmm = miss(zeros(2,4),0);
retcode_ml = zeros(4,1);

_opgtol = 1e-6;
method = 1;
do until method > 4;
  {theta,fmin,grd,retcode_ml[method]} = optmum(&mlProc,sv[.,method]);
  theta_ml[.,method] = theta;
  method = method + 1;
endo;

print ftosa(theta_ml,"%lf",3,2);

__output = 0;
all_theta = {};

let sv[2,4] = 2 2 8  3.2 
              2 2 2  12000;

method = 3;
do until method > 3;
  theta = sv[.,method] .* ones(1,5);
  iter = 2;
  do until iter > 5; 
    sv_gmm = theta[.,iter-1];
    h0 = gmmProc(sv_gmm);
    cov = (h0'h0)/rows(h0);
    _regGMM_invW = inv(cov);
    {theta[.,iter],stderr,cov,Qmin} = regGMM(0,0,&gmmProc,sv_gmm,0,0);
    iter = iter + 1;
  endo;
  all_theta = all_theta | theta;
  theta_gmm[.,method] = theta[.,5];
  method = method + 1;
endo;

print;
print "---------------------------------------------------";
print ftosa(theta_ml,"%lf",3,2);
print "---------------------------------------------------";
print ftosa(theta_gmm,"%lf",3,2);
print "---------------------------------------------------";
print ftosa(all_theta,"%lf",3,2);
print "---------------------------------------------------";


