/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 5.3.2
**
*/

new;
library frm,pgraph;
frmSet;

cls;

mu = 8;
sigma = sqrt(5);
x = seqa(-5,0.1,50) | seqa(0.005,0.005,499) | seqa(2.5,0.1,700);
x = exp(x);
cdf0 = cdfLN(x,mu,sigma);
pdf0 = pdfLN(x,mu,sigma);
H = 1e4;
cdf1 = cdfLN_Threshold(x,mu,sigma,H);
pdf1 = pdfLN_Threshold(x,mu,sigma,H);

indx = selif(seqa(1,1,rows(x)),x .<= H);
e = ones(rows(indx),1);
pline = e.*(1~6)~ln(x[indx])~e*0~ln(x[indx])~cdf0[indx]~e.*(1~0~0);

{m1,m2} = momentLN(mu,sigma);
print ftosa(m1~m2,"%lf",3,2);
{m1c,m2c} = momentLN_Threshold(mu,sigma,0.01);
print ftosa(m1c~m2c,"%lf",3,2);

let H = 1000 10000 20000 50000;
{m1c,m2c} = momentLN_Threshold(mu,sigma,H);
print ftosa(H~(m1c~m2c)./(m1~m2),"%lf",3,2);

H = seqa(0.001,100,500);
{m1,m2} = momentLN_Threshold(mu,sigma,H);

graphset;
  begwind;
  window(2,2,0);
  
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0; 
  _paxht = 0.225; _pnumht = 0.225; _ptitlht = 0.275;
  _pltype = 6|3; _plwidth = 10; _pysci = 0;

  setwind(1);
    title("\216Cumulative distribution function");
    _pcolor = 2|4;
    xtics(0,20,5,5);
    ytics(0,1,0.20,2);
    xlabel("\216ln X");
    _pline = pline;
    _pmsgstr = "H = 10000";
    _pmsgctl = 3.0+ln(1000)~0.05~0.20~0~1~0~8;
    xy(ln(x),cdf0~cdf1);

  _pline = 1~6~0.05+ln(1e4)~0~0.05+ln(1e4)~5e-5~1~0~6;
  _pmsgctl = 9.5~4.5e-5~0.20~0~1~0~8;

  setwind(2);
    title("\216Probability density function");
    _pcolor = 2|4;
    xtics(9.00,15,1,5);
    ytics(0,5e-5,1e-5,0);
    xlabel("\216ln X");
    _plegstr = "Without truncation\000With truncation";
    _plegctl = {2 5 4.5 3.5};
    xy(ln(x),missex(pdf0,pdf1 .== error(0))~pdf1);

  _pline = 0; _pmsgctl = 0; _plegctl = 0;

  setwind(3);
    title("\216Mean");
    _pcolor = 1;
    xtics(0,5e4,1e4,2);
    ytics(0,3e5,1e5,2);
    xlabel("\216H");
    xy(H,m1);

  setwind(4);
    title("\216Variance");
    ytics(0,2e12,0.5e12,2);
    xy(H,m2);

  endwind;