/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 5.3.2
**
*/

new;
library frm,pgraph,optmum;
frmSet;

cls;

declare matrix method;

let loss = 10.1 12.5 14 25 317.3 353 1200 1254 52000 251000;
loss = 1e3 * loss;
H = 5000 * ones(rows(loss),1);

proc mlProc(theta);
  local mu,sigma,pdf,LogL;

  theta = sqrt(theta^2);
  mu = theta[1]; 
  sigma = theta[2];

  if method == 1;
    pdf = pdfLN(loss,mu,sigma);
  else;
    pdf = pdfLN_Threshold(loss,mu,sigma,H);
  endif;
  LogL = sumc(ln(pdf));

  retp(-logl);
endp;

proc gmmProc(theta);
  local mu,sigma,m1,m2,h1,h2;

  theta = sqrt(theta^2);
  mu = theta[1]; 
  sigma = theta[2];

  if method == 1;
    {m1,m2} = momentLN(mu,sigma);
  else;
    {m1,m2} = momentLN_Threshold(mu,sigma,H);
  endif;
  h1 = loss - m1;
  h2 = (loss - m1)^2 - m2;

  retp(h1~h2);
endp;

let sv = 8 2;
theta_ml = miss(zeros(2,2),0);
theta_gmm = miss(zeros(2,2),0);
retcode_ml = zeros(2,1);
retcode_gmm = zeros(2,1);
all_theta = {};

method = 1;
do until method > 2;
  {theta,fmin,grd,retcode_ml[method]} = optmum(&mlProc,sv);
  theta_ml[.,method] = theta;
  method = method + 1;
endo;

__output = 0;

let sv[2,2] = 10 10
               2  2;

method = 1;
do until method > 2;
  theta = sv[.,method] .* ones(1,5);
  iter = 2;
  do until iter > 5; 
    sv_gmm = theta[.,iter-1];
    h0 = gmmProc(sv_gmm);
    cov = (h0'h0)/rows(h0);
    _regGMM_invW = inv(cov);
    {theta[.,iter],stderr,mcov,Qmin} = regGMM(0,0,&gmmProc,sv_gmm,0,0);
    iter = iter + 1;
  endo;
  theta_gmm[.,method] = theta[.,5];
  retcode_gmm[method] = Qmin;
  all_theta = all_theta | theta;
  method = method + 1;
endo;

print;
print "---------------------------------------------------";
print ftosa(theta_ml,"%lf",3,2);
print "---------------------------------------------------";
print ftosa(theta_gmm,"%lf",3,5);
print "---------------------------------------------------";
print ftosa(all_theta,"%lf",3,2);
print "---------------------------------------------------";
print retcode_ml|retcode_gmm;

x = -10|-9|-8|seqa(-5,0.1,50) | seqa(0.005,0.005,499) | seqa(2.5,0.1,700);
x = exp(x);

mu = abs(theta_ml[1,.]~theta_gmm[1,.]);
sigma = abs(theta_ml[2,.]~theta_gmm[2,.]);
cdf = cdfLN(x,mu,sigma);

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0; 
  _pltype = 6|3|1|5; _plwidth = 10; _pysci = 0; _pcolor = 1|4|2|2;
  xlabel("\216ln X");  
  ytics(0,1,0.2,2);
  xtics(-5,25,5,2);
  _plegstr = "ML\000ML (with truncation)\000GMM";
  _plegctl = {2 5 1 5};
  xy(ln(x),cdf[.,1 2 3]);