/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 5.3.2
**
*/

new;
library frm,pgraph,optmum;
frmSet;

cls;

print "-------------------- ML ----------------------";

let N = 57 62 45 24 82 36 98 75 76 45;
lambda1 = meanc(N);

proc mlProc(theta);
  local r,p,pdf,logl;
  
  theta = sqrt(theta^2);
  r = theta[1];
  p = 1/(1+theta[2]);
  
  pdf = pdfNB(n,r,p);
  logl = sumc(ln(pdf));
  
  retp(-logl);
endp;
  
{r_gmm,p_gmm} = regGMM_NB(N);
sv = r_gmm~(1/p_gmm-1);
  
__output = 0;  
{theta,fmin,grd,retcode} = optmum(&mlProc,sv);  
theta = sqrt(theta^2);
r_ml = theta[1];
p_ml = 1/(1+theta[2]);

results = r_gmm~r_ml |
          p_gmm~p_ml ;
print ftosa(results,"%lf",3,4);  

x = seqa(0,1,151);
p = pdfNB(x,r_ml,p_ml);
cdf = cdfNB(x,r_ml,p_ml);
{invcdf,retcode} = invcdfNB(cdf,r_ml,p_ml);

graphset;
  xy(x,invcdf); 

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pcolor = 1; 
  _pbartyp = {6 1};
  xtics(0,150,10,2); 
  xlabel("\216Number of losses");
  ylabel("\216Probability (in %)");
  bar(x,100*p);

print "-------------------- Distribution of lambda----------------------";        
  
alpha_gamma = r_ml;
beta_gamma = (1-p_ml)/p_ml;
results = alpha_gamma |
          beta_gamma ;
print ftosa(results,"%lf",3,4);          

lambda = seqa(1,160/100,101);
pdf_lambda = pdfGamma(lambda,alpha_gamma,beta_gamma);

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pcolor = 2|0; _pltype = 6|3; _plwidth = 10;
  _pline = 1~3~lambda1~0~lambda1~1~1~0~10;
  xlabel("\216\202l\201");
  xtics(0,160,20,2);
  ytics(0,0.02,0.005,2);
  _plegstr = "Negative binomial (gamma)\000Poisson (dirac)";
  _plegctl = {2 5 5 5};
  xy(lambda,pdf_lambda~-pdf_lambda); 