/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 5.3.3
**
*/

new;
library frm,pgraph,optmum;
frmSet;

cls;

initialization = 0;
datFileName = "chap5-3-3-4";

nR = 15;
maxS = 10^6|100;

lambda = 10;
mu_LN = 5;
sigma_LN = 2.5;

if initialization == 1;
  rndseed 123;
  ratio = {};
  save ^datFileName = ratio;
endif;

mean_LN = exp(mu_LN + 0.5*sigma_LN^2);
var_LN = exp(2*mu_LN+sigma_LN^2).*(exp(sigma_LN^2)-1);

sigma_0 = sqrt(lambda) * sqrt(var_LN + mean_LN^2);

nS = log(maxS[1]);
nS = trunc(exp((seqa(0.5,(nS-0.5)/500,501))*ln(10)))|seqa(2*maxS[1],maxS[1],maxS[2]-1);

if initialization == 1;
  ratio = zeros(rows(nS),nR);
  begin_k = 0;
else;
  loadm ratio = ^datFileName;
  begin_k = cols(ratio);
  if begin_k < nR;
    ratio = ratio ~ zeros(rows(nS),nR - begin_k);
  endif;
endif;

k = begin_k + 1;
do until k > nR;

  iter1 = 1;
  do until iter1 > maxS[2];

    N = rndp(maxS[1],1,lambda);
    S = zeros(maxS[1],1);

    iter2 = 1;
    do until iter2 > maxS[1];
      if N[iter2] /= 0;
        X = rndLN(mu_LN,sigma_LN,N[iter2],1);
        S[iter2] = sumc(X);
      endif;
      iter2 = iter2 + 1;
    endo;

    if iter1 == 1;
      j = 1;
      do until j > 501;
        sigma = stdc(S[1:nS[j]]);
        ratio[j,k] = (sigma ./ sigma_0)^1;  
        j = j + 1;
      endo;
      {n_S,m_S,sigma_S} = recursive_moments(0,0,0,S);
    else;
      {n_S,m_S,sigma_S} = recursive_moments(n_S,m_S,sigma_S,S);
      ratio[500+iter1,k] = (sigma_S ./ sigma_0)^1;  
    endif;

    str = ftos(k,"Iteration = %lf",1,0) $+ " / " $+ ftos(iter1,"Simulation = %lf",1,0);
    print /flush str;
 
   if key;
     goto finish;
   endif;

   iter1 = iter1 + 1;
  endo;

  save_ratio = ratio[.,1:k];
  save ^datFileName = save_ratio;

  k = k + 1;
endo;

finish:

graphset;   
    _pdate = "";
    _pnum = 2; _pframe = 0;
    fonts("simplex simgrma");
    _pltype = 6; _plwidth = 8; _pcolor = 1|2|3|4|5|6|7|8;
    _pline = 1~3~0~1~maxS[1]*maxS[2]~1~1~0~10;
    ytics(0,5,1,2);
    xlabel("\216Number of simulations");
    logx(nS,ratio);