/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 5.3.3
**
*/

new;
library frm,pgraph,optmum;
frmSet;

cls;

rndseed 123;

// Poisson
lambda = 5~5~5~50;

// Log-normal
mu = 5~5~5~5;
sigma = 1.0~1.5~2.0~2.0;

// Discretization
let nX    = 100  100  200 1000;
let delta = 100  500 1000 1000;
let nS    = 1e6  1e6 1e6  1e6;
databuf = 0;

let x_max = 5e3 1e4 5e4 3e5;
let d_x   = 1e3 2e3 1e4 0.5e5;

let alpha = 0.90 0.95 0.99 0.995 0.999;
CaR_panjer = miss(zeros(rows(alpha),4),0);
CaR_mc = miss(zeros(rows(alpha),4),0);
CaR_mc2 = miss(zeros(rows(alpha),4),0);
 
i = 1; 
do until i > 4;

  x = seqa(delta[i],delta[i],nX[i]);
  x_m = x - delta[i]/2;
  x_p = x + delta[i]/2;
  x0 = 0;
  f0 = cdfLN(delta[i]/2,mu[i],sigma[i]);
  f = cdfLN(x_p,mu[i],sigma[i]) - cdfLN(x_m,mu[i],sigma[i]);
  {pmf_panjer,cdf_panjer} = compute_Panjer_Recursion(lambda[i],x0,x,f0,f);
  databuf = vput(databuf,(0|x)~cdf_panjer,itos("panjer",i));

  n_star = maxindc(cdf_panjer .>= alpha') - 1.0;
  n_star = miss(n_star,0);
  pos = seqa(1,1,rows(alpha));
  {pos,n_star} = packr2(pos,n_star);
  if not scalmiss(n_star);
    CaR_panjer[pos,i] = x[n_star];
  endif;

  S = rndLDA_LN(lambda[i],mu[i],sigma[i],nS[i]);
  cdf_mc = zeros(nX[i]+1,1);
  cdf_mc[1] = meanc(S .<= x0);
  j = 1;
  do until j > nX[i];
    cdf_mc[j+1] = meanc(S .<= x[j]);
    j = j + 1;
  endo;
  databuf = vput(databuf,(0|x)~cdf_mc,itos("mc",i));

  n_star = maxindc(cdf_mc .>= alpha') - 1.0;
  n_star = miss(n_star,0);
  pos = seqa(1,1,rows(alpha));
  {pos,n_star} = packr2(pos,n_star);
  if not scalmiss(n_star);
    CaR_mc[pos,i] = x[n_star];
  endif;

  CaR_mc2[.,i] = quantile(S,alpha);

  i = i + 1;
endo;

print ftosa(CaR_panjer~CaR_mc,"%lf",3,0);

print ftosa(CaR_panjer~CaR_mc2,"%lf",3,0);

graphset;
  begwind;
  window(2,2,0);

  _pdate = ""; _pnum = 2; _pframe = 0; _pxsci = 1;
  fonts("simplex simgrma");
  _pltype = 6|3; _plwidth = 10; _pcolor = 1|4;
  _pnumht = 0.22; _paxht = 0.22; _ptitlht = 0.250;
  ytics(0,1,0.2,2);
  xlabel("\216Aggregate loss");

  i = 1;
  data_panjer = vread(databuf,itos("panjer",i));
  data_mc = vread(databuf,itos("mc",i));
  setwind(i);
    str = "\202l\201 = "   $+ ftos(lambda[i],"%lf",1,0) $+ 
          ", \202m\201 = " $+ ftos(mu[i],"%lf",1,1) $+ 
          ", \202s\201 = " $+ ftos(sigma[i],"%lf",1,1) $+ 
          ", \202d\201 = " $+ ftos(delta[i],"%lf",1,0);
    title("\216" $+ str);
    xtics(0,x_max[i],d_x[i],0);
    xy(data_panjer[.,1]~data_mc[.,1],data_panjer[.,2]~data_mc[.,2]);

  i = 2;
  data_panjer = vread(databuf,itos("panjer",i));
  data_mc = vread(databuf,itos("mc",i));
  setwind(i);
    str = "\202l\201 = "   $+ ftos(lambda[i],"%lf",1,0) $+ 
          ", \202m\201 = " $+ ftos(mu[i],"%lf",1,1) $+ 
          ", \202s\201 = " $+ ftos(sigma[i],"%lf",1,1) $+ 
          ", \202d\201 = " $+ ftos(delta[i],"%lf",1,0);
    title("\216" $+ str);
    xtics(0,x_max[i],d_x[i],0);
    xy(data_panjer[.,1]~data_mc[.,1],data_panjer[.,2]~data_mc[.,2]);

  i = 3;
  data_panjer = vread(databuf,itos("panjer",i));
  data_mc = vread(databuf,itos("mc",i));
  setwind(i);
    str = "\202l\201 = "   $+ ftos(lambda[i],"%lf",1,0) $+ 
          ", \202m\201 = " $+ ftos(mu[i],"%lf",1,1) $+ 
          ", \202s\201 = " $+ ftos(sigma[i],"%lf",1,1) $+ 
          ", \202d\201 = " $+ ftos(delta[i],"%lf",1,0);
    title("\216" $+ str);
    xtics(0,x_max[i],d_x[i],0);
    xy(data_panjer[.,1]~data_mc[.,1],data_panjer[.,2]~data_mc[.,2]);

  i = 4;
  data_panjer = vread(databuf,itos("panjer",i));
  data_mc = vread(databuf,itos("mc",i));
  setwind(i);
    str = "\202l\201 = "   $+ ftos(lambda[i],"%lf",1,0) $+ 
          ", \202m\201 = " $+ ftos(mu[i],"%lf",1,1) $+ 
          ", \202s\201 = " $+ ftos(sigma[i],"%lf",1,1) $+ 
          ", \202d\201 = " $+ ftos(delta[i],"%lf",1,0);
    title("\216" $+ str);
    xtics(0,x_max[i],d_x[i],0);
    _plegstr = "Panjer\000MC";
    _plegctl = {2 5 5.5 3};
    xy(data_panjer[.,1]~data_mc[.,1],data_panjer[.,2]~data_mc[.,2]);

  endwind;    

