/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 5.4.4
**
*/

new;
library frm,pgraph;
frmSet;

cls;

rndseed 123;

nt = 251;
let rho[2,2] = 1.00 0.50
               0.50 1.00;
let sigma = 0.20 0.28;
cov = rho .* sigma .* sigma';
R = rndmvn_chol(0,cov,nt)/sqrt(260);
P = (100~250) .* cumprodc(1 + R);
P = trunc(100*P)/100;
t = seqa(1,1,nt);
data = t~P;
call SpreadSheetWrite(data,"chap5-4-4-1.xlsx","A2",1);

R = P./lag1(P) - 1;
let nS = 10000 25000;
let P0 = 105.5 353.00;
V0 = nS' * P0;
P1 = P0' .* (1 + R);
V1 = sumr(nS' .* P1);
PnL = V1 - V0;

PnLs = sortc(PnL,1);
VaR_1D = -quantile(packr(PnL),0.01);
VaR_10D = sqrt(10) * VaR_1D;
xi = 0.5;
K = (3 + xi) * VaR_10D;

print "--------------------------------";
print ftosa(PnLs[1:10],"%lf",3,2);
print "--------------------------------";
print ftos(VaR_1D,"VaR(1D) = %lf",3,2);
print ftos(VaR_10D,"VaR(10D) = %lf",3,2);
print ftos(K/1e6,"K = %lf",3,2);
print "--------------------------------";

graphset;
  window3d;
  
  _pdate = ""; _pnum = 2; _pframe = 0;
  _paxht = 0.225; _pnumht = 0.225; _ptitlht = 0.275;
  _plype = 6; _plwidth = 10; _pcolor = 1|2|4;
  xtics(0,250,50,2);
  
  setwind(1);
    title("\216Stock A");
    ytics(90,110,5,2);  
    xy(t,P[.,1]);
  
  setwind(2);  
    _pcolor = 2;
    title("\216Stock B");  
    ytics(200,360,40,2);      
    xy(t,P[.,2]);

  setwind(3);  
    _pcolor = 2; _pnumht = 0.20;
    title("\216Histogram of daily P&L");  
    ytics(0,10,2,2);      
    xtics(-0.4,0.4,0.10,2);      
    xlabel("\216P&L (in $ mn)");
    ylabel("\216Frequency (in %)");
    _pbartyp = {6 4};
    call histp(packr(PnL)/1e6,30);
    
  endwind;  

texFile = "chap5-4-4-1.tex";
call longTabular(100*R[2:25,.],2,3,8,texFile);
call longTabular(PnL[2:25,.],0,4,6,texFile);
call longTabular(PnLs[2:25,.],0,4,6,texFile);

// Results = reshape(R,

proc (0) = longTabular(data,prc,r,c,filename);
  local n,sa,i,x;
  
  n = rows(data);
  if r*c > n;
    data = data | miss(zeros(r*c-n,cols(data)),0);
  endif;
  
  i = 1;
  do until i > cols(data);
    x = reshape(data[.,i],r,c);
    sa = "$" $+ ftosa(x,"%lf",1,prc) $+ "$";     
    call saTabular(sa,fileName);
    i = i + 1;
  endo;
    
  retp;
endp;
  