/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 7.1.1
**
*/

new;
library frm,pgraph,optmum;
frmSet;

cls;

rndseed 123;

N_inf = 1000;
N0 = 0~1000~4000;
t = seqa(0,1/100,101);
lambda = 10;
sigma = 1000;

{mu_N,sigma_N} = NMD_Amount(t,N0,N_inf,lambda,sigma);
x1 = t;
y1 = mu_N;

x = seqa(-2000,10,1101);
t = 1/12~3/12~6/12;
{mu_N,sigma_N} = NMD_Amount(t,5000,N_inf,lambda,sigma);
pdf_N = pdfn2(x,mu_N,sigma_N);
x2 = x;
y2 = pdf_N;

t = seqa(0,1/12/3,12*3*10+1);
nS = 5;
N_t = simulate_OU(1000,lambda,N_inf,sigma,t,nS);
x3 = t;
y3 = N_t;

graphset;
  window3d;
  makewind(9,6.855/2,0,0,1);

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 1|6|3; _pcolor = 1|4|2; _plwidth = 10;
  _pnumht = 0.225; _paxht = 0.225; _ptitlht = 0.275;

  setwind(1);
    title("\216\202m\201](0,t)[ (in $)");
    xtics(0,1,0.2,2);
    ytics(0,4000,1000,2);
    xlabel("\216t");
    _plegstr = "N]0[ = 0\000N]0[ = 1000\000N]0[ = 4000";
    _plegctl = {2 5 5 4};
    xy(x1,y1);

  setwind(2);
    title("\216f\201](0,t)[(x) (in bps) - N]0[ = 5000");
    xtics(0,4000,1000,2);
    ytics(0,20,5,2);
    xlabel("\216N(t)");
    _plegstr = "1M\0003M\0006M";
    _plegctl = {2 5 7 4};
    xy(x2,1e4*y2);

  setwind(4);
    title("\216N(t)");
    xtics(0,10,2,2);
    ytics(400,1600,400,2);
    xlabel("\216t (in years)");
    _plegctl = 0;
    xy(x3,y3[.,1:3]);

  endwind; 


proc (2) = NMD_Amount(t,N0,N_inf,lambda,sigma);
  local mu_N,sigma_N;

  mu_N = N0 .* exp(-lambda.*t) + N_inf .* (1 - exp(-lambda .*t));
  sigma_N = sigma .* sqrt( (1 - exp(-2*lambda.*t)) ./ (2*lambda) );

  retp(mu_N,sigma_N);
endp;


