/*
** Thierry Roncalli
** Lectures Notes on Risk Management & Financial Regulation
**
**> Page ??, Section 7.1.1
**
*/

new;
library frm,pgraph,optmum;
frmSet;

cls;

rndseed 123456;

N_inf = 1000;
N0 = 1000;
lambda = 5;
lambda_plus = 0.00;
sigma = 200;

t = seqa(0,1/12/20,12*20*10+1);
nS = 1;
mu_t = simulate_OU(1000,lambda,N_inf,sigma,t,nS);
N_t = exp(lambda_plus .* t) .* mu_t;
x1 = t;
y1 = N_t;

N_stable = N_inf;
N_nonstable = N_t - N_stable;
x2 = t;
y2 = N_nonstable;

varphi = 0.9;
N_stable = varphi*N_inf;
N_nonstable = N_t - N_stable;
x3 = t;
y3 = N_nonstable;

varphi = 0.8;
N_stable = varphi*N_inf;
N_nonstable = N_t - N_stable;
x4 = t;
y4 = N_nonstable;

t = 0.1;
{mu_N,sigma_N} = NMD_Amount(t,N_inf,N_inf,lambda,sigma);
let alpha = 0.01 0.05 0.10;
N_stable = mu_N + cdfni(alpha) * sigma_N;
varphi = N_stable/N_inf;
results = 100*alpha~N_Stable~100*varphi; 
print ftosa(results,"%lf",3,2);

alpha = 1 - alpha;
varphi = 1 - cdfni(alpha) * sigma * sqrt(1./(2*lambda)) / N_inf;
results = 100*alpha~100*varphi; 
print ftosa(results,"%lf",3,2);

graphset;
  begwind;
  window(2,2,0);

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _pcolor = 3; _plwidth = 10;
  _pnumht = 0.225; _paxht = 0.225; _ptitlht = 0.275;

  setwind(1);
    title("\216Deposit balance");
    xtics(0,10,2,2);
    ytics(800,1300,100,2);
    xy(x1,y1);

  setwind(2);
    title("\216Short deposits");
    _pbartyp = {6 1};
    ytics(-200,300,100,2);
    bar(x2,y2);

  setwind(3);
    title("\216Non-stable deposits (\202f\201 = 90%)");
    _pbartyp = {6 4};
    ytics(-100,400,100,2);
    bar(x3,y3);

  setwind(4);
    title("\216Non-stable deposits (\202f\201 = 80%)");
    _pbartyp = {6 2};
    ytics(0,500,100,2);
    bar(x4,y4);

  endwind;

proc (2) = NMD_Amount(t,N0,N_inf,lambda,sigma);
  local mu_N,sigma_N;

  mu_N = N0 .* exp(-lambda.*t) + N_inf .* (1 - exp(-lambda .*t));
  sigma_N = sigma .* sqrt( (1 - exp(-2*lambda.*t))./(2*lambda) );

  retp(mu_N,sigma_N);
endp;

