clc;
clear;
init_global;
init_colours;
close all;

r = 0.05;
lambda = 0.10;
sigma = 0.02;
M = 1000000;
C = 0.01*M;
d = optimal_refinancing(r,lambda,sigma,M,C);
disp(ftosa(1e4*d,10,0));

M = [10000; 100000; 250000; 500000; 1000000];
C = 2000 + 0.01*M;
d = optimal_refinancing(r,lambda,sigma,M,C);
y1 = [M/1000 1e4*d];

M = 1000000;
C = 0.01*M;
r = [0.01; 0.02; 0.05; 0.08; 0.10];
d = optimal_refinancing(r,lambda,sigma,M,C);
y2 = [100*r 1e4*d];

r = 0.05;
sigma = [0.01; 0.02; 0.03; 0.05; 0.10];
d = optimal_refinancing(r,lambda,sigma,M,C);
y3 = [100*sigma 1e4*d];

sigma = 0.02;
lambda = [0.02; 0.05; 0.10; 0.15; 0.20];
d = optimal_refinancing(r,lambda,sigma,M,C);
y4 = [100*lambda 1e4*d];

disp(ftosa([y1 y2 y3 y4],10,0));

function d = optimal_refinancing(r,lambda,sigma,M,C)
psi = sqrt(2*(r + lambda))./sigma;
phi = 1 + psi .* (r + lambda) .* (C ./ M);
x = -exp(-phi);
W = zeros(rows(x),cols(x));
for i=1:rows(x)
    for j=1:cols(x)
        W(i,j) = Lambert_W(x(i,j));
    end
end
d = (phi + W)./psi;
end

function w = Lambert_W(x,branch)

% Lambert_W  Functional inverse of x = w*exp(w).
% w = Lambert_W(x), same as Lambert_W(x,0)
% w = Lambert_W(x,0)  Primary or upper branch, W_0(x)
% w = Lambert_W(x,-1)  Lower branch, W_{-1}(x)
%
% See: http://blogs.mathworks.com/cleve/2013/09/02/the-lambert-w-function/
% Copyright 2013 The MathWorks, Inc.
% Effective starting guess

if nargin < 2 || branch ~= -1
   w = ones(size(x));  % Start above -1
else  
   w = -2*ones(size(x));  % Start below -1
end
v = inf*w;

% Haley's method

while any(abs(w - v)./abs(w) > 1.e-8)
   v = w;
   e = exp(w);
   f = w.*e - x;  % Iterate to make this quantity zero
   w = w - f./((e.*(w+1) - (w+2).*f./(2*w+2)));
end

end