new;
library frm,pgraph,optmum;
frmset;

cls;

declare matrix _alm_alpha,_alm_beta,_alm_gamma,_alm_delta;
declare matrix _alm_r0,_alm_i0,_alm_D0,_alm_D_inf;
declare matrix _alm_bounds;

proc proc_margin(t,r0,i0,D0,D_inf,alpha,beta,gamma_,delta);
  local y;
  y = exp(-beta.*t) .* (r0  - i0) - alpha./beta .* (1 - exp(-beta.*t));
  retp(y);
endp;

proc proc_Dt(t,r0,i0,D0,D_inf,alpha,beta,gamma_,delta);
  local y;
  y = D_inf + exp(-gamma_.*t) .* (D0  - D_inf) - 
      delta .* (exp(-beta.*t) - exp(-gamma_.*t)) .* (r0 - i0) ./ (gamma_ - beta) + 
      alpha .* delta ./ beta .* ( (1-exp(-gamma_.*t))./gamma_ - (exp(-beta.*t) - exp(-gamma_.*t)) ./ (gamma_ - beta) );
  retp(y);
endp;

proc ___proc_V0(t,r0,i0,D0,D_inf,alpha,beta,gamma_,delta);
  local margin,Dt,y;
  margin = proc_margin(t,r0,i0,D0,D_inf,alpha,beta,gamma_,delta);
  Dt = proc_Dt(t,r0,i0,D0,D_inf,alpha,beta,gamma_,delta);
  y = exp(-r0.*t) .* margin .* Dt;
  retp(y);
endp;

proc __proc_V0(t);
  local y;
  y = ___proc_V0(t,_alm_r0,_alm_i0,_alm_D0,_alm_D_inf,_alm_alpha,_alm_beta,_alm_gamma,_alm_delta);
  retp(y);
endp;

proc _proc_V0(r0);
  local y;
  _alm_r0 = r0;
  y = quadLegendre1(&__proc_V0,_alm_bounds|0,128);
  retp(y);
endp;

proc compute_V0(r0,i0,D0,D_inf,alpha,beta,gamma_,delta);
  local nRows,nCols,e,V0,i,j;

  nRows = maxc(rows(alpha)|rows(beta)|rows(gamma_)|rows(delta)|rows(r0)|rows(i0)|rows(D0)|rows(D_inf));
  nCols = maxc(cols(alpha)|cols(beta)|cols(gamma_)|cols(delta)|cols(r0)|cols(i0)|cols(D0)|cols(D_inf));
  e = ones(nRows,nCols);

  alpha = alpha .* e;
  beta = beta .* e;
  gamma_ = gamma_ .* e;
  delta = delta .* e;
  r0 = r0 .* e;
  i0 = i0 .* e;
  D0 = D0 .* e;
  D_inf = D_inf .* e;

  V0 = zeros(nRows,nCols);
  i = 1;
  do until i > nRows;
    j = 1;
    do until j > nCols;
       _alm_i0 = i0[i,j];
       _alm_D0 = D0[i,j];
       _alm_D_inf = D_inf[i,j];
       _alm_alpha = alpha[i,j];
       _alm_beta = beta[i,j];
       _alm_gamma = gamma_[i,j];
       _alm_delta = delta[i,j];
       V0[i,j] = _proc_V0(r0[i,j]);
       j = j + 1;
     endo;
     i = i + 1;
   endo;

  retp(V0);
endp;

proc compute_S0(r0,i0,D0,D_inf,alpha,beta,gamma_,delta);
  local nRows,nCols,e,S0,i,j;

  nRows = maxc(rows(alpha)|rows(beta)|rows(gamma_)|rows(delta)|rows(r0)|rows(i0)|rows(D0)|rows(D_inf));
  nCols = maxc(cols(alpha)|cols(beta)|cols(gamma_)|cols(delta)|cols(r0)|cols(i0)|cols(D0)|cols(D_inf));
  e = ones(nRows,nCols);

  alpha = alpha .* e;
  beta = beta .* e;
  gamma_ = gamma_ .* e;
  delta = delta .* e;
  r0 = r0 .* e;
  i0 = i0 .* e;
  D0 = D0 .* e;
  D_inf = D_inf .* e;

  S0 = zeros(nRows,nCols);
  i = 1;
  do until i > nRows;
    j = 1;
    do until j > nCols;
       _alm_i0 = i0[i,j];
       _alm_D0 = D0[i,j];
       _alm_D_inf = D_inf[i,j];
       _alm_alpha = alpha[i,j];
       _alm_beta = beta[i,j];
       _alm_gamma = gamma_[i,j];
       _alm_delta = delta[i,j];
       S0[i,j] = gradp(&_proc_V0,r0[i,j]);
       j = j + 1;
     endo;
     i = i + 1;
   endo;

  retp(S0);
endp;

proc (3) = compute_Duration(r0,i0,D0,D_inf,alpha,beta,gamma_,delta);
  local nRows,nCols,e,V0,S0,Duration,i,j;

  nRows = maxc(rows(alpha)|rows(beta)|rows(gamma_)|rows(delta)|rows(r0)|rows(i0)|rows(D0)|rows(D_inf));
  nCols = maxc(cols(alpha)|cols(beta)|cols(gamma_)|cols(delta)|cols(r0)|cols(i0)|cols(D0)|cols(D_inf));
  e = ones(nRows,nCols);

  alpha = alpha .* e;
  beta = beta .* e;
  gamma_ = gamma_ .* e;
  delta = delta .* e;
  r0 = r0 .* e;
  i0 = i0 .* e;
  D0 = D0 .* e;
  D_inf = D_inf .* e;

  V0 = zeros(nRows,nCols);
  S0 = zeros(nRows,nCols);
  i = 1;
  do until i > nRows;
    j = 1;
    do until j > nCols;
       _alm_i0 = i0[i,j];
       _alm_D0 = D0[i,j];
       _alm_D_inf = D_inf[i,j];
       _alm_alpha = alpha[i,j];
       _alm_beta = beta[i,j];
       _alm_gamma = gamma_[i,j];
       _alm_delta = delta[i,j];
       V0[i,j] = _proc_V0(r0[i,j]);
       S0[i,j] = gradp(&_proc_V0,r0[i,j]);
       j = j + 1;
     endo;
     i = i + 1;
   endo;

  Duration = -S0 ./ miss(V0,0); 

  retp(V0,S0,Duration);
endp;

alpha = -0.005~-0.01~-0.02;
beta = 0.5;
gamma_ = 0.7;
delta = 0.5;
i0 = 0.05;
D0 = 100;
D_inf = 150;

_alm_bounds = 30;

r0 = seqa(0,0.20/100,101);
r0 = 0.001|0.005|seqa(0.01,0.01,20);

{V0,S0,Duration} = compute_Duration(r0,i0,D0,D_inf,alpha,beta,gamma_,delta);

graphset;
  window3u;

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3; _plwidth = 10; _pcolor = 1|2|4;
  _paxht = 0.225; _pnumht = 0.225; _ptitlht = 0.275;
  xlabel("\216r]0[ (in %)");
  xtics(0,20,5,2);
  
  setwind(1);
    title("\216Mark-to-market");
    ytics(0,150,25,2);
    _plegstr = "\202a\201 = -50 bps\000\202a\201 = -1%\000\202a\201 = -2\%";
    _plegctl = {2 5 9 3.5};
    xy(100*r0,V0);

  _plegctl = 0;
 
  setwind(2);
    title("\216Sensitivity");
    ytics(-600,150,150,2);
    xy(100*r0,S0);

  setwind(3);
    title("\216Duration");
    ytics(-5,20,5,2);
    xy(100*r0,Duration);

  endwind; 

